@@ -2,7 +2,7 @@
 # make sure configure was run with coverage enabled...
 lcov --directory . --zerocounters
 make check
-rm `find * -name "test_*.gc??"`
+rm `find * -name "test_*.gc??"` `find * -name "perf_*.gc??"`
 for n in `find * -name "*.gc??" | grep libs`
 do
   cd `dirname $n`
