@@ -159,6 +159,8 @@ run (void *cls, struct GNUNET_SERVER_Handle *server,
      sizeof (struct ReservationRequestMessage)},
     {&GAS_handle_preference_change, NULL,
      GNUNET_MESSAGE_TYPE_ATS_PREFERENCE_CHANGE, 0},
+    {&GAS_handle_preference_feedback, NULL,
+     GNUNET_MESSAGE_TYPE_ATS_PREFERENCE_FEEDBACK, 0},
     {&GAS_handle_reset_backoff, NULL,
      GNUNET_MESSAGE_TYPE_ATS_RESET_BACKOFF,
      sizeof (struct ResetBackoffMessage)},
