@@ -29,6 +29,10 @@
 
 #include "gnunet_statistics_service.h"
 
+#define GAS_normalization_queue_length 3
+
+#define BANDWIDTH_ZERO GNUNET_BANDWIDTH_value_init (0)
+
 /**
  * Handle for statistics.
  */
