@@ -52,6 +52,21 @@ void
 GAS_performance_remove_client (struct GNUNET_SERVER_Client *client);
 
 
+void
+GAS_handle_performance_update (struct GNUNET_PeerIdentity *peer,
+															 const char *plugin_name,
+															 const void *plugin_addr,
+															 size_t plugin_addr_len,
+															 const int active,
+															 struct GNUNET_ATS_Information *ats,
+															 uint32_t ats_count,
+															 struct GNUNET_BANDWIDTH_Value32NBO
+															 bandwidth_out,
+															 struct GNUNET_BANDWIDTH_Value32NBO
+															 bandwidth_in);
+
+
+
 /**
  * Transmit the given performance information to all performance
  * clients.
@@ -117,6 +132,24 @@ GAS_handle_preference_change (void *cls,
                               const struct GNUNET_MessageHeader *message);
 
 
+/**
+ * Handle 'preference feedback' messages from clients.
+ *
+ * @param cls unused, NULL
+ * @param client client that sent the request
+ * @param message the request message
+ */
+void
+GAS_handle_preference_feedback (void *cls,
+                              struct GNUNET_SERVER_Client *client,
+                              const struct GNUNET_MessageHeader *message);
+
+
+void
+GAS_handle_monitor (void *cls,
+													struct GNUNET_SERVER_Client *client,
+                          const struct GNUNET_MessageHeader *message);
+
 /**
  * Initialize performance subsystem.
  *
