@@ -110,13 +110,34 @@ struct PerfHandle
    * */
   struct GNUNET_ATS_PluginEnvironment env;
 
-  struct Result *head;
+  /**
+   * Array for results for each iteration with length iterations
+   */
+  struct Iteration *iterations_results;
 
-  struct Result *tail;
+  /**
+   * Array to store averaged full solution result with length #peers
+   */
+  struct Result *averaged_full_result;
 
+  /**
+   * Array to store averaged updated solution result with length #peers
+   */
+  struct Result *averaged_update_result;
+
+  /**
+   * The current result
+   */
   struct Result *current_result;
 
+  /**
+   * Current number of peers benchmarked
+   */
   int current_p;
+
+  /**
+   * Current number of addresses benchmarked
+   */
   int current_a;
 
   /**
@@ -159,6 +180,16 @@ struct PerfHandle
    */
   int measure_updates;
 
+  /**
+   * Number of iterations
+   */
+  int total_iterations;
+
+  /**
+   * Current iteration
+   */
+  int current_iteration;
+
   /**
    * Is a bulk operation running?
    */
@@ -168,43 +199,162 @@ struct PerfHandle
    * Is a bulk operation running?
    */
   int expecting_solution;
+
+  /**
+   * Was the problem just updates?
+   */
+  int performed_update;
 };
 
+/**
+ * Data structure to store results for a single iteration
+ */
+struct Iteration
+{
+  /**
+   * Head of the linked list
+   */
+  struct Result *result_head;
+
+  /**
+   * Tail of the linked list
+   */
+  struct Result *result_tail;
+};
+
+
+/**
+ * Result for a solver calculcation
+ */
 struct Result
 {
+  /**
+   * Previous element in the linked list
+   */
   struct Result *prev;
+
+  /**
+   * Next element in the linked list
+   */
   struct Result *next;
 
+  /**
+   * Number of peers this solution included
+   */
   int peers;
+
+  /**
+   * Number of addresses per peer this solution included
+   */
   int addresses;
+
+  /**
+   * Is this an update or a full solution
+   */
   int update;
 
+  /**
+   * Was the solution valid or did the solver fail
+   */
+  int valid;
+
+  /**
+   * Result of the solver
+   */
   enum GAS_Solver_Additional_Information info;
 
+  /**
+   * Duration of setting up the problem in the solver
+   */
   struct GNUNET_TIME_Relative d_setup;
+
+  /**
+   * Duration of solving the LP problem in the solver
+   * MLP solver only
+   */
   struct GNUNET_TIME_Relative d_lp;
+
+  /**
+   * Duration of solving the MLP problem in the solver
+   * MLP solver only
+   */
   struct GNUNET_TIME_Relative d_mlp;
+
+  /**
+   * Duration of solving whole problem in the solver
+   */
   struct GNUNET_TIME_Relative d_total;
 
+  /**
+   * Start time of setting up the problem in the solver
+   */
   struct GNUNET_TIME_Absolute s_setup;
+
+  /**
+   * Start time of solving the LP problem in the solver
+   * MLP solver only
+   */
   struct GNUNET_TIME_Absolute s_lp;
+
+  /**
+   * Start time of solving the MLP problem in the solver
+   * MLP solver only
+   */
   struct GNUNET_TIME_Absolute s_mlp;
+
+  /**
+   * Start time of solving whole problem in the solver
+   */
   struct GNUNET_TIME_Absolute s_total;
 
+  /**
+   * End time of setting up the problem in the solver
+   */
   struct GNUNET_TIME_Absolute e_setup;
+
+  /**
+   * End time of solving the LP problem in the solver
+   * MLP solver only
+   */
   struct GNUNET_TIME_Absolute e_lp;
+
+  /**
+   * End time of solving the MLP problem in the solver
+   * MLP solver only
+   */
   struct GNUNET_TIME_Absolute e_mlp;
+
+  /**
+   * End time of solving whole problem in the solver
+   */
   struct GNUNET_TIME_Absolute e_total;
 };
 
+/**
+ * Peer used for the benchmarking
+ */
 struct PerfPeer
 {
+  /**
+   * Peer identitity
+   */
   struct GNUNET_PeerIdentity id;
 
+  /**
+   * Head of linked list of addresses used with this peer
+   */
   struct ATS_Address *head;
+
+  /**
+   * Head of linked list of addresses used with this peer
+   */
   struct ATS_Address *tail;
 };
 
+
+/**
+ * ATS performance handle
+ */
 static struct PerfHandle ph;
 
 /**
@@ -214,11 +364,8 @@ static int ret;
 
 
 /**
- * ATS information
+ * Do shutdown
  */
-//static struct GNUNET_ATS_Information ats[2];
-
-
 static void
 end_now (int res)
 {
@@ -227,23 +374,22 @@ end_now (int res)
     GNUNET_STATISTICS_destroy (ph.stat, GNUNET_NO);
     ph.stat = NULL;
   }
-  /*
-   if (NULL != addresses)
-   {
-   GNUNET_CONTAINER_multihashmap_iterate (addresses, &addr_it, NULL);
-   GNUNET_CONTAINER_multihashmap_destroy (addresses);
-   addresses = NULL ;
-   }*/
-  if (NULL != ph.peers)
-  {
-    GNUNET_free(ph.peers);
-  }
+
+  GNUNET_free_non_null (ph.peers);
+  GNUNET_free_non_null (ph.iterations_results);
+  GNUNET_free_non_null (ph.averaged_full_result);
+  GNUNET_free_non_null (ph.averaged_update_result);
 
   GAS_normalization_stop ();
   ret = res;
 }
 
 
+/**
+ * Create a peer used for benchmarking
+ *
+ * @param cp the number of the peer
+ */
 static void
 perf_create_peer (int cp)
 {
@@ -255,7 +401,11 @@ perf_create_peer (int cp)
 }
 
 
-
+/**
+ * Perform an update for an address
+ *
+ * @param cur the address to update
+ */
 static void
 perf_update_address (struct ATS_Address *cur)
 {
@@ -347,6 +497,14 @@ perf_address_initial_update (void *solver,
           / 100);
 }
 
+/**
+ * Update a certain percentage of peers
+ *
+ * @param cp the current number of peers
+ * @param ca the current number of addresses
+ * @param percentage_peers the percentage of peers to update
+ */
+
 static void
 perf_update_all_addresses (unsigned int cp, unsigned int ca, unsigned int percentage_peers)
 {
@@ -392,14 +550,19 @@ perf_update_all_addresses (unsigned int cp, unsigned int ca, unsigned int percen
       {
         if (c_cur_a == r)
           perf_update_address (cur_address);
-
         c_cur_a ++;
       }
     }
   }
 }
 
-
+/**
+ * Create an address for a peer
+ *
+ * @param cp index of the peer
+ * @param ca index of the address
+ * @return the address
+ */
 static struct ATS_Address *
 perf_create_address (int cp, int ca)
 {
@@ -412,6 +575,14 @@ perf_create_address (int cp, int ca)
   return a;
 }
 
+
+/**
+ * Information callback for the solver
+ *
+ * @param op the solver operation
+ * @param stat status of the solver operation
+ * @param add additional solver information
+ */
 static void
 solver_info_cb (void *cls,
     enum GAS_Solver_Operation op,
@@ -444,7 +615,7 @@ solver_info_cb (void *cls,
   switch (op)
   {
     case GAS_OP_SOLVE_START:
-      GNUNET_log(GNUNET_ERROR_TYPE_DEBUG,
+      GNUNET_log(GNUNET_ERROR_TYPE_INFO,
           "Solver notifies `%s' with result `%s' `%s'\n", "GAS_OP_SOLVE_START",
           (GAS_STAT_SUCCESS == stat) ? "SUCCESS" : "FAIL", add_info);
       if (GNUNET_NO == ph.expecting_solution)
@@ -459,7 +630,8 @@ solver_info_cb (void *cls,
         /* Create new result */
         tmp = GNUNET_new (struct Result);
         ph.current_result = tmp;
-        GNUNET_CONTAINER_DLL_insert_tail(ph.head, ph.tail, tmp);
+        GNUNET_CONTAINER_DLL_insert_tail(ph.iterations_results[ph.current_iteration-1].result_head,
+            ph.iterations_results[ph.current_iteration-1].result_tail, tmp);
         ph.current_result->addresses = ph.current_a;
         ph.current_result->peers = ph.current_p;
         ph.current_result->s_total = GNUNET_TIME_absolute_get();
@@ -468,14 +640,19 @@ solver_info_cb (void *cls,
         ph.current_result->d_lp = GNUNET_TIME_UNIT_FOREVER_REL;
         ph.current_result->d_mlp = GNUNET_TIME_UNIT_FOREVER_REL;
         ph.current_result->info = add;
-        if (add == GAS_INFO_UPDATED)
+        if ((add == GAS_INFO_UPDATED) || (GNUNET_YES == ph.performed_update))
+        {
           ph.current_result->update = GNUNET_YES;
+        }
         else
+        {
           ph.current_result->update = GNUNET_NO;
+        }
+
       }
       return;
     case GAS_OP_SOLVE_STOP:
-      GNUNET_log(GNUNET_ERROR_TYPE_DEBUG,
+      GNUNET_log(GNUNET_ERROR_TYPE_INFO,
           "Solver notifies `%s' with result `%s'\n", "GAS_OP_SOLVE_STOP",
           (GAS_STAT_SUCCESS == stat) ? "SUCCESS" : "FAIL", add_info);
       if ((GNUNET_NO == ph.expecting_solution) || (NULL == ph.current_result))
@@ -484,6 +661,12 @@ solver_info_cb (void *cls,
         GNUNET_break (0);
         return;
       }
+
+      if (GAS_STAT_SUCCESS == stat)
+        ph.current_result->valid = GNUNET_YES;
+      else
+        ph.current_result->valid = GNUNET_NO;
+
       if (NULL != ph.current_result)
       {
         /* Finalize result */
@@ -495,7 +678,7 @@ solver_info_cb (void *cls,
       return;
 
     case GAS_OP_SOLVE_SETUP_START:
-      GNUNET_log(GNUNET_ERROR_TYPE_DEBUG,
+      GNUNET_log(GNUNET_ERROR_TYPE_INFO,
           "Solver notifies `%s' with result `%s'\n", "GAS_OP_SOLVE_SETUP_START",
           (GAS_STAT_SUCCESS == stat) ? "SUCCESS" : "FAIL");
       if ((GNUNET_NO == ph.expecting_solution) || (NULL == ph.current_result))
@@ -503,11 +686,17 @@ solver_info_cb (void *cls,
         GNUNET_break(0);
         return;
       }
+
+      if (GAS_STAT_SUCCESS == stat)
+        ph.current_result->valid = GNUNET_YES;
+      else
+        ph.current_result->valid = GNUNET_NO;
+
       ph.current_result->s_setup = GNUNET_TIME_absolute_get ();
       return;
 
     case GAS_OP_SOLVE_SETUP_STOP:
-      GNUNET_log(GNUNET_ERROR_TYPE_DEBUG,
+      GNUNET_log(GNUNET_ERROR_TYPE_INFO,
           "Solver notifies `%s' with result `%s'\n", "GAS_OP_SOLVE_SETUP_STOP",
           (GAS_STAT_SUCCESS == stat) ? "SUCCESS" : "FAIL");
       if ((GNUNET_NO == ph.expecting_solution) || (NULL == ph.current_result))
@@ -515,13 +704,19 @@ solver_info_cb (void *cls,
         GNUNET_break(0);
         return;
       }
+
+      if (GAS_STAT_SUCCESS == stat)
+        ph.current_result->valid = GNUNET_YES;
+      else
+        ph.current_result->valid = GNUNET_NO;
+
       ph.current_result->e_setup = GNUNET_TIME_absolute_get ();
       ph.current_result->d_setup = GNUNET_TIME_absolute_get_difference (
           ph.current_result->s_setup, ph.current_result->e_setup);
       return;
 
     case GAS_OP_SOLVE_MLP_LP_START:
-      GNUNET_log(GNUNET_ERROR_TYPE_DEBUG,
+      GNUNET_log(GNUNET_ERROR_TYPE_INFO,
           "Solver notifies `%s' with result `%s'\n", "GAS_OP_SOLVE_LP_START",
           (GAS_STAT_SUCCESS == stat) ? "SUCCESS" : "FAIL");
       if ((GNUNET_NO == ph.expecting_solution) || (NULL == ph.current_result))
@@ -529,10 +724,16 @@ solver_info_cb (void *cls,
         GNUNET_break(0);
         return;
       }
+
+      if (GAS_STAT_SUCCESS == stat)
+        ph.current_result->valid = GNUNET_YES;
+      else
+        ph.current_result->valid = GNUNET_NO;
+
       ph.current_result->s_lp = GNUNET_TIME_absolute_get ();
       return;
     case GAS_OP_SOLVE_MLP_LP_STOP:
-      GNUNET_log(GNUNET_ERROR_TYPE_DEBUG,
+      GNUNET_log(GNUNET_ERROR_TYPE_INFO,
           "Solver notifies `%s' with result `%s'\n", "GAS_OP_SOLVE_LP_STOP",
           (GAS_STAT_SUCCESS == stat) ? "SUCCESS" : "FAIL");
       if ((GNUNET_NO == ph.expecting_solution) || (NULL == ph.current_result))
@@ -540,13 +741,19 @@ solver_info_cb (void *cls,
         GNUNET_break(0);
         return;
       }
+
+      if (GAS_STAT_SUCCESS == stat)
+        ph.current_result->valid = GNUNET_YES;
+      else
+        ph.current_result->valid = GNUNET_NO;
+
       ph.current_result->e_lp = GNUNET_TIME_absolute_get ();
       ph.current_result->d_lp = GNUNET_TIME_absolute_get_difference (
           ph.current_result->s_lp, ph.current_result->e_lp);
       return;
 
     case GAS_OP_SOLVE_MLP_MLP_START:
-      GNUNET_log(GNUNET_ERROR_TYPE_DEBUG,
+      GNUNET_log(GNUNET_ERROR_TYPE_INFO,
           "Solver notifies `%s' with result `%s'\n", "GAS_OP_SOLVE_MLP_START",
           (GAS_STAT_SUCCESS == stat) ? "SUCCESS" : "FAIL");
       if ((GNUNET_NO == ph.expecting_solution) || (NULL == ph.current_result))
@@ -554,10 +761,16 @@ solver_info_cb (void *cls,
         GNUNET_break(0);
         return;
       }
+
+      if (GAS_STAT_SUCCESS == stat)
+        ph.current_result->valid = GNUNET_YES;
+      else
+        ph.current_result->valid = GNUNET_NO;
+
       ph.current_result->s_mlp = GNUNET_TIME_absolute_get ();
       return;
     case GAS_OP_SOLVE_MLP_MLP_STOP:
-      GNUNET_log(GNUNET_ERROR_TYPE_DEBUG,
+      GNUNET_log(GNUNET_ERROR_TYPE_INFO,
           "Solver notifies `%s' with result `%s'\n", "GAS_OP_SOLVE_MLP_STOP",
           (GAS_STAT_SUCCESS == stat) ? "SUCCESS" : "FAIL");
       if ((GNUNET_NO == ph.expecting_solution) || (NULL == ph.current_result))
@@ -565,19 +778,32 @@ solver_info_cb (void *cls,
         GNUNET_break(0);
         return;
       }
+
+      if (GAS_STAT_SUCCESS == stat)
+        ph.current_result->valid = GNUNET_YES;
+      else
+        ph.current_result->valid = GNUNET_NO;
+
       ph.current_result->e_mlp = GNUNET_TIME_absolute_get ();
       ph.current_result->d_mlp = GNUNET_TIME_absolute_get_difference (
       ph.current_result->s_mlp, ph.current_result->e_mlp);
       return;
     case GAS_OP_SOLVE_UPDATE_NOTIFICATION_START:
-      GNUNET_log(GNUNET_ERROR_TYPE_DEBUG,
+      GNUNET_log(GNUNET_ERROR_TYPE_INFO,
           "Solver notifies `%s' with result `%s'\n", "GAS_OP_SOLVE_UPDATE_NOTIFICATION_START",
           (GAS_STAT_SUCCESS == stat) ? "SUCCESS" : "FAIL");
       return;
     case GAS_OP_SOLVE_UPDATE_NOTIFICATION_STOP:
-      GNUNET_log(GNUNET_ERROR_TYPE_DEBUG,
+      GNUNET_log(GNUNET_ERROR_TYPE_INFO,
           "Solver notifies `%s' with result `%s'\n", "GAS_OP_SOLVE_UPDATE_NOTIFICATION_STOP",
           (GAS_STAT_SUCCESS == stat) ? "SUCCESS" : "FAIL");
+      if (GAS_STAT_SUCCESS != stat)
+      {
+        GNUNET_log(GNUNET_ERROR_TYPE_ERROR,
+            "Solver `%s' failed to update problem with %u peers and %u address!\n",
+            ph.ats_string, ph.current_p, ph.current_a);
+      }
+
       return;
     default:
       break;
@@ -585,11 +811,12 @@ solver_info_cb (void *cls,
 }
 
 static void
-write_gnuplot_script (char * data_fn, int full)
+write_gnuplot_script (char * data_fn, int iteration, int full)
 {
   struct GNUNET_DISK_FileHandle *f;
   char * gfn;
   char *data;
+  char *iter_text;
   char *template;
 
   /* Write header */
@@ -616,11 +843,29 @@ write_gnuplot_script (char * data_fn, int full)
       GNUNET_break (0);
       return;
   }
-
-  if (GNUNET_YES == full)
-    GNUNET_asprintf (&gfn, "perf_%s_full_%u_%u_%u.gnuplot", ph.ats_string, ph.N_peers_start, ph.N_peers_end, ph.N_address);
+  if (-1 == iteration)
+    GNUNET_asprintf (&iter_text, "%s_%u", "avg",ph.total_iterations);
   else
-    GNUNET_asprintf (&gfn, "perf_%s_update_%u_%u_%u.gnuplot", ph.ats_string, ph.N_peers_start, ph.N_peers_end, ph.N_address);
+    GNUNET_asprintf (&iter_text, "%u", iteration);
+  if (GNUNET_YES == full)
+  {
+    GNUNET_asprintf (&gfn, "perf_%s_full_%s-%u_%u_%u.gnuplot",
+        ph.ats_string,
+        iter_text,
+        ph.N_peers_start,
+        ph.N_peers_end,
+        ph.N_address);
+  }
+  else
+  {
+    GNUNET_asprintf (&gfn, "perf_%s_updat_%s-%u_%u_%u.gnuplot",
+        ph.ats_string,
+        iter_text,
+        ph.N_peers_start,
+        ph.N_peers_end,
+        ph.N_address);
+  }
+  GNUNET_free (iter_text);
 
   f = GNUNET_DISK_file_open (gfn,
       GNUNET_DISK_OPEN_WRITE | GNUNET_DISK_OPEN_CREATE,
@@ -681,9 +926,13 @@ write_gnuplot_script (char * data_fn, int full)
 
 }
 
-
+/**
+ * Evaluate results for a specific iteration
+ *
+ * @param iteration the iteration to evaluate
+ */
 static void
-evaluate ()
+evaluate (int iteration)
 {
   struct GNUNET_DISK_FileHandle *f_full;
   struct GNUNET_DISK_FileHandle *f_update;
@@ -692,10 +941,12 @@ evaluate ()
   char * data;
   struct Result *cur;
   struct Result *next;
+  struct Result *cur_res;
   char * str_d_total;
   char * str_d_setup;
   char * str_d_lp;
   char * str_d_mlp;
+  char * iter_text;
 
   f_full = NULL;
   f_update = NULL;
@@ -704,12 +955,18 @@ evaluate ()
 
   if (ph.create_plot)
   {
+    if (-1 == iteration)
+      GNUNET_asprintf (&iter_text, "%s", "avg");
+    else
+      GNUNET_asprintf (&iter_text, "%u", iteration);
     GNUNET_asprintf (&data_fn_full,
-                     "perf_%s_full_%u_%u_%u.data",
+                     "perf_%s_full_%s_%u_%u_%u.data",
                      ph.ats_string,
+                     iter_text,
                      ph.N_peers_start,
                      ph.N_peers_end,
                      ph.N_address);
+    GNUNET_free (iter_text);
     f_full = GNUNET_DISK_file_open (data_fn_full,
         GNUNET_DISK_OPEN_WRITE | GNUNET_DISK_OPEN_CREATE,
         GNUNET_DISK_PERM_USER_EXEC | GNUNET_DISK_PERM_USER_READ | GNUNET_DISK_PERM_USER_WRITE);
@@ -726,13 +983,23 @@ evaluate ()
       GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                   "Cannot write data to log file `%s'\n",
                   data_fn_full);
-    write_gnuplot_script (data_fn_full, GNUNET_YES);
+    write_gnuplot_script (data_fn_full, iteration, GNUNET_YES);
   }
 
   data_fn_update = NULL;
   if ((ph.create_plot) && (GNUNET_YES == ph.measure_updates))
   {
-    GNUNET_asprintf (&data_fn_update, "perf_%s_update_%u_%u_%u.data", ph.ats_string, ph.N_peers_start, ph.N_peers_end, ph.N_address);
+    if (-1 == iteration)
+      GNUNET_asprintf (&iter_text, "%s", "avg");
+    else
+      GNUNET_asprintf (&iter_text, "%u", iteration);
+    GNUNET_asprintf (&data_fn_update, "perf_%s_update_i%u_%u_%u_%u.data",
+        ph.ats_string,
+        iter_text,
+        ph.N_peers_start,
+        ph.N_peers_end,
+        ph.N_address);
+    GNUNET_free (iter_text);
     f_update = GNUNET_DISK_file_open (data_fn_update,
         GNUNET_DISK_OPEN_WRITE | GNUNET_DISK_OPEN_CREATE,
         GNUNET_DISK_PERM_USER_EXEC | GNUNET_DISK_PERM_USER_READ | GNUNET_DISK_PERM_USER_WRITE);
@@ -751,10 +1018,10 @@ evaluate ()
       GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                   "Cannot write data to log file `%s'\n",
                   data_fn_update);
-    write_gnuplot_script (data_fn_update, GNUNET_NO);
+    write_gnuplot_script (data_fn_update, iteration, GNUNET_NO);
   }
 
-  next = ph.head;
+  next = ph.iterations_results[ph.current_iteration -1].result_head;
   while (NULL != (cur = next))
   {
     next = cur->next;
@@ -764,49 +1031,93 @@ evaluate ()
     str_d_mlp = NULL;
 
     /* Print log */
-    if (GNUNET_TIME_UNIT_FOREVER_REL.rel_value_us != cur->d_total.rel_value_us)
+    if (GNUNET_NO == cur->update)
+    {
+      cur_res = &ph.averaged_full_result[cur->peers - ph.N_peers_start];
+    }
+    else
+    {
+      cur_res = &ph.averaged_update_result[cur->peers - ph.N_peers_start];
+    }
+
+    cur_res->peers = cur->peers;
+    cur_res->addresses = cur->addresses;
+    cur_res->update = cur->update;
+
+    if (GNUNET_NO == cur->valid)
     {
       fprintf (stderr,
-               "Total time to solve %s for %u peers %u addresses: %llu us\n",
+               "Total time to solve %s for %u peers %u addresses: %s\n",
                (GNUNET_YES == cur->update) ? "updated" : "full",
-               cur->peers, cur->addresses, (unsigned long long) cur->d_total.rel_value_us);
+               cur->peers, cur->addresses, "Failed to solve!");
+      continue;
+    }
+    else
+      cur_res->valid ++;
+
+    if (GNUNET_TIME_UNIT_FOREVER_REL.rel_value_us != cur->d_total.rel_value_us)
+    {
+      if (GNUNET_TIME_UNIT_FOREVER_REL.rel_value_us == cur_res->d_total.rel_value_us)
+        cur_res->d_total.rel_value_us = 0;
+      if (GNUNET_YES == cur->valid)
+        cur_res->d_total.rel_value_us += cur->d_total.rel_value_us;
+      fprintf (stderr,
+         "Total time to solve %s for %u peers %u addresses: %llu us\n",
+         (GNUNET_YES == cur->update) ? "updated" : "full",
+         cur->peers, cur->addresses,
+         (unsigned long long) cur->d_total.rel_value_us);
       GNUNET_asprintf(&str_d_total,
-                      "%llu",
-                      (unsigned long long) cur->d_total.rel_value_us);
+         "%llu", (unsigned long long) cur->d_total.rel_value_us);
     }
     else
       GNUNET_asprintf(&str_d_total, "-1");
     if (GNUNET_TIME_UNIT_FOREVER_REL.rel_value_us != cur->d_setup.rel_value_us)
     {
+      if (GNUNET_TIME_UNIT_FOREVER_REL.rel_value_us == cur_res->d_setup.rel_value_us)
+        cur_res->d_setup.rel_value_us = 0;
+      if (GNUNET_YES == cur->valid)
+        cur_res->d_setup.rel_value_us += cur->d_setup.rel_value_us;
       fprintf (stderr, "Total time to setup %s %u peers %u addresses: %llu us\n",
           (GNUNET_YES == cur->update) ? "updated" : "full",
-          cur->peers, cur->addresses, (unsigned long long )cur->d_setup.rel_value_us);
-      GNUNET_asprintf(&str_d_setup, "%llu", (unsigned long long )cur->d_setup.rel_value_us);
+          cur->peers, cur->addresses,
+          (unsigned long long) cur->d_setup.rel_value_us);
+      GNUNET_asprintf(&str_d_setup, "%llu",
+          (unsigned long long )cur->d_setup.rel_value_us);
     }
     else
       GNUNET_asprintf(&str_d_setup, "-1");
+
     if (GNUNET_TIME_UNIT_FOREVER_REL.rel_value_us != cur->d_lp.rel_value_us)
     {
+      if (GNUNET_TIME_UNIT_FOREVER_REL.rel_value_us == cur_res->d_lp.rel_value_us)
+        cur_res->d_lp.rel_value_us = 0;
+      if (GNUNET_YES == cur->valid)
+        cur_res->d_lp.rel_value_us += cur->d_lp.rel_value_us;
       fprintf (stderr,
-               "Total time to solve %s LP for %u peers %u addresses: %llu us\n",
-               (GNUNET_YES == cur->update) ? "updated" : "full",
-               cur->peers,
-               cur->addresses,
-               (unsigned long long )cur->d_lp.rel_value_us);
+         "Total time to solve %s LP for %u peers %u addresses: %llu us\n",
+         (GNUNET_YES == cur->update) ? "updated" : "full",
+         cur->peers,
+         cur->addresses,
+         (unsigned long long )cur->d_lp.rel_value_us);
       GNUNET_asprintf (&str_d_lp,
-                       "%llu",
-                       (unsigned long long )cur->d_lp.rel_value_us);
+          "%llu", (unsigned long long )cur->d_lp.rel_value_us);
     }
     else
       GNUNET_asprintf (&str_d_lp, "-1");
+
     if (GNUNET_TIME_UNIT_FOREVER_REL.rel_value_us != cur->d_mlp.rel_value_us)
     {
+      if (GNUNET_TIME_UNIT_FOREVER_REL.rel_value_us == cur_res->d_mlp.rel_value_us)
+        cur_res->d_mlp.rel_value_us = 0;
+      if (GNUNET_YES == cur->valid)
+        cur_res->d_mlp.rel_value_us += cur->d_mlp.rel_value_us;
+
       fprintf (stderr, "Total time to solve %s MLP for %u peers %u addresses: %llu us\n",
           (GNUNET_YES == cur->update) ? "updated" : "full",
-          cur->peers, cur->addresses, (unsigned long long )cur->d_mlp.rel_value_us);
+          cur->peers, cur->addresses,
+          (unsigned long long )cur->d_mlp.rel_value_us);
       GNUNET_asprintf (&str_d_mlp,
-                       "%llu",
-                       (unsigned long long )cur->d_mlp.rel_value_us);
+          "%llu", (unsigned long long )cur->d_mlp.rel_value_us);
     }
     else
       GNUNET_asprintf (&str_d_mlp, "-1");
@@ -843,7 +1154,8 @@ evaluate ()
     GNUNET_free_non_null (str_d_lp);
     GNUNET_free_non_null (str_d_mlp);
 
-    GNUNET_CONTAINER_DLL_remove (ph.head, ph.tail, cur);
+    GNUNET_CONTAINER_DLL_remove (ph.iterations_results[ph.current_iteration-1].result_head,
+        ph.iterations_results[ph.current_iteration-1].result_tail, cur);
     GNUNET_free (cur);
   }
 
@@ -858,9 +1170,236 @@ evaluate ()
   GNUNET_free_non_null (data_fn_update);
 }
 
+/**
+ * Evaluate average results for all iterations
+ */
+static void
+evaluate_average (void)
+{
+  int c_o;
+  int c_i;
+
+  struct GNUNET_DISK_FileHandle *f_full;
+  struct GNUNET_DISK_FileHandle *f_update;
+  struct Result *cur;
+  char * data_fn_full;
+  char * data_fn_update;
+  char * data;
+  char * str_d_total;
+  char * str_d_setup;
+  char * str_d_lp;
+  char * str_d_mlp;
+
+  f_full = NULL;
+  f_update = NULL;
+
+  data_fn_full = NULL;
+
+  if (ph.create_plot)
+  {
+    GNUNET_asprintf (&data_fn_full,
+                     "perf_%s_full_avg_%u-%u_%u_%u.data",
+                     ph.ats_string,
+                     ph.total_iterations,
+                     ph.N_peers_start,
+                     ph.N_peers_end,
+                     ph.N_address);
+    f_full = GNUNET_DISK_file_open (data_fn_full,
+        GNUNET_DISK_OPEN_WRITE | GNUNET_DISK_OPEN_CREATE,
+        GNUNET_DISK_PERM_USER_EXEC | GNUNET_DISK_PERM_USER_READ | GNUNET_DISK_PERM_USER_WRITE);
+    if (NULL == f_full)
+    {
+      GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+                  "Cannot open gnuplot file `%s'\n",
+                  data_fn_full);
+      GNUNET_free (data_fn_full);
+      return;
+    }
+    data = "#peers;addresses;time total in us;#time setup in us;#time lp in us;#time mlp in us;\n";
+    if (GNUNET_SYSERR == GNUNET_DISK_file_write(f_full, data, strlen(data)))
+      GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+                  "Cannot write data to log file `%s'\n",
+                  data_fn_full);
+    write_gnuplot_script (data_fn_full, -1, GNUNET_YES);
+  }
+
+  data_fn_update = NULL;
+  if ((ph.create_plot) && (GNUNET_YES == ph.measure_updates))
+  {
+    GNUNET_asprintf (&data_fn_update, "perf_%s_update_avg_%u-%u_%u_%u.data",
+        ph.ats_string,
+        ph.total_iterations,
+        ph.N_peers_start,
+        ph.N_peers_end,
+        ph.N_address);
+    f_update = GNUNET_DISK_file_open (data_fn_update,
+        GNUNET_DISK_OPEN_WRITE | GNUNET_DISK_OPEN_CREATE,
+        GNUNET_DISK_PERM_USER_EXEC | GNUNET_DISK_PERM_USER_READ | GNUNET_DISK_PERM_USER_WRITE);
+    if (NULL == f_update)
+    {
+      GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+                  "Cannot open gnuplot file `%s'\n", data_fn_update);
+      GNUNET_free (data_fn_update);
+      if (NULL != f_full)
+        GNUNET_DISK_file_close (f_full);
+      GNUNET_free (data_fn_full);
+      return;
+    }
+    data = "#peers;addresses;time total in us;#time setup in us;#time lp in us;#time mlp in us;\n";
+    if (GNUNET_SYSERR == GNUNET_DISK_file_write (f_update, data, strlen(data)))
+      GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+                  "Cannot write data to log file `%s'\n",
+                  data_fn_update);
+    write_gnuplot_script (data_fn_update, -1, GNUNET_NO);
+  }
+
+  for (c_o = 0; c_o < 2; c_o++)
+  {
+    if (0 == c_o)
+      fprintf (stderr,
+          "Duration for a full solution averaged over %i iterations\n",
+          ph.total_iterations);
+    if (1 == c_o)
+      fprintf (stderr,
+          "Duration for a full solution averaged over %i iterations\n",
+          ph.total_iterations);
+
+    for (c_i = 0; c_i <= ph.N_peers_end - ph.N_peers_start; c_i++)
+    {
+      if (0 == c_o)
+      {
+        cur = &ph.averaged_full_result[c_i];
+      }
+      else if ((GNUNET_YES == ph.measure_updates) && (1 == c_o))
+      {
+        cur = &ph.averaged_update_result[c_i];
+      }
+      else
+         break;
+
+      if (0 == cur->peers)
+        continue;
+
+      str_d_total = NULL;
+      str_d_setup = NULL;
+      str_d_lp = NULL;
+      str_d_mlp = NULL;
+
+      if (0 >= cur->valid)
+      {
+        fprintf (stderr,
+           "No valid results for %s for %u peers %u addresses!\n",
+           (GNUNET_YES == cur->update) ? "updated" : "full",
+               cur->peers, cur->addresses);
+        continue;
+      }
+
+      if (GNUNET_TIME_UNIT_FOREVER_REL.rel_value_us != cur->d_total.rel_value_us)
+      {
+        fprintf (stderr,
+           "Average total time from %u iterations to solve %s for %u peers %u addresses: %llu us\n",
+           cur->valid,
+           (GNUNET_YES == cur->update) ? "updated" : "full",
+               cur->peers, cur->addresses,
+           (unsigned long long) cur->d_total.rel_value_us / cur->valid);
+        GNUNET_asprintf(&str_d_total, "%llu",
+           (unsigned long long) cur->d_total.rel_value_us / cur->valid);
+      }
+      else
+        GNUNET_asprintf (&str_d_total, "-1");
+
+      if (GNUNET_TIME_UNIT_FOREVER_REL.rel_value_us != cur->d_setup.rel_value_us)
+      {
+        fprintf (stderr,
+           "Average total time from %u iterations to setup for %u peers %u addresses: %llu us\n",
+           cur->valid, cur->peers, cur->addresses,
+           (unsigned long long) cur->d_setup.rel_value_us / cur->valid);
+        GNUNET_asprintf(&str_d_setup, "%llu",
+           (unsigned long long) cur->d_setup.rel_value_us / cur->valid);
+
+      }
+      else
+        GNUNET_asprintf (&str_d_setup, "-1");
+
+      if (GNUNET_TIME_UNIT_FOREVER_REL.rel_value_us != cur->d_lp.rel_value_us)
+      {
+        fprintf (stderr,
+           "Average total time from %u iterations to solve lp %s for %u peers %u addresses: %llu us\n",
+           cur->valid,
+           (GNUNET_YES == cur->update) ? "updated" : "full",
+           cur->peers, cur->addresses,
+           (unsigned long long) cur->d_lp.rel_value_us / cur->valid);
+        GNUNET_asprintf(&str_d_lp, "%llu",
+           (unsigned long long) cur->d_lp.rel_value_us / ph.total_iterations);
+      }
+      else
+        GNUNET_asprintf (&str_d_lp, "-1");
+
+      if (GNUNET_TIME_UNIT_FOREVER_REL.rel_value_us != cur->d_mlp.rel_value_us)
+      {
+        fprintf (stderr,
+           "Average total time from %u iterations to solve mlp %s for %u peers %u addresses: %llu us\n",
+           cur->valid,
+           (GNUNET_YES == cur->update) ? "updated" : "full",
+               cur->peers, cur->addresses,
+           (unsigned long long) cur->d_mlp.rel_value_us / cur->valid);
+        GNUNET_asprintf(&str_d_mlp, "%llu",
+           (unsigned long long) cur->d_mlp.rel_value_us / cur->valid);
+      }
+      else
+        GNUNET_asprintf (&str_d_mlp, "-1");
+
+      data = NULL;
+      if (GNUNET_YES == ph.create_plot)
+      {
+        GNUNET_asprintf (&data,
+                         "%u;%u;%s;%s;%s;%s\n",
+                         cur->peers, cur->addresses,
+                         str_d_total,
+                         str_d_setup,
+                         str_d_lp,
+                         str_d_mlp);
+        if (cur->update == GNUNET_NO)
+        {
+          if (GNUNET_SYSERR == GNUNET_DISK_file_write (f_full, data, strlen(data)))
+            GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+                        "Cannot write data to log file `%s'\n",
+                        data_fn_full);
+        }
+        if ((cur->update == GNUNET_YES) && (NULL != f_update))
+        {
+          if (GNUNET_SYSERR == GNUNET_DISK_file_write (f_update, data, strlen(data)))
+            GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+                        "Cannot write data to log file `%s'\n",
+                        data_fn_update);
+        }
+        GNUNET_free (data);
+      }
+
+      GNUNET_free_non_null (str_d_total);
+      GNUNET_free_non_null (str_d_setup);
+      GNUNET_free_non_null (str_d_lp);
+      GNUNET_free_non_null (str_d_mlp);
+    }
+  }
+
+  if ((NULL != f_full) && (GNUNET_SYSERR == GNUNET_DISK_file_close (f_full)))
+    GNUNET_log(GNUNET_ERROR_TYPE_ERROR, "Cannot close log file `%s'\n",
+        data_fn_full);
+  GNUNET_free_non_null (data_fn_full);
+
+  if ((NULL != f_update) && (GNUNET_SYSERR == GNUNET_DISK_file_close (f_update)))
+    GNUNET_log(GNUNET_ERROR_TYPE_ERROR, "Cannot close log file `%s'\n",
+        data_fn_update);
+  GNUNET_free_non_null (data_fn_update);
+}
+
+/**
+ * Run a performance iteration
+ */
 
 static void
-perf_run ()
+perf_run_iteration (void)
 {
   struct ATS_Address *cur;
   struct ATS_Address *next;
@@ -872,11 +1411,10 @@ perf_run ()
 
 
   ph.peers = GNUNET_malloc ((count_p) * sizeof (struct PerfPeer));
-
   for (cp = 0; cp < count_p; cp++)
     perf_create_peer (cp);
   GNUNET_log(GNUNET_ERROR_TYPE_INFO,
-      "Added %u peers\n", cp);
+      "Iteration %u of %u, added %u peers\n", ph.current_iteration, ph.total_iterations, cp);
 
   for (cp = 0; cp < count_p; cp++)
   {
@@ -909,9 +1447,7 @@ perf_run ()
         ph.env.sf.s_bulk_stop (ph.solver);
       }
       else
-      {
         GNUNET_break (0);
-      }
 
       /* Problem is solved by the solver here due to unlocking */
       ph.expecting_solution = GNUNET_NO;
@@ -924,6 +1460,7 @@ perf_run ()
             "Updating problem with %u peers and %u addresses\n", cp + 1, ca);
 
         ph.expecting_solution = GNUNET_YES;
+        ph.performed_update = GNUNET_YES;
         if (GNUNET_NO == ph.bulk_running)
         {
           ph.bulk_running = GNUNET_YES;
@@ -933,6 +1470,7 @@ perf_run ()
         ph.bulk_running = GNUNET_NO;
         ph.env.sf.s_bulk_stop (ph.solver);
         /* Problem is solved by the solver here due to unlocking */
+        ph.performed_update = GNUNET_NO;
         ph.expecting_solution = GNUNET_NO;
       }
       GNUNET_assert (GNUNET_NO == ph.bulk_running);
@@ -960,11 +1498,11 @@ perf_run ()
       GNUNET_CONTAINER_DLL_remove(ph.peers[cp].head, ph.peers[cp].tail, cur);
       GNUNET_free(cur);
     }
-
   }
-  GNUNET_free(ph.peers);
 
-  evaluate ();
+  GNUNET_log(GNUNET_ERROR_TYPE_INFO,
+      "Iteration done\n");
+  GNUNET_free(ph.peers);
 }
 
 
@@ -1039,11 +1577,11 @@ run (void *cls, char * const *args, const char *cfgfile,
     ph.N_address = DEFAULT_ADDRESSES;
 
   if (ph.N_peers_start != ph.N_peers_end)
-    fprintf (stderr, "Benchmarking solver `%s' with %u to %u peers and %u addresses\n",
-        ph.ats_string, ph.N_peers_start, ph.N_peers_end, ph.N_address);
+    fprintf (stderr, "Benchmarking solver `%s' with %u to %u peers and %u addresses in %u iterations\n",
+        ph.ats_string, ph.N_peers_start, ph.N_peers_end, ph.N_address, ph.total_iterations);
   else
-    fprintf (stderr, "Benchmarking solver `%s' with %u peers and %u addresses\n",
-        ph.ats_string, ph.N_peers_end, ph.N_address);
+    fprintf (stderr, "Benchmarking solver `%s' with %u peers and %u addresses in %u iterations\n",
+        ph.ats_string, ph.N_peers_end, ph.N_address, ph.total_iterations);
 
   if (0 == ph.opt_update_percent)
     ph.opt_update_percent = DEFAULT_UPDATE_PERCENTAGE;
@@ -1064,6 +1602,25 @@ run (void *cls, char * const *args, const char *cfgfile,
     return;
   }
 
+  /* Create array of DLL to store results for iterations */
+  ph.iterations_results = GNUNET_malloc (sizeof (struct Iteration) * ph.total_iterations);
+  ph.averaged_full_result = GNUNET_malloc (sizeof (struct Result) * ((ph.N_peers_end + 1) - ph.N_peers_start));
+  for (c = 0; c <= ph.N_peers_end - ph.N_peers_start; c++)
+  {
+    ph.averaged_full_result[c].d_setup = GNUNET_TIME_UNIT_FOREVER_REL;
+    ph.averaged_full_result[c].d_total = GNUNET_TIME_UNIT_FOREVER_REL;
+    ph.averaged_full_result[c].d_lp = GNUNET_TIME_UNIT_FOREVER_REL;
+    ph.averaged_full_result[c].d_mlp = GNUNET_TIME_UNIT_FOREVER_REL;
+  }
+  ph.averaged_update_result = GNUNET_malloc (sizeof (struct Result) * ((ph.N_peers_end + 1) - ph.N_peers_start));
+  for (c = 0; c <= ph.N_peers_end - ph.N_peers_start; c++)
+  {
+    ph.averaged_update_result[c].d_setup = GNUNET_TIME_UNIT_FOREVER_REL;
+    ph.averaged_update_result[c].d_total = GNUNET_TIME_UNIT_FOREVER_REL;
+    ph.averaged_update_result[c].d_lp = GNUNET_TIME_UNIT_FOREVER_REL;
+    ph.averaged_update_result[c].d_mlp = GNUNET_TIME_UNIT_FOREVER_REL;
+  }
+
   /* Load solver */
   ph.env.cfg = solver_cfg;
   ph.stat = GNUNET_STATISTICS_create ("ats", cfg);
@@ -1099,17 +1656,32 @@ run (void *cls, char * const *args, const char *cfgfile,
     return;
   }
 
-  /* Do work */
-  perf_run ();
+  /* Do the benchmark */
+  for (ph.current_iteration = 1; ph.current_iteration <= ph.total_iterations; ph.current_iteration++)
+  {
+    perf_run_iteration ();
+    evaluate (ph.current_iteration);
+  }
+  evaluate_average ();
 
   /* Unload solver*/
   GNUNET_log(GNUNET_ERROR_TYPE_INFO, _("Unloading solver `%s'\n"), ph.ats_string);
   GNUNET_PLUGIN_unload (plugin, ph.solver);
   GNUNET_free (plugin);
+  GNUNET_free (ph.iterations_results);
+  GNUNET_free (ph.averaged_full_result);
+  GNUNET_free (ph.averaged_update_result);
   GNUNET_CONFIGURATION_destroy (solver_cfg);
+  GNUNET_STATISTICS_destroy (ph.stat, GNUNET_NO);
   ph.solver = NULL;
 }
 
+/**
+ * Main function of the benchmark
+ *
+ * @param argc argument count
+ * @param argv argument values
+ */
 int
 main (int argc, char *argv[])
 {
@@ -1121,6 +1693,7 @@ main (int argc, char *argv[])
   ph.ats_string = NULL;
   ph.create_plot = GNUNET_NO;
   ph.measure_updates = GNUNET_NO;
+  ph.total_iterations = 1;
 
   static struct GNUNET_GETOPT_CommandLineOption options[] = {
       { 'a', "addresses", NULL,
@@ -1132,6 +1705,9 @@ main (int argc, char *argv[])
       { 'e', "end", NULL,
           gettext_noop ("end with peer"),
           1, &GNUNET_GETOPT_set_uint, &ph.N_peers_end },
+      { 'i', "iterations", NULL,
+          gettext_noop ("number of iterations used for averaging (default: 1)"),
+          1, &GNUNET_GETOPT_set_uint, &ph.total_iterations },
       { 'p', "percentage", NULL,
           gettext_noop ("update a fix percentage of addresses"),
           1, &GNUNET_GETOPT_set_uint, &ph.opt_update_percent },
@@ -1145,7 +1721,6 @@ main (int argc, char *argv[])
   };
 
   GNUNET_PROGRAM_run (argc, argv, argv[0], NULL, options, &run, argv[0]);
-
   return ret;
 }
 
