@@ -404,7 +404,6 @@ libgnunet_plugin_ats_proportional_init (void *cls)
 
   GNUNET_assert (NULL != env);
   GNUNET_assert(NULL != env->cfg);
-  GNUNET_assert(NULL != env->stats);
   GNUNET_assert(NULL != env->bandwidth_changed_cb);
   GNUNET_assert(NULL != env->get_preferences);
   GNUNET_assert(NULL != env->get_property);
@@ -454,7 +453,8 @@ libgnunet_plugin_ats_proportional_init (void *cls)
         "# ATS addresses %s total", cur->desc);
     GNUNET_asprintf (&cur->stat_active,
         "# ATS active addresses %s total", cur->desc);
-    LOG (GNUNET_ERROR_TYPE_INFO, "Added network %u `%s' %p\n", c, cur->desc, s);
+    LOG (GNUNET_ERROR_TYPE_INFO, "Added network %u `%s' (%llu/%llu)\n",
+        c, cur->desc, cur->total_quota_in, cur->total_quota_out);
   }
   return s;
 }
