@@ -36,6 +36,10 @@
 
 /**
  * Log macro that prefixes the local peer and the peer we are in contact with.
+ *
+ * @param kind log level
+ * @param cpi ConsensusPeerInformation of the partner peer
+ * @param m log message
  */
 #define LOG_PP(kind, cpi, m,...) GNUNET_log (kind, "P%d for P%d: " m, \
    cpi->session->local_peer_idx, (int) (cpi - cpi->session->info),##__VA_ARGS__)
@@ -44,7 +48,8 @@
 /**
  * Number of exponential rounds, used in the exp and completion round.
  */
-#define NUM_EXP_ROUNDS 4
+#define NUM_EXP_REPETITIONS 4
+
 
 /* forward declarations */
 
@@ -70,13 +75,7 @@ enum ConsensusRound
    */
   CONSENSUS_ROUND_EXCHANGE,
   /**
-   * Exchange which elements each peer has, but don't
-   * transmit the element's data, only their SHA-512 hashes.
-   * This round uses the all-to-all scheme.
-   */
-  CONSENSUS_ROUND_INVENTORY,
-  /**
-   * Collect and distribute missing values with the exponential scheme.
+   * Collect and distribute missing values.
    */
   CONSENSUS_ROUND_COMPLETION,
   /**
@@ -88,8 +87,7 @@ enum ConsensusRound
 
 
 /**
- * Complete information about the current round and all
- * subrounds.
+ * Information about the current round.
  */
 struct RoundInfo
 {
@@ -98,10 +96,10 @@ struct RoundInfo
    */
   enum ConsensusRound round;
   /**
-   * The current exp round, valid if
+   * The current exp round repetition, valid if
    * the main round is an exp round.
    */
-  uint32_t exp_round;
+  uint32_t exp_repetition;
   /**
    * The current exp subround, valid if
    * the main round is an exp round.
@@ -141,15 +139,20 @@ struct ConsensusSession
    */
   struct GNUNET_MQ_Handle *client_mq;
 
+  /**
+   * Time when the conclusion of the consensus should begin.
+   */
+  struct GNUNET_TIME_Absolute conclude_start;
+
   /**
    * Timeout for all rounds together, single rounds will schedule a timeout task
    * with a fraction of the conclude timeout.
    * Only valid once the current round is not CONSENSUS_ROUND_BEGIN.
    */
-  struct GNUNET_TIME_Relative conclude_timeout;
+  struct GNUNET_TIME_Absolute conclude_deadline;
 
   /**
-   * Timeout task identifier for the current round.
+   * Timeout task identifier for the current round or subround.
    */
   GNUNET_SCHEDULER_TaskIdentifier round_timeout_tid;
 
@@ -187,7 +190,7 @@ struct ConsensusSession
   /**
    * Current round of the exponential scheme.
    */
-  uint32_t exp_round;
+  uint32_t exp_repetition;
 
   /**
    * Current sub-round of the exponential scheme.
@@ -195,12 +198,16 @@ struct ConsensusSession
   uint32_t exp_subround;
 
   /**
-   * The partner for the current exp-round
+   * The partner for the current exp-round.
+   * The local peer will initiate the set reconciliation with the
+   * outgoing peer.
    */
   struct ConsensusPeerInformation *partner_outgoing;
 
   /**
    * The partner for the current exp-round
+   * The incoming peer will initiate the set reconciliation with
+   * the incoming peer.
    */
   struct ConsensusPeerInformation *partner_incoming;
 
@@ -234,9 +241,9 @@ struct ConsensusPeerInformation
   struct ConsensusSession *session;
 
   /**
-   * We have finishes the exp-subround with the peer.
+   * Have we finished the set operation for this (sub-)round?
    */
-  int exp_subround_finished;
+  int set_op_finished;
 
   /**
    * Set operation we are currently executing with this peer.
@@ -281,16 +288,27 @@ static struct GNUNET_SERVER_Handle *srv;
 static struct GNUNET_PeerIdentity my_peer;
 
 
+/**
+ * Check if the current subround has finished.
+ * Must only be called when an exp-round is the current round.
+ *
+ * @param session session to check for exp-round completion
+ * @return GNUNET_YES if the subround has finished,
+ *         GNUNET_NO if not
+ */
 static int
 have_exp_subround_finished (const struct ConsensusSession *session)
 {
   int not_finished;
+
+  GNUNET_assert (CONSENSUS_ROUND_EXCHANGE == session->current_round);
+
   not_finished = 0;
   if ( (NULL != session->partner_outgoing) &&
-       (GNUNET_NO == session->partner_outgoing->exp_subround_finished) )
+       (GNUNET_NO == session->partner_outgoing->set_op_finished) )
     not_finished++;
   if ( (NULL != session->partner_incoming) &&
-       (GNUNET_NO == session->partner_incoming->exp_subround_finished) )
+       (GNUNET_NO == session->partner_incoming->set_op_finished) )
     not_finished++;
   if (0 == not_finished)
     return GNUNET_YES;
@@ -408,6 +426,8 @@ static void
 round_over (void *cls, const struct GNUNET_SCHEDULER_TaskContext *tc)
 {
   struct ConsensusSession *session;
+  unsigned int i;
+  int res;
 
   /* don't kick off next round if we're shutting down */
   if ((NULL != tc) && (tc->reason & GNUNET_SCHEDULER_REASON_SHUTDOWN))
@@ -416,24 +436,48 @@ round_over (void *cls, const struct GNUNET_SCHEDULER_TaskContext *tc)
   session = cls;
   GNUNET_log (GNUNET_ERROR_TYPE_DEBUG, "P%d: round over\n", session->local_peer_idx);
 
+  if (tc != NULL)
+    session->round_timeout_tid = GNUNET_SCHEDULER_NO_TASK;
+
   if (session->round_timeout_tid != GNUNET_SCHEDULER_NO_TASK)
   {
     GNUNET_SCHEDULER_cancel (session->round_timeout_tid);
     session->round_timeout_tid = GNUNET_SCHEDULER_NO_TASK;
   }
 
+  for (i = 0; i < session->num_peers; i++)
+  {
+    if (NULL != session->info[i].set_op)
+    {
+      GNUNET_log (GNUNET_ERROR_TYPE_DEBUG, "P%d: canceling stray op with P%d\n",
+                  session->local_peer_idx, i);
+      GNUNET_SET_operation_cancel (session->info[i].set_op);
+      session->info[i].set_op = NULL;
+    }
+    /* we're in the new round, nothing finished yet */
+    session->info[i].set_op_finished = GNUNET_NO;
+  }
+
   switch (session->current_round)
   {
     case CONSENSUS_ROUND_BEGIN:
       session->current_round = CONSENSUS_ROUND_EXCHANGE;
-      session->exp_round = 0;
+      session->exp_repetition = 0;
       subround_over (session, NULL);
       break;
     case CONSENSUS_ROUND_EXCHANGE:
       GNUNET_log (GNUNET_ERROR_TYPE_DEBUG, "P%d: finished, sending elements to client\n",
                   session->local_peer_idx);
       session->current_round = CONSENSUS_ROUND_FINISH;
-      GNUNET_SET_iterate (session->element_set, send_to_client_iter, session);
+      res = GNUNET_SET_iterate (session->element_set, send_to_client_iter, session);
+      if (GNUNET_SYSERR == res)
+      {
+        GNUNET_log (GNUNET_ERROR_TYPE_ERROR, "can't iterate set: set invalid\n");
+      }
+      else if (GNUNET_NO == res)
+      {
+        GNUNET_log (GNUNET_ERROR_TYPE_ERROR, "can't iterate set: iterator already active\n");
+      }
       break;
     default:
       GNUNET_assert (0);
@@ -460,7 +504,7 @@ shuffle (struct ConsensusSession *session)
     session->shuffle_inv = GNUNET_malloc (session->num_peers * sizeof (*session->shuffle_inv));
 
   GNUNET_CRYPTO_kdf (randomness, sizeof (randomness),
-		     &session->exp_round, sizeof (uint32_t),
+		     &session->exp_repetition, sizeof (uint32_t),
                      &session->global_id, sizeof (struct GNUNET_HashCode),
 		     NULL);
 
@@ -507,16 +551,16 @@ find_partners (struct ConsensusSession *session)
   while (largest_arc < session->num_peers)
     largest_arc <<= 1;
   num_ghosts = largest_arc - session->num_peers;
-  GNUNET_log (GNUNET_ERROR_TYPE_DEBUG, "largest arc: %u\n", largest_arc);
-  GNUNET_log (GNUNET_ERROR_TYPE_DEBUG, "arc: %u\n", arc);
-  GNUNET_log (GNUNET_ERROR_TYPE_DEBUG, "num ghosts: %u\n", num_ghosts);
+  // GNUNET_log (GNUNET_ERROR_TYPE_DEBUG, "largest arc: %u\n", largest_arc);
+  // GNUNET_log (GNUNET_ERROR_TYPE_DEBUG, "arc: %u\n", arc);
+  // GNUNET_log (GNUNET_ERROR_TYPE_DEBUG, "num ghosts: %u\n", num_ghosts);
 
   if (0 == (my_idx & arc))
   {
     /* we are outgoing */
     partner_idx = (my_idx + arc) % session->num_peers;
     session->partner_outgoing = &session->info[session->shuffle_inv[partner_idx]];
-    session->partner_outgoing->exp_subround_finished = GNUNET_NO;
+    GNUNET_assert (GNUNET_NO == session->partner_outgoing->set_op_finished);
     /* are we a 'ghost' of a peer that would exist if
      * the number of peers was a power of two, and thus have to partner
      * with an additional peer?
@@ -524,7 +568,7 @@ find_partners (struct ConsensusSession *session)
     if (my_idx < num_ghosts)
     {
       int ghost_partner_idx;
-      GNUNET_log (GNUNET_ERROR_TYPE_DEBUG, "my index %d, arc %d, peers %u\n", my_idx, arc, session->num_peers);
+      // GNUNET_log (GNUNET_ERROR_TYPE_DEBUG, "my index %d, arc %d, peers %u\n", my_idx, arc, session->num_peers);
       ghost_partner_idx = (my_idx - (int) arc) % (int) session->num_peers;
       /* platform dependent; modulo sometimes returns negative values */
       if (ghost_partner_idx < 0)
@@ -532,9 +576,9 @@ find_partners (struct ConsensusSession *session)
       /* we only need to have a ghost partner if the partner is outgoing */
       if (0 == (ghost_partner_idx & arc))
       {
-        GNUNET_log (GNUNET_ERROR_TYPE_DEBUG, "ghost partner is %d\n", ghost_partner_idx);
+        // GNUNET_log (GNUNET_ERROR_TYPE_DEBUG, "ghost partner is %d\n", ghost_partner_idx);
         session->partner_incoming = &session->info[session->shuffle_inv[ghost_partner_idx]];
-        session->partner_incoming->exp_subround_finished = GNUNET_NO;
+        GNUNET_assert (GNUNET_NO == session->partner_incoming->set_op_finished);
         return;
       }
     }
@@ -547,7 +591,7 @@ find_partners (struct ConsensusSession *session)
     partner_idx += session->num_peers;
   session->partner_outgoing = NULL;
   session->partner_incoming = &session->info[session->shuffle_inv[partner_idx]];
-  session->partner_incoming->exp_subround_finished = GNUNET_NO;
+  GNUNET_assert (GNUNET_NO == session->partner_incoming->set_op_finished);
 }
 
 
@@ -568,6 +612,9 @@ set_result_cb (void *cls,
   unsigned int remote_idx = cpi - cpi->session->info;
   unsigned int local_idx = cpi->session->local_peer_idx;
 
+  GNUNET_log (GNUNET_ERROR_TYPE_DEBUG, "P%u: set result from P%u with status %u\n",
+              local_idx, remote_idx, (unsigned int) status);
+
   GNUNET_assert ((cpi == cpi->session->partner_outgoing) ||
                  (cpi == cpi->session->partner_incoming));
 
@@ -586,7 +633,7 @@ set_result_cb (void *cls,
     case GNUNET_SET_STATUS_DONE:
       GNUNET_log (GNUNET_ERROR_TYPE_DEBUG, "P%u: set result from P%u: done\n",
                   local_idx, remote_idx);
-      cpi->exp_subround_finished = GNUNET_YES;
+      cpi->set_op_finished = GNUNET_YES;
       cpi->set_op = NULL;
       if (have_exp_subround_finished (cpi->session) == GNUNET_YES)
       {
@@ -607,6 +654,7 @@ set_result_cb (void *cls,
 
   switch (cpi->session->current_round)
   {
+    case CONSENSUS_ROUND_COMPLETION:
     case CONSENSUS_ROUND_EXCHANGE:
       GNUNET_SET_add_element (cpi->session->element_set, element, NULL, NULL);
       break;
@@ -635,18 +683,18 @@ rounds_compare (struct ConsensusSession *session,
     return 1;
   if (session->current_round == CONSENSUS_ROUND_EXCHANGE)
   {
-    if (session->exp_round < ri->exp_round)
+    if (session->exp_repetition < ri->exp_repetition)
       return -1;
-    if (session->exp_round > ri->exp_round)
+    if (session->exp_repetition > ri->exp_repetition)
       return 1;
     if (session->exp_subround < ri->exp_subround)
       return -1;
-    if (session->exp_subround < ri->exp_subround)
+    if (session->exp_subround > ri->exp_subround)
       return 1;
     return 0;
   }
-  /* comparing rounds when we are not in a exp round */
-  GNUNET_assert (0);
+  /* other rounds have no subrounds / repetitions to compare */
+  return 0;
 }
 
 
@@ -662,12 +710,24 @@ static void
 subround_over (void *cls, const struct GNUNET_SCHEDULER_TaskContext *tc)
 {
   struct ConsensusSession *session;
+  struct GNUNET_TIME_Relative subround_timeout;
   int i;
 
   /* don't kick off next subround if we're shutting down */
   if ((NULL != tc) && (tc->reason & GNUNET_SCHEDULER_REASON_SHUTDOWN))
     return;
+
   session = cls;
+
+  GNUNET_assert (CONSENSUS_ROUND_EXCHANGE == session->current_round);
+
+  if (tc != NULL)
+  {
+    session->round_timeout_tid = GNUNET_SCHEDULER_NO_TASK;
+    GNUNET_log (GNUNET_ERROR_TYPE_WARNING, "P%u: consensus subround timed out\n",
+                session->local_peer_idx);
+  }
+
   /* cancel timeout */
   if (session->round_timeout_tid != GNUNET_SCHEDULER_NO_TASK)
   {
@@ -675,16 +735,29 @@ subround_over (void *cls, const struct GNUNET_SCHEDULER_TaskContext *tc)
     session->round_timeout_tid = GNUNET_SCHEDULER_NO_TASK;
   }
 
-  if (session->exp_round >= NUM_EXP_ROUNDS)
+  for (i = 0; i < session->num_peers; i++)
+  {
+    if (NULL != session->info[i].set_op)
+    {
+      GNUNET_log (GNUNET_ERROR_TYPE_DEBUG, "P%d: canceling stray op with P%d\n",
+                  session->local_peer_idx, i);
+      GNUNET_SET_operation_cancel (session->info[i].set_op);
+      session->info[i].set_op = NULL;
+    }
+    /* we're in the new round, nothing finished yet */
+    session->info[i].set_op_finished = GNUNET_NO;
+  }
+
+  if (session->exp_repetition >= NUM_EXP_REPETITIONS)
   {
     round_over (session, NULL);
     return;
   }
 
-  if (session->exp_round == 0)
+  if (session->exp_repetition == 0)
   {
     /* initialize everything for the log-rounds */
-    session->exp_round = 1;
+    session->exp_repetition = 1;
     session->exp_subround = 0;
     if (NULL == session->shuffle)
       session->shuffle = GNUNET_malloc ((sizeof (int)) * session->num_peers);
@@ -696,7 +769,7 @@ subround_over (void *cls, const struct GNUNET_SCHEDULER_TaskContext *tc)
   else if (session->exp_subround + 1 >= (int) ceil (log2 (session->num_peers)))
   {
     /* subrounds done, start new log-round */
-    session->exp_round++;
+    session->exp_repetition++;
     session->exp_subround = 0;
     shuffle (session);
   }
@@ -705,6 +778,14 @@ subround_over (void *cls, const struct GNUNET_SCHEDULER_TaskContext *tc)
     session->exp_subround++;
   }
 
+  subround_timeout =
+      GNUNET_TIME_relative_divide (GNUNET_TIME_absolute_get_difference (session->conclude_start, session->conclude_deadline),
+                                   2 * NUM_EXP_REPETITIONS * ((int) ceil (log2 (session->num_peers))));
+
+  GNUNET_log (GNUNET_ERROR_TYPE_DEBUG, "subround timeout: %u ms\n", subround_timeout.rel_value_us / 1000);
+
+  session->round_timeout_tid = GNUNET_SCHEDULER_add_delayed (subround_timeout, subround_over, session);
+
   /* determine the incoming and outgoing partner */
   find_partners (session);
 
@@ -719,11 +800,12 @@ subround_over (void *cls, const struct GNUNET_SCHEDULER_TaskContext *tc)
     msg->header.type = htons (GNUNET_MESSAGE_TYPE_CONSENSUS_P2P_ROUND_CONTEXT);
     msg->header.size = htons (sizeof *msg);
     msg->round = htonl (session->current_round);
-    msg->exp_round = htonl (session->exp_round);
+    msg->exp_repetition = htonl (session->exp_repetition);
     msg->exp_subround = htonl (session->exp_subround);
 
     if (NULL != session->partner_outgoing->set_op)
     {
+      GNUNET_break (0);
       GNUNET_SET_operation_cancel (session->partner_outgoing->set_op);
     }
     session->partner_outgoing->set_op =
@@ -734,7 +816,12 @@ subround_over (void *cls, const struct GNUNET_SCHEDULER_TaskContext *tc)
                             GNUNET_SET_RESULT_ADDED,
                             set_result_cb, session->partner_outgoing);
     GNUNET_free (msg);
-    GNUNET_SET_commit (session->partner_outgoing->set_op, session->element_set);
+    if (GNUNET_OK != GNUNET_SET_commit (session->partner_outgoing->set_op, session->element_set))
+    {
+      GNUNET_break (0);
+      session->partner_outgoing->set_op = NULL;
+      session->partner_outgoing->set_op_finished = GNUNET_YES;
+    }
   }
 
   /* commit to the delayed set operation */
@@ -744,12 +831,16 @@ subround_over (void *cls, const struct GNUNET_SCHEDULER_TaskContext *tc)
 
     if (NULL != session->partner_incoming->set_op)
     {
+      GNUNET_break (0);
       GNUNET_SET_operation_cancel (session->partner_incoming->set_op);
       session->partner_incoming->set_op = NULL;
     }
     if (cmp == 0)
     {
-      GNUNET_SET_commit (session->partner_incoming->delayed_set_op, session->element_set);
+      if (GNUNET_OK != GNUNET_SET_commit (session->partner_incoming->delayed_set_op, session->element_set))
+      {
+        GNUNET_break (0);
+      }
       session->partner_incoming->set_op = session->partner_incoming->delayed_set_op;
       session->partner_incoming->delayed_set_op = NULL;
       GNUNET_log (GNUNET_ERROR_TYPE_DEBUG, "P%d resumed delayed round with P%d\n",
@@ -775,7 +866,7 @@ subround_over (void *cls, const struct GNUNET_SCHEDULER_TaskContext *tc)
     else
       in = (int) (session->partner_incoming - session->info);
     GNUNET_log (GNUNET_ERROR_TYPE_DEBUG, "P%u: doing exp-round, r=%d, sub=%d, in: %d, out: %d\n", session->local_peer_idx,
-                session->exp_round, session->exp_subround, in, out);
+                session->exp_repetition, session->exp_subround, in, out);
   }
 #endif /* GNUNET_EXTRA_LOGGING */
 
@@ -832,18 +923,16 @@ compute_global_id (struct ConsensusSession *session,
 
 
 /**
- * Although GNUNET_CRYPTO_hash_cmp exisits, it does not have
- * the correct signature to be used with e.g. qsort.
- * We use this function instead.
+ * Compare two peer identities.
  *
- * @param h1 some hash code
- * @param h2 some hash code
+ * @param h1 some peer identity
+ * @param h2 some peer identity
  * @return 1 if h1 > h2, -1 if h1 < h2 and 0 if h1 == h2.
  */
 static int
-hash_cmp (const void *h1, const void *h2)
+peer_id_cmp (const void *h1, const void *h2)
 {
-  return GNUNET_CRYPTO_hash_cmp ((struct GNUNET_HashCode *) h1, (struct GNUNET_HashCode *) h2);
+  return memcmp (h1, h2, sizeof (struct GNUNET_PeerIdentity));
 }
 
 
@@ -889,7 +978,7 @@ initialize_session_peer_list (struct ConsensusSession *session,
     peers[session->num_peers - 1] = my_peer;
 
   memcpy (peers, msg_peers, listed_peers * sizeof (struct GNUNET_PeerIdentity));
-  qsort (peers, session->num_peers, sizeof (struct GNUNET_PeerIdentity), &hash_cmp);
+  qsort (peers, session->num_peers, sizeof (struct GNUNET_PeerIdentity), &peer_id_cmp);
 
   session->info = GNUNET_malloc (session->num_peers * sizeof (struct ConsensusPeerInformation));
 
@@ -949,7 +1038,7 @@ set_listen_cb (void *cls,
   }
 
   round_info.round = ntohl (msg->round);
-  round_info.exp_round = ntohl (msg->exp_round);
+  round_info.exp_repetition = ntohl (msg->exp_repetition);
   round_info.exp_subround = ntohl (msg->exp_subround);
 
   cpi = &session->info[index];
@@ -966,7 +1055,7 @@ set_listen_cb (void *cls,
       if (cmp > 0)
       {
         /* the other peer is too late */
-        GNUNET_break_op (0);
+        LOG_PP (GNUNET_ERROR_TYPE_DEBUG, cpi, "too late for the current round\n");
         return;
       }
       /* kill old request, if any. this is legal,
@@ -974,23 +1063,38 @@ set_listen_cb (void *cls,
        * complete the old one! */
       if (NULL != cpi->set_op)
       {
+        LOG_PP (GNUNET_ERROR_TYPE_INFO, cpi, "got new request from same peer, canceling old one\n");
         GNUNET_SET_operation_cancel (cpi->set_op);
         cpi->set_op = NULL;
       }
       set_op = GNUNET_SET_accept (request, GNUNET_SET_RESULT_ADDED,
-                                       set_result_cb, &session->info[index]);
+                                  set_result_cb, &session->info[index]);
       if (cmp == 0)
       {
+        /* we're in exactly the right round for the incoming request */
+        if (cpi != cpi->session->partner_incoming)
+        {
+          GNUNET_log (GNUNET_ERROR_TYPE_ERROR, "P%u: got request from %u (with matching round), "
+                      "but incoming partner is %d\n", cpi->session->local_peer_idx, cpi - cpi->session->info,
+                      ((NULL == cpi->session->partner_incoming) ? -1 : (cpi->session->partner_incoming - cpi->session->info)));
+          GNUNET_SET_operation_cancel (set_op);
+          return;
+        }
         cpi->set_op = set_op;
-        GNUNET_SET_commit (set_op, session->element_set);
+        if (GNUNET_OK != GNUNET_SET_commit (set_op, session->element_set))
+        {
+          GNUNET_break (0);
+        }
         GNUNET_log (GNUNET_ERROR_TYPE_DEBUG, "P%d commited to set request from P%d\n", session->local_peer_idx, index);
       }
       else
       {
-        /* if there's a exp subround running, mark it as finished, as the set op has been canceled! */
+        /* we still have wait until we have finished the current round,
+         * as the other peer's round is larger */
         cpi->delayed_set_op = set_op;
         cpi->delayed_round_info = round_info;
-        cpi->exp_subround_finished = GNUNET_YES;
+        /* The current setop is finished, as we canceled the current setop above. */
+        cpi->set_op_finished = GNUNET_YES;
         GNUNET_log (GNUNET_ERROR_TYPE_DEBUG, "P%d delaying set request from P%d\n", session->local_peer_idx, index);
       }
       break;
@@ -1039,6 +1143,12 @@ initialize_session (struct ConsensusSession *session,
     other_session = other_session->next;
   }
 
+  session->conclude_deadline = GNUNET_TIME_absolute_ntoh (join_msg->deadline);
+  session->conclude_start = GNUNET_TIME_absolute_ntoh (join_msg->start);
+
+  GNUNET_log (GNUNET_ERROR_TYPE_INFO, "consensus with timeout %ums created\n",
+              (GNUNET_TIME_absolute_get_difference (session->conclude_start, session->conclude_deadline)).rel_value_us / 1000);
+
   session->local_peer_idx = get_peer_idx (&my_peer, session);
   GNUNET_assert (-1 != session->local_peer_idx);
   session->element_set = GNUNET_SET_create (cfg, GNUNET_SET_OPERATION_UNION);
@@ -1151,7 +1261,7 @@ client_insert (void *cls,
   GNUNET_free (element);
   GNUNET_SERVER_receive_done (client, GNUNET_OK);
 
-  GNUNET_log (GNUNET_ERROR_TYPE_DEBUG, "P%u: element added\n", session->local_peer_idx);
+  // GNUNET_log (GNUNET_ERROR_TYPE_DEBUG, "P%u: element added\n", session->local_peer_idx);
 }
 
 
@@ -1168,11 +1278,8 @@ client_conclude (void *cls,
                  const struct GNUNET_MessageHeader *message)
 {
   struct ConsensusSession *session;
-  struct GNUNET_CONSENSUS_ConcludeMessage *cmsg;
-
 
   GNUNET_log (GNUNET_ERROR_TYPE_DEBUG, "conclude requested\n");
-  cmsg = (struct GNUNET_CONSENSUS_ConcludeMessage *) message;
   session = get_session_by_client (client);
   if (NULL == session)
   {
@@ -1194,7 +1301,6 @@ client_conclude (void *cls,
   }
   else
   {
-    session->conclude_timeout = GNUNET_TIME_relative_ntoh (cmsg->timeout);
     /* the 'begin' round is over, start with the next, actual round */
     round_over (session, NULL);
   }
@@ -1238,7 +1344,10 @@ handle_client_disconnect (void *cls, struct GNUNET_SERVER_Client *client)
     return;
   if ((CONSENSUS_ROUND_BEGIN == session->current_round) ||
       (CONSENSUS_ROUND_FINISH == session->current_round))
+  {
+    GNUNET_log (GNUNET_ERROR_TYPE_DEBUG, "client disconnected, destroying session\n");
     destroy_session (session);
+  }
   else
     GNUNET_log (GNUNET_ERROR_TYPE_DEBUG, "client disconnected, but waiting for consensus to finish\n");
 }
@@ -1257,7 +1366,7 @@ run (void *cls, struct GNUNET_SERVER_Handle *server,
 {
   static const struct GNUNET_SERVER_MessageHandler server_handlers[] = {
     {&client_conclude, NULL, GNUNET_MESSAGE_TYPE_CONSENSUS_CLIENT_CONCLUDE,
-        sizeof (struct GNUNET_CONSENSUS_ConcludeMessage)},
+        sizeof (struct GNUNET_MessageHeader)},
     {&client_insert, NULL, GNUNET_MESSAGE_TYPE_CONSENSUS_CLIENT_INSERT, 0},
     {&client_join, NULL, GNUNET_MESSAGE_TYPE_CONSENSUS_CLIENT_JOIN, 0},
     {NULL, NULL, 0, 0}
