@@ -59,7 +59,7 @@ insert_done (void *cls, int success)
   GNUNET_assert (GNUNET_NO == called);
   called = GNUNET_YES;
   GNUNET_log (GNUNET_ERROR_TYPE_INFO, "insert done\n");
-  GNUNET_CONSENSUS_conclude (consensus, GNUNET_TIME_UNIT_SECONDS, &conclude_done, NULL);
+  GNUNET_CONSENSUS_conclude (consensus, &conclude_done, NULL);
 }
 
 
@@ -100,7 +100,10 @@ run (void *cls,
   GNUNET_SCHEDULER_add_delayed (GNUNET_TIME_UNIT_FOREVER_REL, &on_shutdown, NULL);
 
   GNUNET_CRYPTO_hash (str, strlen (str), &session_id);
-  consensus = GNUNET_CONSENSUS_create (cfg, 0, NULL, &session_id, on_new_element, &consensus);
+  consensus = GNUNET_CONSENSUS_create (cfg, 0, NULL, &session_id,
+      GNUNET_TIME_relative_to_absolute (GNUNET_TIME_UNIT_SECONDS),
+      GNUNET_TIME_relative_to_absolute (GNUNET_TIME_UNIT_MINUTES),
+      on_new_element, &consensus);
   GNUNET_assert (consensus != NULL);
 
   GNUNET_CONSENSUS_insert (consensus, &el1, NULL, &consensus);
