@@ -37,6 +37,7 @@ struct GNUNET_CORE_RequestContext
   struct GNUNET_CLIENT_Connection *client;
 
   /**
+
    * Handle for transmitting a request.
    */
   struct GNUNET_CLIENT_TransmitHandle *th;
@@ -70,7 +71,6 @@ receive_info (void *cls, const struct GNUNET_MessageHeader *msg)
 {
   struct GNUNET_CORE_RequestContext *request_context = cls;
   const struct ConnectNotifyMessage *connect_message;
-  uint32_t ats_count;
   uint16_t msize;
 
   /* Handle last message or error case, disconnect and clean up */
@@ -79,7 +79,7 @@ receive_info (void *cls, const struct GNUNET_MessageHeader *msg)
        (ntohs (msg->size) == sizeof (struct GNUNET_MessageHeader))))
   {
     if (request_context->peer_cb != NULL)
-      request_context->peer_cb (request_context->cb_cls, NULL, NULL, 0);
+      request_context->peer_cb (request_context->cb_cls, NULL);
     GNUNET_CLIENT_disconnect (request_context->client);
     GNUNET_free (request_context);
     return;
@@ -92,29 +92,24 @@ receive_info (void *cls, const struct GNUNET_MessageHeader *msg)
   {
     GNUNET_break (0);
     if (request_context->peer_cb != NULL)
-      request_context->peer_cb (request_context->cb_cls, NULL, NULL, 0);
+      request_context->peer_cb (request_context->cb_cls, NULL);
     GNUNET_CLIENT_disconnect (request_context->client);
     GNUNET_free (request_context);
     return;
   }
   connect_message = (const struct ConnectNotifyMessage *) msg;
-  ats_count = ntohl (connect_message->ats_count);
-  if (msize !=
-      sizeof (struct ConnectNotifyMessage) +
-      ats_count * sizeof (struct GNUNET_ATS_Information))
+  if (msize != sizeof (struct ConnectNotifyMessage))
   {
     GNUNET_break (0);
     if (request_context->peer_cb != NULL)
-      request_context->peer_cb (request_context->cb_cls, NULL, NULL, 0);
+      request_context->peer_cb (request_context->cb_cls, NULL);
     GNUNET_CLIENT_disconnect (request_context->client);
     GNUNET_free (request_context);
     return;
   }
   /* Normal case */
   if (request_context->peer_cb != NULL)
-    request_context->peer_cb (request_context->cb_cls, &connect_message->peer,
-                              (const struct GNUNET_ATS_Information *)
-                              &connect_message[1], ats_count);
+    request_context->peer_cb (request_context->cb_cls, &connect_message->peer);
   GNUNET_CLIENT_receive (request_context->client, &receive_info,
                          request_context, GNUNET_TIME_UNIT_FOREVER_REL);
 }
@@ -126,7 +121,7 @@ receive_info (void *cls, const struct GNUNET_MessageHeader *msg)
  * NULL and "size" zero if the socket was closed for
  * writing in the meantime.
  *
- * @param cls closure
+ * @param cls closure, always NULL
  * @param size number of bytes available in buf
  * @param buf where the callee should write the message
  * @return number of bytes written to buf
@@ -135,32 +130,19 @@ static size_t
 transmit_request (void *cls, size_t size, void *buf)
 {
   struct GNUNET_MessageHeader *msg;
-  struct GNUNET_PeerIdentity *peer = cls;
   int msize;
 
-  if (peer == NULL)
-    msize = sizeof (struct GNUNET_MessageHeader);
-  else
-    msize =
-        sizeof (struct GNUNET_MessageHeader) +
-        sizeof (struct GNUNET_PeerIdentity);
+  msize = sizeof (struct GNUNET_MessageHeader);
   if ((size < msize) || (buf == NULL))
     return 0;
   msg = (struct GNUNET_MessageHeader *) buf;
   msg->size = htons (msize);
-  if (peer != NULL)
-  {
-    msg->type = htons (GNUNET_MESSAGE_TYPE_CORE_PEER_CONNECTED);
-    memcpy (&msg[1], peer, sizeof (struct GNUNET_PeerIdentity));
-  }
-  else
-    msg->type = htons (GNUNET_MESSAGE_TYPE_CORE_ITERATE_PEERS);
+  msg->type = htons (GNUNET_MESSAGE_TYPE_CORE_ITERATE_PEERS);
 
   return msize;
 }
 
 
-
 /**
  * Iterate over all currently connected peers.
  * Calls peer_cb with each connected peer, and then
@@ -169,9 +151,8 @@ transmit_request (void *cls, size_t size, void *buf)
  *
  * @param cfg configuration to use
  * @param peer_cb function to call with the peer information
- * @param cb_cls closure for peer_cb
- *
- * @return GNUNET_OK if iterating, GNUNET_SYSERR on error
+ * @param cb_cls closure for @a peer_cb
+ * @return #GNUNET_OK if iterating, #GNUNET_SYSERR on error
  */
 int
 GNUNET_CORE_iterate_peers (const struct GNUNET_CONFIGURATION_Handle *cfg,
@@ -184,7 +165,7 @@ GNUNET_CORE_iterate_peers (const struct GNUNET_CONFIGURATION_Handle *cfg,
   client = GNUNET_CLIENT_connect ("core", cfg);
   if (client == NULL)
     return GNUNET_SYSERR;
-  request_context = GNUNET_malloc (sizeof (struct GNUNET_CORE_RequestContext));
+  request_context = GNUNET_new (struct GNUNET_CORE_RequestContext);
   request_context->client = client;
   request_context->peer_cb = peer_cb;
   request_context->cb_cls = cb_cls;
