@@ -43,7 +43,7 @@
 
 
 /**
- * Data structure for each client connected to the core service.
+ * Data structure for each client connected to the CORE service.
  */
 struct GSC_Client
 {
@@ -64,14 +64,14 @@ struct GSC_Client
 
   /**
    * Array of the types of messages this peer cares
-   * about (with "tcnt" entries).  Allocated as part
+   * about (with @e tcnt entries).  Allocated as part
    * of this client struct, do not free!
    */
   const uint16_t *types;
 
   /**
    * Map of peer identities to active transmission requests of this
-   * client to the peer (of type 'struct GSC_ClientActiveRequest').
+   * client to the peer (of type `struct GSC_ClientActiveRequest`).
    */
   struct GNUNET_CONTAINER_MultiPeerMap *requests;
 
@@ -88,7 +88,7 @@ struct GSC_Client
 
   /**
    * Number of types of incoming messages this client
-   * specifically cares about.  Size of the "types" array.
+   * specifically cares about.  Size of the @e types array.
    */
   unsigned int tcnt;
 
@@ -190,7 +190,7 @@ GSC_CLIENTS_send_to_client (struct GNUNET_SERVER_Client *client,
  *
  * @param type message type
  * @param c client to test
- * @return GNUNET_YES if 'c' is interested, GNUNET_NO if not.
+ * @return #GNUNET_YES if @a c is interested, #GNUNET_NO if not.
  */
 static int
 type_match (uint16_t type, struct GSC_Client *c)
@@ -225,7 +225,7 @@ send_to_all_clients (const struct GNUNET_PeerIdentity *partner,
   struct GSC_Client *c;
   int tm;
 
-  for (c = client_head; c != NULL; c = c->next)
+  for (c = client_head; NULL != c; c = c->next)
   {
     tm = type_match (type, c);
     if (!  ( (0 != (c->options & options)) ||
@@ -374,7 +374,7 @@ handle_client_send_request (void *cls, struct GNUNET_SERVER_Client *client,
   }
 
   car = GNUNET_CONTAINER_multipeermap_get (c->requests, &req->peer);
-  if (car == NULL)
+  if (NULL == car)
   {
     /* create new entry */
     car = GNUNET_new (struct GSC_ClientActiveRequest);
@@ -391,7 +391,7 @@ handle_client_send_request (void *cls, struct GNUNET_SERVER_Client *client,
   }
   car->target = req->peer;
   car->deadline = GNUNET_TIME_absolute_ntoh (req->deadline);
-  car->priority = ntohl (req->priority);
+  car->priority = (enum GNUNET_CORE_Priority) ntohl (req->priority);
   car->msize = ntohs (req->size);
   car->smr_id = req->smr_id;
   car->was_solicited = GNUNET_NO;
@@ -408,7 +408,7 @@ handle_client_send_request (void *cls, struct GNUNET_SERVER_Client *client,
 
 
 /**
- * Closure for the 'client_tokenizer_callback'.
+ * Closure for the #client_tokenizer_callback().
  */
 struct TokenizerContext
 {
@@ -418,6 +418,11 @@ struct TokenizerContext
    */
   struct GSC_ClientActiveRequest *car;
 
+  /**
+   * How important is this message.
+   */
+  enum GNUNET_CORE_Priority priority;
+
   /**
    * Is corking allowed (set only once we have the real message).
    */
@@ -431,7 +436,7 @@ struct TokenizerContext
  *
  * @param cls unused
  * @param client the client issuing the request
- * @param message the "struct SendMessage"
+ * @param message the `struct SendMessage`
  */
 static void
 handle_client_send (void *cls, struct GNUNET_SERVER_Client *client,
@@ -454,7 +459,7 @@ handle_client_send (void *cls, struct GNUNET_SERVER_Client *client,
   msize -= sizeof (struct SendMessage);
   GNUNET_break (0 == ntohl (sm->reserved));
   c = find_client (client);
-  if (c == NULL)
+  if (NULL == c)
   {
     /* client did not send INIT first! */
     GNUNET_break (0);
@@ -482,10 +487,13 @@ handle_client_send (void *cls, struct GNUNET_SERVER_Client *client,
                                                        &sm->peer,
                                                        tc.car));
   tc.cork = ntohl (sm->cork);
+  tc.priority = (enum GNUNET_CORE_Priority) ntohl (sm->priority);
   GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
-              "Client asked for transmission of %u bytes to `%s' %s\n", msize,
+              "Client asked for transmission of %u bytes to `%s' %s\n",
+              msize,
               GNUNET_i2s (&sm->peer), tc.cork ? "now" : "");
-  GNUNET_SERVER_mst_receive (client_mst, &tc, (const char *) &sm[1], msize,
+  GNUNET_SERVER_mst_receive (client_mst, &tc,
+                             (const char *) &sm[1], msize,
                              GNUNET_YES, GNUNET_NO);
   if (0 !=
       memcmp (&tc.car->target, &GSC_my_identity,
@@ -503,7 +511,7 @@ handle_client_send (void *cls, struct GNUNET_SERVER_Client *client,
  * or other CLIENT (for loopback).
  *
  * @param cls closure
- * @param client reservation request ('struct GSC_ClientActiveRequest')
+ * @param client reservation request (`struct GSC_ClientActiveRequest`)
  * @param message the actual message
  */
 static int
@@ -541,7 +549,8 @@ client_tokenizer_callback (void *cls, void *client,
   else
   {
     GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
-                "Delivering message of type %u to %s\n", ntohs (message->type),
+                "Delivering message of type %u to %s\n",
+                ntohs (message->type),
                 GNUNET_i2s (&car->target));
     GSC_CLIENTS_deliver_message (&car->target, message,
 				 ntohs (message->size),
@@ -549,7 +558,10 @@ client_tokenizer_callback (void *cls, void *client,
     GSC_CLIENTS_deliver_message (&car->target, message,
 				 sizeof (struct GNUNET_MessageHeader),
 				 GNUNET_CORE_OPTION_SEND_HDR_OUTBOUND);
-    GSC_SESSIONS_transmit (car, message, tc->cork);
+    GSC_SESSIONS_transmit (car,
+                           message,
+                           tc->cork,
+                           tc->priority);
   }
   return GNUNET_OK;
 }
@@ -560,7 +572,7 @@ client_tokenizer_callback (void *cls, void *client,
  *
  * @param cls NULL
  * @param key identity of peer for which this is an active request
- * @param value the 'struct GSC_ClientActiveRequest' to free
+ * @param value the `struct GSC_ClientActiveRequest` to free
  * @return #GNUNET_YES (continue iteration)
  */
 static int
