@@ -110,7 +110,7 @@ struct ClientList
 
 
 /**
- * Entry in the DHT routing table for a client's GET request.
+ * Entry in the local forwarding map for a client's GET request.
  */
 struct ClientQueryRecord
 {
@@ -253,7 +253,7 @@ static struct ClientMonitorRecord *monitor_head;
 static struct ClientMonitorRecord *monitor_tail;
 
 /**
- * Hashmap for fast key based lookup, maps keys to 'struct ClientQueryRecord' entries.
+ * Hashmap for fast key based lookup, maps keys to `struct ClientQueryRecord` entries.
  */
 static struct GNUNET_CONTAINER_MultiHashMap *forward_map;
 
@@ -427,7 +427,9 @@ transmit_request (struct ClientQueryRecord *cqr)
                                          GNUNET_CONSTANTS_BLOOMFILTER_K);
   LOG (GNUNET_ERROR_TYPE_DEBUG,
        "Initiating GET for %s, replication %u, already have %u replies\n",
-       GNUNET_h2s(&cqr->key), cqr->replication, cqr->seen_replies_count);
+       GNUNET_h2s (&cqr->key),
+       cqr->replication,
+       cqr->seen_replies_count);
   GDS_NEIGHBOURS_handle_get (cqr->type, cqr->msg_options, cqr->replication,
                              0 /* hop count */ ,
                              &cqr->key, cqr->xquery, cqr->xquery_size, reply_bf,
@@ -511,8 +513,8 @@ handle_dht_local_put (void *cls, struct GNUNET_SERVER_Client *client,
                             ("# PUT requests received from clients"), 1,
                             GNUNET_NO);
   dht_msg = (const struct GNUNET_DHT_ClientPutMessage *) message;
-  LOG_TRAFFIC (GNUNET_ERROR_TYPE_DEBUG, "XDHT CLIENT-PUT %s @ %u\n",
-               GNUNET_h2s (&dht_msg->key), getpid ());
+  LOG_TRAFFIC (GNUNET_ERROR_TYPE_DEBUG, "R5N CLIENT-PUT %s\n",
+               GNUNET_h2s_full (&dht_msg->key));
   /* give to local clients */
   LOG (GNUNET_ERROR_TYPE_DEBUG,
        "Handling local PUT of %u-bytes for query %s\n",
@@ -598,8 +600,8 @@ handle_dht_local_get (void *cls, struct GNUNET_SERVER_Client *client,
        "Received GET request for %s from local client %p, xq: %.*s\n",
        GNUNET_h2s (&get->key), client, xquery_size, xquery);
 
-  LOG_TRAFFIC (GNUNET_ERROR_TYPE_DEBUG, "XDHT CLIENT-GET %s @ %u\n",
-               GNUNET_h2s (&get->key), getpid ());
+  LOG_TRAFFIC (GNUNET_ERROR_TYPE_DEBUG, "R5N CLIENT-GET %s\n",
+               GNUNET_h2s_full (&get->key));
 
 
   cqr = GNUNET_malloc (sizeof (struct ClientQueryRecord) + xquery_size);
@@ -1025,8 +1027,8 @@ forward_reply (void *cls, const struct GNUNET_HashCode * key, void *value)
   unsigned int i;
 
   LOG_TRAFFIC (GNUNET_ERROR_TYPE_DEBUG,
-	       "XDHT CLIENT-RESULT %s\n",
-               GNUNET_h2s (key));
+	       "R5N CLIENT-RESULT %s\n",
+               GNUNET_h2s_full (key));
   if ((record->type != GNUNET_BLOCK_TYPE_ANY) && (record->type != frc->type))
   {
     LOG (GNUNET_ERROR_TYPE_DEBUG,
@@ -1198,6 +1200,7 @@ GDS_CLIENTS_handle_reply (struct GNUNET_TIME_Absolute expiration,
   frc.type = type;
   GNUNET_CONTAINER_multihashmap_get_multiple (forward_map, key, &forward_reply,
                                               &frc);
+
   if (GNUNET_NO == frc.do_copy)
   {
     /* did not match any of the requests, free! */
