@@ -37,7 +37,7 @@
 /**
  * Size of the individual blocks used for file-sharing.
  */
-#define DBLOCK_SIZE (32*1024)
+#define DBLOCK_SIZE (32 * 1024)
 
 /**
  * Blocksize to use when hashing files for indexing (blocksize for IO,
@@ -83,7 +83,7 @@ struct IndexStartMessage
 {
 
   /**
-   * Message type will be GNUNET_MESSAGE_TYPE_FS_INDEX_START.
+   * Message type will be #GNUNET_MESSAGE_TYPE_FS_INDEX_START.
    */
   struct GNUNET_MessageHeader header;
 
@@ -129,7 +129,7 @@ struct IndexInfoMessage
 {
   /**
    * Message type will be
-   * GNUNET_MESSAGE_TYPE_FS_INDEX_LIST_ENTRY.
+   * #GNUNET_MESSAGE_TYPE_FS_INDEX_LIST_ENTRY.
    */
   struct GNUNET_MessageHeader header;
 
@@ -161,8 +161,7 @@ struct UnindexMessage
 {
 
   /**
-   * Message type will be
-   * GNUNET_MESSAGE_TYPE_FS_UNINDEX.
+   * Message type will be #GNUNET_MESSAGE_TYPE_FS_UNINDEX.
    */
   struct GNUNET_MessageHeader header;
 
@@ -206,8 +205,7 @@ struct SearchMessage
 {
 
   /**
-   * Message type will be
-   * GNUNET_MESSAGE_TYPE_FS_START_SEARCH.
+   * Message type will be #GNUNET_MESSAGE_TYPE_FS_START_SEARCH.
    */
   struct GNUNET_MessageHeader header;
 
@@ -240,26 +238,21 @@ struct SearchMessage
    * nevertheless, we should probably not use it for a DHT-lookup
    * or similar blunt actions in order to avoid exposing ourselves).
    * <p>
-   * If the request is for an SBLOCK, this is the identity of the
-   * pseudonym to which the SBLOCK belongs.
-   * <p>
-   * If the request is for a KBLOCK, "target" must be all zeros.
+   * Otherwise, "target" must be all zeros.
    */
-  struct GNUNET_HashCode target;
+  struct GNUNET_PeerIdentity target;
 
   /**
-   * Hash of the keyword (aka query) for KBLOCKs; Hash of
-   * the CHK-encoded block for DBLOCKS and IBLOCKS (aka query)
-   * and hash of the identifier XORed with the target for
-   * SBLOCKS (aka query).
+   * Hash of the public key for UBLOCKs; Hash of
+   * the CHK-encoded block for DBLOCKS and IBLOCKS.
    */
   struct GNUNET_HashCode query;
 
   /* this is followed by the hash codes of already-known
    * results (which should hence be excluded from what
    * the service returns); naturally, this only applies
-   * to queries that can have multiple results, such as
-   * those for KBLOCKS (KSK) and SBLOCKS (SKS) */
+   * to queries that can have multiple results (UBLOCKS).
+   */
 };
 
 
@@ -273,7 +266,7 @@ struct PutMessage
 {
 
   /**
-   * Message type will be GNUNET_MESSAGE_TYPE_FS_PUT.
+   * Message type will be #GNUNET_MESSAGE_TYPE_FS_PUT.
    */
   struct GNUNET_MessageHeader header;
 
@@ -301,7 +294,7 @@ struct ClientPutMessage
 {
 
   /**
-   * Message type will be GNUNET_MESSAGE_TYPE_FS_PUT.
+   * Message type will be #GNUNET_MESSAGE_TYPE_FS_PUT.
    */
   struct GNUNET_MessageHeader header;
 
