@@ -268,7 +268,7 @@ struct GSF_ConnectedPeer
 
   /**
    * Set to 1 if we're currently in the process of calling
-   * 'GNUNET_CORE_notify_transmit_ready' (so while cth is
+   * #GNUNET_CORE_notify_transmit_ready() (so while @e cth is
    * NULL, we should not call notify_transmit_ready for this
    * handle right now).
    */
@@ -439,7 +439,8 @@ schedule_transmission (struct GSF_PeerTransmitHandle *pth)
   GNUNET_assert (NULL == cp->cth);
   cp->cth_in_progress++;
   cp->cth =
-    GNUNET_CORE_notify_transmit_ready (GSF_core, GNUNET_YES, pth->priority,
+    GNUNET_CORE_notify_transmit_ready (GSF_core, GNUNET_YES,
+                                       GNUNET_CORE_PRIO_BACKGROUND,
 				       GNUNET_TIME_absolute_get_remaining
 				       (pth->timeout), &target, pth->size,
 				       &peer_transmit_ready_cb, cp);
@@ -558,7 +559,8 @@ ats_reserve_callback (void *cls, const struct GNUNET_PeerIdentity *peer,
     /* reservation success, try transmission now! */
     cp->cth_in_progress++;
     cp->cth =
-        GNUNET_CORE_notify_transmit_ready (GSF_core, GNUNET_YES, pth->priority,
+        GNUNET_CORE_notify_transmit_ready (GSF_core, GNUNET_YES,
+                                           GNUNET_CORE_PRIO_BACKGROUND,
                                            GNUNET_TIME_absolute_get_remaining
                                            (pth->timeout), peer, pth->size,
                                            &peer_transmit_ready_cb, cp);
