@@ -139,28 +139,30 @@ progress_cb (void *cls, const struct GNUNET_FS_ProgressInfo *event)
 		(unsigned long long) event->value.publish.specifics.
 		progress.offset);
     break;
+  case GNUNET_FS_STATUS_PUBLISH_PROGRESS_DIRECTORY:
+    break;
   case GNUNET_FS_STATUS_PUBLISH_COMPLETED:
     fprintf (stdout,
 	     "Publishing complete, %llu kb/s.\n",
-	     (unsigned long long) (FILESIZE * 1000LL /
+	     (unsigned long long) (FILESIZE * 1000000LL /
 				   (1 +
 				    GNUNET_TIME_absolute_get_duration
-				    (start).rel_value) / 1024LL));
-    GAUGER ("FS", 
-	    (GNUNET_YES == indexed) 
+				    (start).rel_value_us) / 1024LL));
+    GAUGER ("FS",
+	    (GNUNET_YES == indexed)
 	    ? "Publishing speed (indexing)"
 	     : "Publishing speed (insertion)",
-	    (unsigned long long) (FILESIZE * 1000LL /
+	    (unsigned long long) (FILESIZE * 1000000LL /
 				  (1 +
 				   GNUNET_TIME_absolute_get_duration
-				   (start).rel_value) / 1024LL), "kb/s");
+				   (start).rel_value_us) / 1024LL), "kb/s");
     fn = GNUNET_DISK_mktemp ("gnunet-download-test-dst");
     start = GNUNET_TIME_absolute_get ();
     download =
         GNUNET_FS_download_start (fs,
                                   event->value.publish.specifics.
                                   completed.chk_uri, NULL, fn, NULL, 0,
-                                  FILESIZE, anonymity_level, 
+                                  FILESIZE, anonymity_level,
 				  GNUNET_FS_DOWNLOAD_OPTION_NONE,
                                   "download", NULL);
     GNUNET_assert (download != NULL);
@@ -168,18 +170,18 @@ progress_cb (void *cls, const struct GNUNET_FS_ProgressInfo *event)
   case GNUNET_FS_STATUS_DOWNLOAD_COMPLETED:
     fprintf (stdout,
 	     "Download complete,  %llu kb/s.\n",
-	     (unsigned long long) (FILESIZE * 1000LL /
+	     (unsigned long long) (FILESIZE * 1000000LL /
 				   (1 +
 				    GNUNET_TIME_absolute_get_duration
-				    (start).rel_value) / 1024LL));
+				    (start).rel_value_us) / 1024LL));
     GAUGER ("FS",
-	    (GNUNET_YES == indexed) 
-	    ? "Local download speed (indexed)"  
+	    (GNUNET_YES == indexed)
+	    ? "Local download speed (indexed)"
 	    : "Local download speed (inserted)",
-            (unsigned long long) (FILESIZE * 1000LL /
+            (unsigned long long) (FILESIZE * 1000000LL /
                                   (1 +
                                    GNUNET_TIME_absolute_get_duration
-                                   (start).rel_value) / 1024LL), "kb/s");
+                                   (start).rel_value_us) / 1024LL), "kb/s");
     GNUNET_SCHEDULER_add_now (&abort_download_task, NULL);
     break;
   case GNUNET_FS_STATUS_DOWNLOAD_PROGRESS:
@@ -329,10 +331,10 @@ main (int argc, char *argv[])
     binary_name = "test-fs-download-indexed";
     config_name = "test_fs_download_indexed.conf";
   }
-  if (NULL != strstr (argv[0], "stream"))
+  if (NULL != strstr (argv[0], "mesh"))
   {
-    binary_name = "test-fs-download-stream";
-    config_name = "test_fs_download_stream.conf";
+    binary_name = "test-fs-download-mesh";
+    config_name = "test_fs_download_mesh.conf";
   }
   if (0 != GNUNET_TESTING_peer_run (binary_name,
 				    config_name,
