@@ -40,6 +40,11 @@ static const struct GNUNET_CONFIGURATION_Handle *cfg;
  */
 static struct GNUNET_GNS_Handle *gns;
 
+/**
+ * Desired timeout for the lookup (default is no timeout).
+ */
+static struct GNUNET_TIME_Relative timeout;
+
 /**
  * GNS name to lookup. (-u option)
  */
@@ -60,6 +65,11 @@ static char *zone_ego_name;
  */
 static char *public_key;
 
+/**
+ * Set to GNUNET_GNS_LO_LOCAL_MASTER if we are looking up in the master zone.
+ */
+static enum GNUNET_GNS_LocalOptions local_options;
+
 /**
  * raw output
  */
@@ -206,7 +216,7 @@ lookup_with_keys (const struct GNUNET_CRYPTO_EcdsaPublicKey *pkey,
 					lookup_name,
 					pkey,
 					rtype,
-					GNUNET_NO, /* Use DHT */
+					local_options,
 					shorten_key,
 					&process_lookup_result,
 					lookup_name);
@@ -328,6 +338,7 @@ identity_master_cb (void *cls,
 		    const char *name)
 {
   struct GNUNET_CRYPTO_EcdsaPublicKey pkey;
+  const char *dot;
 
   id_op = NULL;
   if (NULL == ego)
@@ -338,6 +349,16 @@ identity_master_cb (void *cls,
     return;
   }
   GNUNET_IDENTITY_ego_get_public_key (ego, &pkey);
+  /* main name is our own master zone, do no look for that in the DHT */
+  local_options = GNUNET_GNS_LO_LOCAL_MASTER;
+
+  /* if the name is of the form 'label.gnu', never go to the DHT */
+  dot = NULL;
+  if (NULL != lookup_name)
+    dot = strchr (lookup_name, '.');
+  if ( (NULL != dot) &&
+       (0 == strcasecmp (dot, ".gnu")) )
+    local_options = GNUNET_GNS_LO_NO_DHT;
   lookup_with_public_key (&pkey);
 }
 
@@ -365,7 +386,7 @@ run (void *cls, char *const *args, const char *cfgfile,
 	     _("Failed to connect to GNS\n"));
     return;
   }
-  GNUNET_SCHEDULER_add_delayed (GNUNET_TIME_UNIT_FOREVER_REL,
+  GNUNET_SCHEDULER_add_delayed (timeout,
 				&do_shutdown, NULL);
   if (NULL != public_key)
   {
@@ -430,6 +451,9 @@ main (int argc, char *const *argv)
     {'t', "type", "TYPE",
       gettext_noop ("Specify the type of the record to lookup"), 1,
       &GNUNET_GETOPT_set_string, &lookup_type},
+    { 'T', "timeout", "DELAY",
+      gettext_noop ("Specify timeout for the lookup"), 1,
+      &GNUNET_GETOPT_set_relative_time, &timeout },
     {'r', "raw", NULL,
       gettext_noop ("No unneeded output"), 0,
       &GNUNET_GETOPT_set_one, &raw},
@@ -443,6 +467,7 @@ main (int argc, char *const *argv)
   };
   int ret;
 
+  timeout = GNUNET_TIME_UNIT_FOREVER_REL;
   if (GNUNET_OK != GNUNET_STRINGS_get_utf8_args (argc, argv, &argc, &argv))
     return 2;
 
