@@ -27,23 +27,22 @@
 
 #include "gnunet_util_lib.h"
 
+
 /**
- * Initialize dns interceptor
+ * Initialize DNS interceptor
  *
- * @param zone the zone
- * @param key the private key of the local zone
+ * @param gnu_zone the zone we start lookups in
  * @param c the configuration
  * @return GNUNET_YES on success GNUNET_SYSERR on error
  */
 int
-gns_interceptor_init (struct GNUNET_CRYPTO_ShortHashCode zone,
-		      struct GNUNET_CRYPTO_RsaPrivateKey *key,
+GNS_interceptor_init (const struct GNUNET_CRYPTO_EcdsaPublicKey *gnu_zone,
 		      const struct GNUNET_CONFIGURATION_Handle *c);
 
 /**
  * Stops the interceptor
  */
 void
-gns_interceptor_stop (void);
+GNS_interceptor_done (void);
 
 #endif
