@@ -28,6 +28,36 @@
 #include "gnunet_util_lib.h"
 
 
+/**
+ * Check if an address has a local option set
+ *
+ * @param address the address to check
+ * @param option the respective option to check for
+ * @return GNUNET_YES or GNUNET_NO
+ */
+int
+GNUNET_HELLO_address_check_option (const struct GNUNET_HELLO_Address * address,
+    enum GNUNET_HELLO_AddressInfo option)
+{
+  if (option == (address->local_info & option))
+    return GNUNET_YES;
+  return GNUNET_NO;
+}
+
+/**
+ * Get the size of an address struct.
+ *
+ * @param address address
+ * @return the size
+ */
+size_t
+GNUNET_HELLO_address_get_size (const struct GNUNET_HELLO_Address * address)
+{
+  return sizeof (struct GNUNET_HELLO_Address) + address->address_length +
+        strlen (address->transport_name) + 1;
+}
+
+
 /**
  * Allocate an address struct.
  *
@@ -35,12 +65,14 @@
  * @param transport_name plugin name
  * @param address binary address
  * @param address_length number of bytes in 'address'
+ * @param local_info additional local information for the address
  * @return the address struct
  */
 struct GNUNET_HELLO_Address *
 GNUNET_HELLO_address_allocate (const struct GNUNET_PeerIdentity *peer,
                                const char *transport_name, const void *address,
-                               size_t address_length)
+                               size_t address_length,
+                               enum GNUNET_HELLO_AddressInfo local_info)
 {
   struct GNUNET_HELLO_Address *addr;
   size_t slen;
@@ -58,25 +90,12 @@ GNUNET_HELLO_address_allocate (const struct GNUNET_PeerIdentity *peer,
   memcpy (end, address, address_length);
   addr->address_length = address_length;
   addr->transport_name = &end[address_length];
+  addr->local_info = local_info;
   memcpy (&end[address_length], transport_name, slen);
   return addr;
 }
 
 
-/**
- * Get the size of an address struct.
- *
- * @param address address
- * @return the size
- */
-size_t
-GNUNET_HELLO_address_get_size (const struct GNUNET_HELLO_Address * address)
-{
-  return sizeof (struct GNUNET_HELLO_Address) + address->address_length +
-      strlen (address->transport_name) + 1;
-}
-
-
 /**
  * Copy an address struct.
  *
@@ -88,7 +107,8 @@ GNUNET_HELLO_address_copy (const struct GNUNET_HELLO_Address *address)
 {
   return GNUNET_HELLO_address_allocate (&address->peer, address->transport_name,
                                         address->address,
-                                        address->address_length);
+                                        address->address_length,
+                                        address->local_info);
 }
 
 
