@@ -381,7 +381,8 @@ connect_handler (void *cls, const struct GNUNET_PeerIdentity *peer)
               "Asked core to transmit advertisement message with a size of %u bytes to peer `%s'\n",
               size, GNUNET_i2s (peer));
   if (NULL ==
-      GNUNET_CORE_notify_transmit_ready (core, GNUNET_YES, 0,
+      GNUNET_CORE_notify_transmit_ready (core, GNUNET_YES,
+                                         GNUNET_CORE_PRIO_BEST_EFFORT,
                                          GNUNET_ADV_TIMEOUT, peer, size,
                                          &adv_transmit_ready, NULL))
   {
