@@ -4,7 +4,7 @@
 
      GNUnet is free software; you can redistribute it and/or modify
      it under the terms of the GNU General Public License as published
-     by the Free Software Foundation; either version 2, or (at your
+     by the Free Software Foundation; either version 3, or (at your
      option) any later version.
 
      GNUnet is distributed in the hope that it will be useful, but
@@ -25,12 +25,6 @@
 #ifndef GNUNET_DISK_LIB_H
 #define GNUNET_DISK_LIB_H
 
-#if WINDOWS
-#define OFF_T uint64_t
-#else
-#define OFF_T off_t
-#endif
-
 /**
  * Handle used to manage a pipe.
  */
@@ -43,8 +37,8 @@ enum GNUNET_FILE_Type
 {
   /**
    * Handle represents a file.
-   */ 
-  GNUNET_DISK_HANLDE_TYPE_FILE, 
+   */
+  GNUNET_DISK_HANLDE_TYPE_FILE,
 
   /**
    * Handle represents a pipe.
@@ -157,7 +151,7 @@ enum GNUNET_DISK_MapType
    * Read-only memory map.
    */
   GNUNET_DISK_MAP_TYPE_READ = 1,
-  
+
   /**
    * Write-able memory map.
    */
@@ -267,22 +261,11 @@ enum GNUNET_DISK_PipeEnd
 };
 
 
-/**
- * Get the number of blocks that are left on the partition that
- * contains the given file (for normal users).
- *
- * @param part a file on the partition to check
- * @return -1 on errors, otherwise the number of free blocks
- */
-long
-GNUNET_DISK_get_blocks_available (const char *part);
-
-
 /**
  * Checks whether a handle is invalid
  *
  * @param h handle to check
- * @return GNUNET_YES if invalid, GNUNET_NO if valid
+ * @return #GNUNET_YES if invalid, #GNUNET_NO if valid
  */
 int
 GNUNET_DISK_handle_invalid (const struct GNUNET_DISK_FileHandle *h);
@@ -293,7 +276,7 @@ GNUNET_DISK_handle_invalid (const struct GNUNET_DISK_FileHandle *h);
  * (of a file that exists and that is not a directory).
  *
  * @param fil filename to check
- * @return GNUNET_YES if yes, GNUNET_NO if not a file, GNUNET_SYSERR if something
+ * @return #GNUNET_YES if yes, #GNUNET_NO if not a file, #GNUNET_SYSERR if something
  * else (will print an error message in that case, too).
  */
 int
@@ -318,8 +301,8 @@ GNUNET_DISK_file_backup (const char *fil);
  * @param whence specification to which position the offset parameter relates to
  * @return the new position on success, GNUNET_SYSERR otherwise
  */
-OFF_T
-GNUNET_DISK_file_seek (const struct GNUNET_DISK_FileHandle *h, OFF_T offset,
+off_t
+GNUNET_DISK_file_seek (const struct GNUNET_DISK_FileHandle *h, off_t offset,
                        enum GNUNET_DISK_Seek whence);
 
 
@@ -331,15 +314,16 @@ GNUNET_DISK_file_seek (const struct GNUNET_DISK_FileHandle *h, OFF_T offset,
  * @param size set to the size of the file (or,
  *             in the case of directories, the sum
  *             of all sizes of files in the directory)
- * @param includeSymLinks should symbolic links be
+ * @param include_symbolic_links should symbolic links be
  *        included?
- * @param singleFileMode GNUNET_YES to only get size of one file
- *        and return GNUNET_SYSERR for directories.
- * @return GNUNET_SYSERR on error, GNUNET_OK on success
+ * @param single_file_mode #GNUNET_YES to only get size of one file
+ *        and return #GNUNET_SYSERR for directories.
+ * @return #GNUNET_SYSERR on error, #GNUNET_OK on success
  */
 int
-GNUNET_DISK_file_size (const char *filename, uint64_t * size,
-                       int includeSymLinks, int singleFileMode);
+GNUNET_DISK_file_size (const char *filename, uint64_t *size,
+                       int include_symbolic_links,
+                       int single_file_mode);
 
 
 /**
@@ -358,8 +342,9 @@ GNUNET_DISK_file_size (const char *filename, uint64_t * size,
  * @return GNUNET_OK on success
  */
 int
-GNUNET_DISK_file_get_identifiers (const char *filename, uint64_t * dev,
-                                  uint64_t * ino);
+GNUNET_DISK_file_get_identifiers (const char *filename,
+                                  uint64_t *dev,
+                                  uint64_t *ino);
 
 
 /**
@@ -398,12 +383,13 @@ GNUNET_DISK_mkdtemp (const char *t);
  * @param fn file name to be opened
  * @param flags opening flags, a combination of GNUNET_DISK_OPEN_xxx bit flags
  * @param perm permissions for the newly created file, use
- *             GNUNET_DISK_PERM_NONE if a file could not be created by this
+ *             #GNUNET_DISK_PERM_NONE if a file could not be created by this
  *             call (because of flags)
  * @return IO handle on success, NULL on error
  */
 struct GNUNET_DISK_FileHandle *
-GNUNET_DISK_file_open (const char *fn, enum GNUNET_DISK_OpenFlags flags,
+GNUNET_DISK_file_open (const char *fn,
+                       enum GNUNET_DISK_OpenFlags flags,
                        enum GNUNET_DISK_AccessPermissions perm);
 
 
@@ -412,44 +398,49 @@ GNUNET_DISK_file_open (const char *fn, enum GNUNET_DISK_OpenFlags flags,
  *
  * @param fh open file handle
  * @param size where to write size of the file
- * @return GNUNET_OK on success, GNUNET_SYSERR on error
+ * @return #GNUNET_OK on success, #GNUNET_SYSERR on error
  */
 int
 GNUNET_DISK_file_handle_size (struct GNUNET_DISK_FileHandle *fh,
-			      OFF_T *size);
+			      off_t *size);
 
 
 /**
  * Creates an interprocess channel
  *
- * @param blocking_read creates an asynchronous pipe for reading if set to GNUNET_NO
- * @param blocking_write creates an asynchronous pipe for writing if set to GNUNET_NO
+ * @param blocking_read creates an asynchronous pipe for reading if set to #GNUNET_NO
+ * @param blocking_write creates an asynchronous pipe for writing if set to #GNUNET_NO
  * @param inherit_read 1 to make read handle inheritable, 0 otherwise (NT only)
  * @param inherit_write 1 to make write handle inheritable, 0 otherwise (NT only)
  * @return handle to the new pipe, NULL on error
  */
 struct GNUNET_DISK_PipeHandle *
-GNUNET_DISK_pipe (int blocking_read, int blocking_write, int inherit_read, int inherit_write);
+GNUNET_DISK_pipe (int blocking_read,
+                  int blocking_write,
+                  int inherit_read,
+                  int inherit_write);
 
 
 /**
  * Creates a pipe object from a couple of file descriptors.
  * Useful for wrapping existing pipe FDs.
  *
- * @param blocking_read creates an asynchronous pipe for reading if set to GNUNET_NO
- * @param blocking_write creates an asynchronous pipe for writing if set to GNUNET_NO
+ * @param blocking_read creates an asynchronous pipe for reading if set to #GNUNET_NO
+ * @param blocking_write creates an asynchronous pipe for writing if set to #GNUNET_NO
  * @param fd an array of two fd values. One of them may be -1 for read-only or write-only pipes
  *
  * @return handle to the new pipe, NULL on error
  */
 struct GNUNET_DISK_PipeHandle *
-GNUNET_DISK_pipe_from_fd (int blocking_read, int blocking_write, int fd[2]);
+GNUNET_DISK_pipe_from_fd (int blocking_read,
+                          int blocking_write,
+                          int fd[2]);
 
 
 /**
  * Closes an interprocess channel
  * @param p pipe
- * @return GNUNET_OK on success, GNUNET_SYSERR otherwise
+ * @return #GNUNET_OK on success, #GNUNET_SYSERR otherwise
  */
 int
 GNUNET_DISK_pipe_close (struct GNUNET_DISK_PipeHandle *p);
@@ -460,17 +451,34 @@ GNUNET_DISK_pipe_close (struct GNUNET_DISK_PipeHandle *p);
  *
  * @param p pipe to close end of
  * @param end which end of the pipe to close
- * @return GNUNET_OK on success, GNUNET_SYSERR otherwise
+ * @return #GNUNET_OK on success, #GNUNET_SYSERR otherwise
  */
 int
 GNUNET_DISK_pipe_close_end (struct GNUNET_DISK_PipeHandle *p,
                             enum GNUNET_DISK_PipeEnd end);
 
+
+/**
+ * Detaches one of the ends from the pipe.
+ * Detached end is a fully-functional FileHandle, it will
+ * not be affected by anything you do with the pipe afterwards.
+ * Each end of a pipe can only be detched from it once (i.e.
+ * it is not duplicated).
+ *
+ * @param p pipe to detach an end from
+ * @param end which end of the pipe to detach
+ * @return Detached end on success, NULL on failure
+ * (or if that end is not present or is closed).
+ */
+struct GNUNET_DISK_FileHandle *
+GNUNET_DISK_pipe_detach_end (struct GNUNET_DISK_PipeHandle *p,
+                             enum GNUNET_DISK_PipeEnd end);
+
 /**
  * Close an open file.
  *
  * @param h file handle
- * @return GNUNET_OK on success, GNUNET_SYSERR otherwise
+ * @return #GNUNET_OK on success, #GNUNET_SYSERR otherwise
  */
 int
 GNUNET_DISK_file_close (struct GNUNET_DISK_FileHandle *h);
@@ -488,6 +496,44 @@ GNUNET_DISK_pipe_handle (const struct GNUNET_DISK_PipeHandle *p,
                          enum GNUNET_DISK_PipeEnd n);
 
 
+#if WINDOWS
+/**
+ * Get a GNUnet file handle from a W32 handle (W32-only).
+ * Do not call on non-W32 platforms (returns NULL).
+ *
+ * @param handle native handle
+ * @return GNUnet file handle corresponding to the W32 handle
+ */
+struct GNUNET_DISK_FileHandle *
+GNUNET_DISK_get_handle_from_w32_handle (HANDLE osfh);
+#else
+
+/**
+ * Update POSIX permissions mask of a file on disk.  If both argumets
+ * are #GNUNET_NO, the file is made world-read-write-executable (777).
+ *
+ * @param fn name of the file to update
+ * @param require_uid_match #GNUNET_YES means 700
+ * @param require_gid_match #GNUNET_YES means 770 unless @a require_uid_match is set
+ */
+void
+GNUNET_DISK_fix_permissions (const char *fn,
+                             int require_uid_match,
+                             int require_gid_match);
+
+#endif
+
+
+/**
+ * Get a handle from a native integer FD.
+ *
+ * @param fno native integer file descriptor
+ * @return file handle corresponding to the descriptor
+ */
+struct GNUNET_DISK_FileHandle *
+GNUNET_DISK_get_handle_from_int_fd (int fno);
+
+
 /**
  * Get a handle from a native FD.
  *
@@ -500,13 +546,15 @@ GNUNET_DISK_get_handle_from_native (FILE *fd);
 
 /**
  * Read the contents of a binary file into a buffer.
+ *
  * @param h handle to an open file
  * @param result the buffer to write the result to
  * @param len the maximum number of bytes to read
- * @return the number of bytes read on success, GNUNET_SYSERR on failure
+ * @return the number of bytes read on success, #GNUNET_SYSERR on failure
  */
 ssize_t
-GNUNET_DISK_file_read (const struct GNUNET_DISK_FileHandle *h, void *result,
+GNUNET_DISK_file_read (const struct GNUNET_DISK_FileHandle *h,
+                       void *result,
                        size_t len);
 
 
@@ -518,11 +566,12 @@ GNUNET_DISK_file_read (const struct GNUNET_DISK_FileHandle *h, void *result,
  * @param h handle to an open file
  * @param result the buffer to write the result to
  * @param len the maximum number of bytes to read
- * @return the number of bytes read on success, GNUNET_SYSERR on failure
+ * @return the number of bytes read on success, #GNUNET_SYSERR on failure
  */
 ssize_t
 GNUNET_DISK_file_read_non_blocking (const struct GNUNET_DISK_FileHandle * h,
-				    void *result, size_t len);
+				    void *result,
+                                    size_t len);
 
 
 /**
@@ -531,10 +580,12 @@ GNUNET_DISK_file_read_non_blocking (const struct GNUNET_DISK_FileHandle * h,
  * @param fn file name
  * @param result the buffer to write the result to
  * @param len the maximum number of bytes to read
- * @return number of bytes read, GNUNET_SYSERR on failure
+ * @return number of bytes read, #GNUNET_SYSERR on failure
  */
 ssize_t
-GNUNET_DISK_fn_read (const char *fn, void *result, size_t len);
+GNUNET_DISK_fn_read (const char *fn,
+                     void *result,
+                     size_t len);
 
 
 /**
@@ -543,23 +594,27 @@ GNUNET_DISK_fn_read (const char *fn, void *result, size_t len);
  * @param h handle to open file
  * @param buffer the data to write
  * @param n number of bytes to write
- * @return number of bytes written on success, GNUNET_SYSERR on error
+ * @return number of bytes written on success, #GNUNET_SYSERR on error
  */
 ssize_t
 GNUNET_DISK_file_write (const struct GNUNET_DISK_FileHandle *h,
-                        const void *buffer, size_t n);
+                        const void *buffer,
+                        size_t n);
 
 
 /**
  * Write a buffer to a file, blocking, if necessary.
+ *
  * @param h handle to open file
  * @param buffer the data to write
  * @param n number of bytes to write
- * @return number of bytes written on success, GNUNET_SYSERR on error
+ * @return number of bytes written on success, #GNUNET_SYSERR on error
  */
 ssize_t
-GNUNET_DISK_file_write_blocking (const struct GNUNET_DISK_FileHandle * h,
-    const void *buffer, size_t n);
+GNUNET_DISK_file_write_blocking (const struct GNUNET_DISK_FileHandle *h,
+				 const void *buffer,
+				 size_t n);
+
 
 /**
  * Write a buffer to a file.  If the file is longer than
@@ -569,10 +624,12 @@ GNUNET_DISK_file_write_blocking (const struct GNUNET_DISK_FileHandle * h,
  * @param buffer the data to write
  * @param n number of bytes to write
  * @param mode file permissions
- * @return number of bytes written on success, GNUNET_SYSERR on error
+ * @return number of bytes written on success, #GNUNET_SYSERR on error
  */
 ssize_t
-GNUNET_DISK_fn_write (const char *fn, const void *buffer, size_t n,
+GNUNET_DISK_fn_write (const char *fn,
+                      const void *buffer,
+                      size_t n,
                       enum GNUNET_DISK_AccessPermissions mode);
 
 
@@ -581,7 +638,7 @@ GNUNET_DISK_fn_write (const char *fn, const void *buffer, size_t n,
  *
  * @param src file to copy
  * @param dst destination file name
- * @return GNUNET_OK on success, GNUNET_SYSERR on error
+ * @return #GNUNET_OK on success, #GNUNET_SYSERR on error
  */
 int
 GNUNET_DISK_file_copy (const char *src, const char *dst);
@@ -590,13 +647,13 @@ GNUNET_DISK_file_copy (const char *src, const char *dst);
 /**
  * Scan a directory for files.
  *
- * @param dirName the name of the directory
+ * @param dir_name the name of the directory
  * @param callback the method to call for each file
- * @param callback_cls closure for callback
+ * @param callback_cls closure for @a callback
  * @return the number of files found, -1 on error
  */
 int
-GNUNET_DISK_directory_scan (const char *dirName,
+GNUNET_DISK_directory_scan (const char *dir_name,
                             GNUNET_FileNameCallback callback,
                             void *callback_cls);
 
@@ -611,16 +668,14 @@ struct GNUNET_DISK_DirectoryIterator;
  * Function called to iterate over a directory.
  *
  * @param cls closure
- * @param di argument to pass to "GNUNET_DISK_directory_iterator_next" to
+ * @param di argument to pass to #GNUNET_DISK_directory_iterator_next to
  *           get called on the next entry (or finish cleanly);
  *           NULL on error (will be the last call in that case)
  * @param filename complete filename (absolute path)
  * @param dirname directory name (absolute path)
  */
 typedef void (*GNUNET_DISK_DirectoryIteratorCallback) (void *cls,
-                                                       struct
-                                                       GNUNET_DISK_DirectoryIterator
-                                                       * di,
+                                                       struct GNUNET_DISK_DirectoryIterator *di,
                                                        const char *filename,
                                                        const char *dirname);
 
@@ -631,10 +686,10 @@ typedef void (*GNUNET_DISK_DirectoryIteratorCallback) (void *cls,
  * filename in the directory (if there is one).
  *
  * @param iter opaque handle for the iterator
- * @param can set to GNUNET_YES to terminate the iteration early
- * @return GNUNET_YES if iteration will continue,
- *         GNUNET_NO if this was the last entry (and iteration is complete),
- *         GNUNET_SYSERR if "can" was YES
+ * @param can set to #GNUNET_YES to terminate the iteration early
+ * @return #GNUNET_YES if iteration will continue,
+ *         #GNUNET_NO if this was the last entry (and iteration is complete),
+ *         #GNUNET_SYSERR if @a can was #GNUNET_YES
  */
 int
 GNUNET_DISK_directory_iterator_next (struct GNUNET_DISK_DirectoryIterator *iter,
@@ -648,15 +703,15 @@ GNUNET_DISK_directory_iterator_next (struct GNUNET_DISK_DirectoryIterator *iter,
  * may provide a simpler API.
  *
  * @param prio priority to use
- * @param dirName the name of the directory
+ * @param dir_name the name of the directory
  * @param callback the method to call for each file
- * @param callback_cls closure for callback
- * @return GNUNET_YES if directory is not empty and 'callback'
- *         will be called later, GNUNET_NO otherwise, GNUNET_SYSERR on error.
+ * @param callback_cls closure for @a callback
+ * @return #GNUNET_YES if directory is not empty and @a callback
+ *         will be called later, #GNUNET_NO otherwise, #GNUNET_SYSERR on error.
  */
 int
 GNUNET_DISK_directory_iterator_start (enum GNUNET_SCHEDULER_Priority prio,
-                                      const char *dirName,
+                                      const char *dir_name,
                                       GNUNET_DISK_DirectoryIteratorCallback
                                       callback, void *callback_cls);
 
@@ -666,24 +721,24 @@ GNUNET_DISK_directory_iterator_start (enum GNUNET_SCHEDULER_Priority prio,
  * a file.
  *
  * @param filename name of a file in the directory
- * @returns GNUNET_OK on success, GNUNET_SYSERR on failure,
- *          GNUNET_NO if directory exists but is not writeable
+ * @returns #GNUNET_OK on success, #GNUNET_SYSERR on failure,
+ *          #GNUNET_NO if directory exists but is not writeable
  */
 int
 GNUNET_DISK_directory_create_for_file (const char *filename);
 
 
 /**
- * Test if "fil" is a directory and listable. Optionally, also check if the
+ * Test if @a fil is a directory and listable. Optionally, also check if the
  * directory is readable.  Will not print an error message if the directory does
- * not exist.  Will log errors if GNUNET_SYSERR is returned (i.e., a file exists
+ * not exist.  Will log errors if #GNUNET_SYSERR is returned (i.e., a file exists
  * with the same name).
  *
  * @param fil filename to test
- * @param is_readable GNUNET_YES to additionally check if "fil" is readable;
- *          GNUNET_NO to disable this check
- * @return GNUNET_YES if yes, GNUNET_NO if not; GNUNET_SYSERR if it
- *           does not exist or stat'ed
+ * @param is_readable #GNUNET_YES to additionally check if @a fil is readable;
+ *          #GNUNET_NO to disable this check
+ * @return #GNUNET_YES if yes, #GNUNET_NO if not; #GNUNET_SYSERR if it
+ *           does not exist or `stat`ed
  */
 int
 GNUNET_DISK_directory_test (const char *fil, int is_readable);
@@ -694,7 +749,7 @@ GNUNET_DISK_directory_test (const char *fil, int is_readable);
  * caution.
  *
  * @param filename the file to remove
- * @return GNUNET_OK on success, GNUNET_SYSERR on error
+ * @return #GNUNET_OK on success, #GNUNET_SYSERR on error
  */
 int
 GNUNET_DISK_directory_remove (const char *filename);
@@ -704,7 +759,7 @@ GNUNET_DISK_directory_remove (const char *filename);
  * Implementation of "mkdir -p"
  *
  * @param dir the directory to create
- * @returns GNUNET_SYSERR on failure, GNUNET_OK otherwise
+ * @returns #GNUNET_SYSERR on failure, #GNUNET_OK otherwise
  */
 int
 GNUNET_DISK_directory_create (const char *dir);
@@ -714,26 +769,29 @@ GNUNET_DISK_directory_create (const char *dir);
  * Lock a part of a file.
  *
  * @param fh file handle
- * @param lockStart absolute position from where to lock
- * @param lockEnd absolute position until where to lock
- * @param excl GNUNET_YES for an exclusive lock
- * @return GNUNET_OK on success, GNUNET_SYSERR on error
+ * @param lock_start absolute position from where to lock
+ * @param lock_end absolute position until where to lock
+ * @param excl #GNUNET_YES for an exclusive lock
+ * @return #GNUNET_OK on success, #GNUNET_SYSERR on error
  */
 int
-GNUNET_DISK_file_lock (struct GNUNET_DISK_FileHandle *fh, OFF_T lockStart,
-                       OFF_T lockEnd, int excl);
+GNUNET_DISK_file_lock (struct GNUNET_DISK_FileHandle *fh,
+                       off_t lock_start,
+                       off_t lock_end, int excl);
 
 
 /**
- * Unlock a part of a file
+ * Unlock a part of a file.
+ *
  * @param fh file handle
- * @param unlockStart absolute position from where to unlock
- * @param unlockEnd absolute position until where to unlock
- * @return GNUNET_OK on success, GNUNET_SYSERR on error
+ * @param unlock_start absolute position from where to unlock
+ * @param unlock_end absolute position until where to unlock
+ * @return #GNUNET_OK on success, #GNUNET_SYSERR on error
  */
 int
-GNUNET_DISK_file_unlock (struct GNUNET_DISK_FileHandle *fh, OFF_T unlockStart,
-                         OFF_T unlockEnd);
+GNUNET_DISK_file_unlock (struct GNUNET_DISK_FileHandle *fh,
+                         off_t unlock_start,
+                         off_t unlock_end);
 
 
 /**
@@ -748,31 +806,12 @@ GNUNET_DISK_filename_canonicalize (char *fn);
  * @brief Change owner of a file
  * @param filename file to change
  * @param user new owner of the file
- * @return GNUNET_OK on success, GNUNET_SYSERR on failure
+ * @return #GNUNET_OK on success, #GNUNET_SYSERR on failure
  */
 int
 GNUNET_DISK_file_change_owner (const char *filename, const char *user);
 
 
-/**
- * Construct full path to a file inside of the private
- * directory used by GNUnet.  Also creates the corresponding
- * directory.  If the resulting name is supposed to be
- * a directory, end the last argument in '/' (or pass
- * DIR_SEPARATOR_STR as the last argument before NULL).
- *
- * @param cfg configuration to use
- * @param serviceName name of the service asking
- * @param ... is NULL-terminated list of
- *                path components to append to the
- *                private directory name.
- * @return the constructed filename
- */
-char *
-GNUNET_DISK_get_home_filename (const struct GNUNET_CONFIGURATION_Handle *cfg,
-                               const char *serviceName, ...);
-
-
 /**
  * Opaque handle for a memory-mapping operation.
  */
@@ -795,7 +834,7 @@ GNUNET_DISK_file_map (const struct GNUNET_DISK_FileHandle *h,
  * Unmap a file
  *
  * @param h mapping handle
- * @return GNUNET_OK on success, GNUNET_SYSERR otherwise
+ * @return #GNUNET_OK on success, #GNUNET_SYSERR otherwise
  */
 int
 GNUNET_DISK_file_unmap (struct GNUNET_DISK_MapHandle *h);
@@ -803,7 +842,7 @@ GNUNET_DISK_file_unmap (struct GNUNET_DISK_MapHandle *h);
 /**
  * Write file changes to disk
  * @param h handle to an open file
- * @return GNUNET_OK on success, GNUNET_SYSERR otherwise
+ * @return #GNUNET_OK on success, #GNUNET_SYSERR otherwise
  */
 int
 GNUNET_DISK_file_sync (const struct GNUNET_DISK_FileHandle *h);
