@@ -22,6 +22,7 @@
 #include "platform.h"
 #include "gnunet_util_lib.h"
 
+#include "gnunet_transport_service.h"
 #include "gnunet_core_service.h"
 #include "gnunet_statistics_service.h"
 
@@ -159,6 +160,11 @@ struct MeshPeer
    * How many messages are in the queue to this peer.
    */
   unsigned int queue_n;
+
+  /**
+   * Hello message.
+   */
+  struct GNUNET_HELLO_Message* hello;
 };
 
 
@@ -201,6 +207,10 @@ static unsigned long long drop_percent;
  */
 static struct GNUNET_CORE_Handle *core_handle;
 
+/**
+ * Handle to try to start new connections.
+ */
+static struct GNUNET_TRANSPORT_Handle *transport_handle;
 
 /******************************************************************************/
 /***************************** CORE CALLBACKS *********************************/
@@ -268,18 +278,19 @@ core_connect (void *cls, const struct GNUNET_PeerIdentity *peer)
 {
   struct MeshPeer *mp;
   struct MeshPeerPath *path;
+  char own_id[16];
 
-  LOG (GNUNET_ERROR_TYPE_DEBUG, "Peer connected\n");
-  LOG (GNUNET_ERROR_TYPE_DEBUG, "     %s\n", GNUNET_i2s (&my_full_id));
+  strncpy (own_id, GNUNET_i2s (&my_full_id), 15);
   mp = GMP_get (peer);
   if (myid == mp->id)
   {
-    LOG (GNUNET_ERROR_TYPE_DEBUG, "     (self)\n");
+    LOG (GNUNET_ERROR_TYPE_INFO, "CONNECTED %s (self)\n", own_id);
     path = path_new (1);
   }
   else
   {
-    LOG (GNUNET_ERROR_TYPE_DEBUG, "     %s\n", GNUNET_i2s (peer));
+    LOG (GNUNET_ERROR_TYPE_INFO, "CONNECTED %s <= %s\n",
+         own_id, GNUNET_i2s (peer));
     path = path_new (2);
     path->peers[1] = mp->id;
     GNUNET_PEER_change_rc (mp->id, 1);
@@ -290,6 +301,10 @@ core_connect (void *cls, const struct GNUNET_PeerIdentity *peer)
   GMP_add_path (mp, path, GNUNET_YES);
 
   mp->connections = GNUNET_CONTAINER_multihashmap_create (32, GNUNET_YES);
+
+  if (NULL != GMP_get_tunnel (mp))
+    GMP_connect (mp);
+
   return;
 }
 
@@ -305,8 +320,9 @@ core_disconnect (void *cls, const struct GNUNET_PeerIdentity *peer)
 {
   struct MeshPeer *p;
   struct MeshPeerPath *direct_path;
+  char own_id[16];
 
-  LOG (GNUNET_ERROR_TYPE_DEBUG, "Peer disconnected\n");
+  strncpy (own_id, GNUNET_i2s (&my_full_id), 15);
   p = GNUNET_CONTAINER_multipeermap_get (peers, peer);
   if (NULL == p)
   {
@@ -314,10 +330,10 @@ core_disconnect (void *cls, const struct GNUNET_PeerIdentity *peer)
     return;
   }
   if (myid == p->id)
-    LOG (GNUNET_ERROR_TYPE_DEBUG, "     (self: %s)\n", GMP_2s (p));
+    LOG (GNUNET_ERROR_TYPE_INFO, "DISCONNECTED %s (self)\n", own_id);
   else
-    LOG (GNUNET_ERROR_TYPE_DEBUG, "     %s\n", GMP_2s (p));
-
+    LOG (GNUNET_ERROR_TYPE_DEBUG, "DISCONNECTED %s <= %s\n",
+         own_id, GNUNET_i2s (peer));
   direct_path = pop_direct_path (p);
   GNUNET_CONTAINER_multihashmap_iterate (p->connections, &notify_broken, p);
   GNUNET_CONTAINER_multihashmap_destroy (p->connections);
@@ -338,16 +354,13 @@ core_disconnect (void *cls, const struct GNUNET_PeerIdentity *peer)
  * Functions to handle messages from core
  */
 static struct GNUNET_CORE_MessageHandler core_handlers[] = {
-  {&GMC_handle_create, GNUNET_MESSAGE_TYPE_MESH_CONNECTION_CREATE,
-    0},
+  {&GMC_handle_create, GNUNET_MESSAGE_TYPE_MESH_CONNECTION_CREATE, 0},
   {&GMC_handle_confirm, GNUNET_MESSAGE_TYPE_MESH_CONNECTION_ACK,
     sizeof (struct GNUNET_MESH_ConnectionACK)},
   {&GMC_handle_broken, GNUNET_MESSAGE_TYPE_MESH_CONNECTION_BROKEN,
     sizeof (struct GNUNET_MESH_ConnectionBroken)},
   {&GMC_handle_destroy, GNUNET_MESSAGE_TYPE_MESH_CONNECTION_DESTROY,
     sizeof (struct GNUNET_MESH_ConnectionDestroy)},
-  {&GMC_handle_keepalive, GNUNET_MESSAGE_TYPE_MESH_KEEPALIVE,
-    sizeof (struct GNUNET_MESH_ConnectionKeepAlive)},
   {&GMC_handle_ack, GNUNET_MESSAGE_TYPE_MESH_ACK,
     sizeof (struct GNUNET_MESH_ACK)},
   {&GMC_handle_poll, GNUNET_MESSAGE_TYPE_MESH_POLL,
@@ -495,9 +508,6 @@ send_core_connection_ack (struct MeshConnection *c, size_t size, void *buf)
   msg->header.size = htons (sizeof (struct GNUNET_MESH_ConnectionACK));
   msg->header.type = htons (GNUNET_MESSAGE_TYPE_MESH_CONNECTION_ACK);
   msg->cid = *GMC_get_id (c);
-  msg->reserved = 0;
-
-  /* TODO add signature */
 
   LOG (GNUNET_ERROR_TYPE_DEBUG, "CONNECTION ACK sent!\n");
   return sizeof (struct GNUNET_MESH_ConnectionACK);
@@ -508,6 +518,46 @@ send_core_connection_ack (struct MeshConnection *c, size_t size, void *buf)
 /********************************   STATIC  ***********************************/
 /******************************************************************************/
 
+
+/**
+ * Get priority for a queued message.
+ *
+ * @param q Queued message
+ *
+ * @return CORE priority to use.
+ */
+static enum GNUNET_CORE_Priority
+get_priority (struct MeshPeerQueue *q)
+{
+  enum GNUNET_CORE_Priority low;
+  enum GNUNET_CORE_Priority high;
+
+  if (NULL == q)
+  {
+    GNUNET_break (0);
+    return GNUNET_CORE_PRIO_BACKGROUND;
+  }
+
+  /* Relayed traffic has lower priority, our own traffic has higher */
+  if (NULL == q->c || GNUNET_NO == GMC_is_origin (q->c, q->fwd))
+  {
+    low = GNUNET_CORE_PRIO_BEST_EFFORT;
+    high = GNUNET_CORE_PRIO_URGENT;
+  }
+  else
+  {
+    low = GNUNET_CORE_PRIO_URGENT;
+    high = GNUNET_CORE_PRIO_CRITICAL_CONTROL;
+  }
+
+  /* Bulky payload has lower priority, control traffic has higher. */
+  if (GNUNET_MESSAGE_TYPE_MESH_ENCRYPTED == q->type)
+    return low;
+  else
+    return high;
+}
+
+
 /**
  * Iterator over tunnel hash map entries to destroy the tunnel during shutdown.
  *
@@ -531,7 +581,6 @@ shutdown_tunnel (void *cls,
 }
 
 
-
 /**
  * Destroy the peer_info and free any allocated resources linked to it
  *
@@ -721,7 +770,6 @@ queue_is_sendable (struct MeshPeerQueue *q)
     case GNUNET_MESSAGE_TYPE_MESH_CONNECTION_ACK:
     case GNUNET_MESSAGE_TYPE_MESH_CONNECTION_DESTROY:
     case GNUNET_MESSAGE_TYPE_MESH_CONNECTION_BROKEN:
-    case GNUNET_MESSAGE_TYPE_MESH_KEEPALIVE:
       return GNUNET_YES;
 
     case GNUNET_MESSAGE_TYPE_MESH_ENCRYPTED:
@@ -731,10 +779,7 @@ queue_is_sendable (struct MeshPeerQueue *q)
       GNUNET_break (0);
   }
 
-  if (GMC_is_sendable (q->c, q->fwd))
-    return GNUNET_YES;
-
-  return GNUNET_NO;
+  return GMC_is_sendable (q->c, q->fwd);
 }
 
 
@@ -743,7 +788,7 @@ queue_is_sendable (struct MeshPeerQueue *q)
  *
  * @param peer The destination peer.
  *
- * @return Best current known path towards the peer, if any.
+ * @return First transmittable message, if any. Otherwise, NULL.
  */
 static struct MeshPeerQueue *
 peer_get_first_message (const struct MeshPeer *peer)
@@ -752,6 +797,7 @@ peer_get_first_message (const struct MeshPeer *peer)
 
   for (q = peer->queue_head; NULL != q; q = q->next)
   {
+    LOG (GNUNET_ERROR_TYPE_DEBUG, "Checking %p towards %s\n", q, GMC_2s (q->c));
     if (queue_is_sendable (q))
       return q;
   }
@@ -792,6 +838,7 @@ search_handler (void *cls, const struct MeshPeerPath *path)
 }
 
 
+
 /**
  * Core callback to write a queued packet to core buffer
  *
@@ -824,7 +871,7 @@ queue_send (void *cls, size_t size, void *buf)
   queue = peer_get_first_message (peer);
   if (NULL == queue)
   {
-    GNUNET_break (0); /* Core tmt_rdy should've been canceled */
+    GNUNET_assert (0); /* Core tmt_rdy should've been canceled */
     return 0;
   }
   c = queue->c;
@@ -834,17 +881,17 @@ queue_send (void *cls, size_t size, void *buf)
   /* Check if buffer size is enough for the message */
   if (queue->size > size)
   {
-      LOG (GNUNET_ERROR_TYPE_DEBUG, "*   not enough room, reissue\n");
-      peer->core_transmit =
-          GNUNET_CORE_notify_transmit_ready (core_handle,
-                                             GNUNET_NO,
-                                             0,
-                                             GNUNET_TIME_UNIT_FOREVER_REL,
-                                             dst_id,
-                                             queue->size,
-                                             &queue_send,
-                                             peer);
-      return 0;
+    LOG (GNUNET_ERROR_TYPE_WARNING, "not enough room (%u vs %u), reissue\n",
+         queue->size, size);
+    peer->core_transmit =
+      GNUNET_CORE_notify_transmit_ready (core_handle,
+                                         GNUNET_NO, get_priority (queue),
+                                         GNUNET_TIME_UNIT_FOREVER_REL,
+                                         dst_id,
+                                         queue->size,
+                                         &queue_send,
+                                         peer);
+    return 0;
   }
   LOG (GNUNET_ERROR_TYPE_DEBUG, "*   size %u ok\n", queue->size);
 
@@ -853,7 +900,6 @@ queue_send (void *cls, size_t size, void *buf)
   {
     case GNUNET_MESSAGE_TYPE_MESH_CONNECTION_DESTROY:
     case GNUNET_MESSAGE_TYPE_MESH_CONNECTION_BROKEN:
-    case GNUNET_MESSAGE_TYPE_MESH_KEEPALIVE:
     case GNUNET_MESSAGE_TYPE_MESH_ENCRYPTED:
     case GNUNET_MESSAGE_TYPE_MESH_KX:
     case GNUNET_MESSAGE_TYPE_MESH_ACK:
@@ -892,14 +938,19 @@ queue_send (void *cls, size_t size, void *buf)
   if (0 < drop_percent &&
       GNUNET_CRYPTO_random_u32 (GNUNET_CRYPTO_QUALITY_WEAK, 101) < drop_percent)
   {
-    LOG (GNUNET_ERROR_TYPE_WARNING,
-                "Dropping message of type %s\n",
-                GM_m2s (queue->type));
+    LOG (GNUNET_ERROR_TYPE_WARNING, "DD %s on connection\n",
+         GM_m2s (queue->type), GMC_2s (c));
     data_size = 0;
   }
+  else
+  {
+    LOG (GNUNET_ERROR_TYPE_INFO,
+        "ss %s on connection %s (%p) %s (size %u)\n",
+        GM_m2s (queue->type), GMC_2s (c), c, GM_f2s (queue->fwd), data_size);
+  }
 
   /* Free queue, but cls was freed by send_core_* */
-  GMP_queue_destroy (queue, GNUNET_NO);
+  GMP_queue_destroy (queue, GNUNET_NO, GNUNET_YES);
 
   /* If more data in queue, send next */
   queue = peer_get_first_message (peer);
@@ -910,7 +961,7 @@ queue_send (void *cls, size_t size, void *buf)
     {
       peer->core_transmit =
           GNUNET_CORE_notify_transmit_ready (core_handle,
-                                             0, 0,
+                                             GNUNET_NO, get_priority (queue),
                                              GNUNET_TIME_UNIT_FOREVER_REL,
                                              dst_id,
                                              queue->size,
@@ -946,9 +997,10 @@ queue_send (void *cls, size_t size, void *buf)
  *
  * @param queue Queue handler to cancel.
  * @param clear_cls Is it necessary to free associated cls?
+ * @param sent Was it really sent? (Could have been canceled)
  */
 void
-GMP_queue_destroy (struct MeshPeerQueue *queue, int clear_cls)
+GMP_queue_destroy (struct MeshPeerQueue *queue, int clear_cls, int sent)
 {
   struct MeshPeer *peer;
 
@@ -956,8 +1008,8 @@ GMP_queue_destroy (struct MeshPeerQueue *queue, int clear_cls)
 
   if (GNUNET_YES == clear_cls)
   {
-    LOG (GNUNET_ERROR_TYPE_DEBUG, "#   queue destroy type %s\n",
-                GM_m2s (queue->type));
+    LOG (GNUNET_ERROR_TYPE_DEBUG, "queue destroy type %s\n",
+         GM_m2s (queue->type));
     switch (queue->type)
     {
       case GNUNET_MESSAGE_TYPE_MESH_CONNECTION_DESTROY:
@@ -976,8 +1028,8 @@ GMP_queue_destroy (struct MeshPeerQueue *queue, int clear_cls)
 
       default:
         GNUNET_break (0);
-        LOG (GNUNET_ERROR_TYPE_ERROR, "#   type %s unknown!\n",
-                    GM_m2s (queue->type));
+        LOG (GNUNET_ERROR_TYPE_ERROR, " type %s unknown!\n",
+             GM_m2s (queue->type));
     }
   }
   GNUNET_CONTAINER_DLL_remove (peer->queue_head, peer->queue_tail, queue);
@@ -990,13 +1042,19 @@ GMP_queue_destroy (struct MeshPeerQueue *queue, int clear_cls)
 
   if (NULL != queue->callback)
   {
-    LOG (GNUNET_ERROR_TYPE_DEBUG, "#   Calling callback\n");
+    LOG (GNUNET_ERROR_TYPE_DEBUG, " calling callback\n");
     queue->callback (queue->callback_cls,
-                     queue->c, queue->type,
+                     queue->c, sent, queue->type,
                      queue->fwd, queue->size,
                      GNUNET_TIME_absolute_get_duration (queue->start_waiting));
   }
 
+  if (NULL == peer_get_first_message (peer) && NULL != peer->core_transmit)
+  {
+    GNUNET_CORE_notify_transmit_ready_cancel (peer->core_transmit);
+    peer->core_transmit = NULL;
+  }
+
   GNUNET_free (queue);
 }
 
@@ -1026,10 +1084,8 @@ GMP_queue_add (struct MeshPeer *peer, void *cls, uint16_t type, size_t size,
   int priority;
   int call_core;
 
-  LOG (GNUNET_ERROR_TYPE_DEBUG,
-       "queue add %s %s towards %s (size %u) on c %p (%s)\n",
-       GM_f2s (fwd),  GM_m2s (type), GMP_2s(peer),
-       size, c, GMC_2s (c));
+  LOG (GNUNET_ERROR_TYPE_INFO, "qq %s on connection %s (%p) %s towards %s (size %u)\n",
+       GM_m2s (type), GMC_2s (c), c, GM_f2s (fwd), GMP_2s(peer), size);
 
   if (NULL == peer->connections)
   {
@@ -1078,8 +1134,7 @@ GMP_queue_add (struct MeshPeer *peer, void *cls, uint16_t type, size_t size,
                 GMP_2s (peer), size);
     peer->core_transmit =
         GNUNET_CORE_notify_transmit_ready (core_handle,
-                                           0,
-                                           0,
+                                           GNUNET_NO, get_priority (queue),
                                            GNUNET_TIME_UNIT_FOREVER_REL,
                                            GNUNET_PEER_resolve2 (peer->id),
                                            size,
@@ -1087,11 +1142,16 @@ GMP_queue_add (struct MeshPeer *peer, void *cls, uint16_t type, size_t size,
                                            peer);
     queue->start_waiting = GNUNET_TIME_absolute_get ();
   }
+  else if (GNUNET_NO == call_core)
+  {
+    LOG (GNUNET_ERROR_TYPE_DEBUG, "core tmt rdy towards %s not needed\n",
+         GMP_2s (peer));
+
+  }
   else
   {
-    LOG (GNUNET_ERROR_TYPE_DEBUG,
-                "core tmt rdy towards %s already called\n",
-                GMP_2s (peer));
+    LOG (GNUNET_ERROR_TYPE_DEBUG, "core tmt rdy towards %s already called\n",
+         GMP_2s (peer));
 
   }
   return queue;
@@ -1118,7 +1178,7 @@ GMP_queue_cancel (struct MeshPeer *peer, struct MeshConnection *c)
     if (q->c == c)
     {
       LOG (GNUNET_ERROR_TYPE_DEBUG, "GMP_cancel_queue %s\n", GM_m2s (q->type));
-      GMP_queue_destroy (q, GNUNET_YES);
+      GMP_queue_destroy (q, GNUNET_YES, GNUNET_NO);
 
       /* Get next from prev, q->next might be already freed:
        * queue destroy -> callback -> GMC_destroy -> cancel_queues -> here
@@ -1173,6 +1233,53 @@ connection_get_first_message (struct MeshPeer *peer, struct MeshConnection *c)
 }
 
 
+/**
+ * Get the first message for a connection and unqueue it.
+ *
+ * @param peer Neighboring peer.
+ * @param c Connection.
+ *
+ * @return First message for this connection.
+ */
+struct GNUNET_MessageHeader *
+GMP_connection_pop (struct MeshPeer *peer, struct MeshConnection *c)
+{
+  struct MeshPeerQueue *q;
+  struct MeshPeerQueue *next;
+  struct GNUNET_MessageHeader *msg;
+
+  LOG (GNUNET_ERROR_TYPE_DEBUG, "Connection pop on %s\n", GMC_2s (c));
+  for (q = peer->queue_head; NULL != q; q = next)
+  {
+    next = q->next;
+    if (q->c != c)
+      continue;
+    switch (q->type)
+    {
+      case GNUNET_MESSAGE_TYPE_MESH_CONNECTION_CREATE:
+      case GNUNET_MESSAGE_TYPE_MESH_CONNECTION_ACK:
+      case GNUNET_MESSAGE_TYPE_MESH_CONNECTION_DESTROY:
+      case GNUNET_MESSAGE_TYPE_MESH_CONNECTION_BROKEN:
+      case GNUNET_MESSAGE_TYPE_MESH_ACK:
+      case GNUNET_MESSAGE_TYPE_MESH_POLL:
+        GMP_queue_destroy (q, GNUNET_YES, GNUNET_NO);
+        continue;
+
+      case GNUNET_MESSAGE_TYPE_MESH_KX:
+      case GNUNET_MESSAGE_TYPE_MESH_ENCRYPTED:
+        msg = (struct GNUNET_MessageHeader *) q->cls;
+        GMP_queue_destroy (q, GNUNET_NO, GNUNET_NO);
+        return msg;
+
+      default:
+        GNUNET_break (0);
+    }
+  }
+
+  return NULL;
+}
+
+
 void
 GMP_queue_unlock (struct MeshPeer *peer, struct MeshConnection *c)
 {
@@ -1195,8 +1302,7 @@ GMP_queue_unlock (struct MeshPeer *peer, struct MeshConnection *c)
   size = q->size;
   peer->core_transmit =
       GNUNET_CORE_notify_transmit_ready (core_handle,
-                                         GNUNET_NO,
-                                         0,
+                                         GNUNET_NO, get_priority (q),
                                          GNUNET_TIME_UNIT_FOREVER_REL,
                                          GNUNET_PEER_resolve2 (peer->id),
                                          size,
@@ -1232,12 +1338,11 @@ GMP_init (const struct GNUNET_CONFIGURATION_Handle *c)
   }
   else
   {
-    LOG (GNUNET_ERROR_TYPE_WARNING,
-                "\n***************************************\n"
-                "Mesh is running with drop mode enabled.\n"
-                "This is NOT a good idea!\n"
-                "Remove the DROP_PERCENT option from your configuration.\n"
-                "***************************************\n");
+    LOG (GNUNET_ERROR_TYPE_WARNING, "**************************************\n");
+    LOG (GNUNET_ERROR_TYPE_WARNING, "Mesh is running with DROP enabled.\n");
+    LOG (GNUNET_ERROR_TYPE_WARNING, "This is NOT a good idea!\n");
+    LOG (GNUNET_ERROR_TYPE_WARNING, "Remove DROP_PERCENT from config file.\n");
+    LOG (GNUNET_ERROR_TYPE_WARNING, "**************************************\n");
   }
 
   core_handle = GNUNET_CORE_connect (c, /* Main configuration */
@@ -1250,12 +1355,30 @@ GMP_init (const struct GNUNET_CONFIGURATION_Handle *c)
                                      NULL,      /* Don't notify about all outbound messages */
                                      GNUNET_NO, /* For header-only out notification */
                                      core_handlers);    /* Register these handlers */
+  if (GNUNET_YES !=
+    GNUNET_CONFIGURATION_get_value_yesno (c, "MESH", "DISABLE_TRY_CONNECT"))
+  {
+    transport_handle = GNUNET_TRANSPORT_connect (c, &my_full_id, NULL, /* cls */
+                                                 /* Notify callbacks */
+                                                 NULL, NULL, NULL);
+  }
+  else
+  {
+    LOG (GNUNET_ERROR_TYPE_WARNING, "**************************************\n");
+    LOG (GNUNET_ERROR_TYPE_WARNING, "*  DISABLE TRYING CONNECT in config  *\n");
+    LOG (GNUNET_ERROR_TYPE_WARNING, "*  Use this only for test purposes.  *\n");
+    LOG (GNUNET_ERROR_TYPE_WARNING, "**************************************\n");
+  }
+
+
+
   if (NULL == core_handle)
   {
     GNUNET_break (0);
     GNUNET_SCHEDULER_shutdown ();
     return;
   }
+
 }
 
 /**
@@ -1271,6 +1394,11 @@ GMP_shutdown (void)
     GNUNET_CORE_disconnect (core_handle);
     core_handle = NULL;
   }
+  if (transport_handle != NULL)
+  {
+    GNUNET_TRANSPORT_disconnect (transport_handle);
+    transport_handle = NULL;
+  }
   GNUNET_PEER_change_rc (myid, -1);
 }
 
@@ -1320,6 +1448,25 @@ GMP_get_short (const GNUNET_PEER_Id peer)
 }
 
 
+/**
+ * Try to connect to a peer on transport level.
+ *
+ * @param cls Closure (peer).
+ * @param tc TaskContext.
+ */
+static void
+try_connect (void *cls, const struct GNUNET_SCHEDULER_TaskContext *tc)
+{
+  struct MeshPeer *peer = cls;
+
+  if (0 != (GNUNET_SCHEDULER_REASON_SHUTDOWN & tc->reason))
+    return;
+
+  GNUNET_TRANSPORT_try_connect (transport_handle,
+                                GNUNET_PEER_resolve2 (peer->id), NULL, NULL);
+}
+
+
 /**
  * Try to establish a new connection to this peer (in its tunnel).
  * If the peer doesn't have any path to it yet, try to get one.
@@ -1336,6 +1483,10 @@ GMP_connect (struct MeshPeer *peer)
   int rerun_search;
 
   LOG (GNUNET_ERROR_TYPE_DEBUG, "peer_connect towards %s\n", GMP_2s (peer));
+
+  /* If we have a current hello, try to connect using it. */
+  GMP_try_connect (peer);
+
   t = peer->tunnel;
   c = NULL;
   rerun_search = GNUNET_NO;
@@ -1346,7 +1497,12 @@ GMP_connect (struct MeshPeer *peer)
     p = peer_get_best_path (peer);
     if (NULL != p)
     {
-      LOG (GNUNET_ERROR_TYPE_DEBUG, "  %u hops\n", p->length);
+      char *s;
+
+      s = path_2s (p);
+      LOG (GNUNET_ERROR_TYPE_DEBUG, "  path to use: %s\n", s);
+      GNUNET_free (s);
+
       c = GMT_use_path (t, p);
       if (NULL == c)
       {
@@ -1402,19 +1558,6 @@ GMP_connect (struct MeshPeer *peer)
 }
 
 
-/**
- * Set tunnel.
- *
- * @param peer Peer.
- * @param t Tunnel.
- */
-void
-GMP_set_tunnel (struct MeshPeer *peer, struct MeshTunnel3 *t)
-{
-  peer->tunnel = t;
-}
-
-
 /**
  * Chech whether there is a direct (core level)  connection to peer.
  *
@@ -1490,7 +1633,7 @@ GMP_add_connection (struct MeshPeer *peer,
        "peer %s ok, has %u connections.\n",
        GMP_2s (peer), GNUNET_CONTAINER_multihashmap_size (peer->connections));
   result = GNUNET_CONTAINER_multihashmap_put (peer->connections,
-                                              GMC_get_id (c),
+                                              GMC_get_h (c),
                                               c,
                                               GNUNET_CONTAINER_MULTIHASHMAPOPTION_UNIQUE_FAST);
   LOG (GNUNET_ERROR_TYPE_DEBUG,
@@ -1537,17 +1680,12 @@ GMP_add_path (struct MeshPeer *peer, struct MeshPeerPath *path,
     path_destroy (path);
     return NULL;
   }
-  if (2 >= path->length && GNUNET_NO == trusted)
-  {
-    /* Only allow CORE to tell us about direct paths */
-    path_destroy (path);
-    return NULL;
-  }
+
   for (l = 1; l < path->length; l++)
   {
     if (path->peers[l] == myid)
     {
-      LOG (GNUNET_ERROR_TYPE_DEBUG, "shortening path by %u\n", l);
+      LOG (GNUNET_ERROR_TYPE_DEBUG, " shortening path by %u\n", l);
       for (l2 = 0; l2 < path->length - l; l2++)
       {
         path->peers[l2] = path->peers[l + l2];
@@ -1559,7 +1697,14 @@ GMP_add_path (struct MeshPeer *peer, struct MeshPeerPath *path,
     }
   }
 
-  LOG (GNUNET_ERROR_TYPE_DEBUG, "adding path [%u]\n", path->length);
+  LOG (GNUNET_ERROR_TYPE_DEBUG, " final length: %u\n", path->length);
+
+  if (2 >= path->length && GNUNET_NO == trusted)
+  {
+    /* Only allow CORE to tell us about direct paths */
+    path_destroy (path);
+    return NULL;
+  }
 
   l = path_get_length (path);
   if (0 == l)
@@ -1577,6 +1722,10 @@ GMP_add_path (struct MeshPeer *peer, struct MeshPeerPath *path,
       LOG (GNUNET_ERROR_TYPE_DEBUG, "  added\n");
       GNUNET_CONTAINER_DLL_insert_before (peer->path_head,
                                           peer->path_tail, aux, path);
+      if (NULL != peer->tunnel && 3 < GMT_count_connections (peer->tunnel))
+      {
+        GMP_connect (peer);
+      }
       return path;
     }
     else
@@ -1592,6 +1741,10 @@ GMP_add_path (struct MeshPeer *peer, struct MeshPeerPath *path,
   GNUNET_CONTAINER_DLL_insert_tail (peer->path_head, peer->path_tail,
                                     path);
   LOG (GNUNET_ERROR_TYPE_DEBUG, "  added last\n");
+  if (NULL != peer->tunnel && 3 < GMT_count_connections (peer->tunnel))
+  {
+    GMP_connect (peer);
+  }
   return path;
 }
 
@@ -1670,9 +1823,8 @@ GMP_remove_path (struct MeshPeer *peer, struct MeshPeerPath *path)
                      sizeof (GNUNET_PEER_Id) * path->length))
     {
       GNUNET_CONTAINER_DLL_remove (peer->path_head, peer->path_tail, iter);
-      path_destroy (iter);
-      if (path == iter)
-        return;
+      if (iter != path)
+        path_destroy (iter);
     }
   }
   path_destroy (path);
@@ -1706,7 +1858,7 @@ GMP_remove_connection (struct MeshPeer *peer,
        GMP_2s (peer), GNUNET_CONTAINER_multihashmap_size (peer->connections));
 
   return GNUNET_CONTAINER_multihashmap_remove (peer->connections,
-                                               GMC_get_id (c),
+                                               GMC_get_h (c),
                                                c);
 }
 
@@ -1740,7 +1892,6 @@ GMP_stop_search (struct MeshPeer *peer)
 {
   if (NULL == peer->search_h)
   {
-    GNUNET_break (0);
     return;
   }
 
@@ -1777,6 +1928,23 @@ GMP_get_short_id (const struct MeshPeer *peer)
 }
 
 
+/**
+ * Set tunnel.
+ *
+ * @param peer Peer.
+ * @param t Tunnel.
+ */
+void
+GMP_set_tunnel (struct MeshPeer *peer, struct MeshTunnel3 *t)
+{
+  peer->tunnel = t;
+  if (NULL == t && NULL != peer->search_h)
+  {
+    GMP_stop_search (peer);
+  }
+}
+
+
 /**
  * Get the tunnel towards a peer.
  *
@@ -1787,11 +1955,179 @@ GMP_get_short_id (const struct MeshPeer *peer)
 struct MeshTunnel3 *
 GMP_get_tunnel (const struct MeshPeer *peer)
 {
-  GNUNET_assert (NULL != peer->tunnel);
   return peer->tunnel;
 }
 
 
+/**
+ * Set the hello message.
+ *
+ * @param peer Peer whose message to set.
+ * @param hello Hello message.
+ */
+void
+GMP_set_hello (struct MeshPeer *peer, const struct GNUNET_HELLO_Message *hello)
+{
+  struct GNUNET_HELLO_Message *old;
+  size_t size;
+
+  LOG (GNUNET_ERROR_TYPE_DEBUG, "set hello for %s\n", GMP_2s (peer));
+  if (NULL == hello)
+    return;
+
+  old = GMP_get_hello (peer);
+  if (NULL == old)
+  {
+    size = GNUNET_HELLO_size (hello);
+    LOG (GNUNET_ERROR_TYPE_DEBUG, " new (%u bytes)\n", size);
+    peer->hello = GNUNET_malloc (size);
+    memcpy (peer->hello, hello, size);
+  }
+  else
+  {
+    peer->hello = GNUNET_HELLO_merge (old, hello);
+    LOG (GNUNET_ERROR_TYPE_DEBUG, " merge into %p (%u bytes)\n",
+         peer->hello, GNUNET_HELLO_size (hello));
+    GNUNET_free (old);
+  }
+}
+
+
+/**
+ * Get the hello message.
+ *
+ * @param peer Peer whose message to get.
+ *
+ * @return Hello message.
+ */
+struct GNUNET_HELLO_Message *
+GMP_get_hello (struct MeshPeer *peer)
+{
+  struct GNUNET_TIME_Absolute expiration;
+  struct GNUNET_TIME_Relative remaining;
+
+  if (NULL == peer->hello)
+    return NULL;
+
+  expiration = GNUNET_HELLO_get_last_expiration (peer->hello);
+  remaining = GNUNET_TIME_absolute_get_remaining (expiration);
+  if (0 == remaining.rel_value_us)
+  {
+    LOG (GNUNET_ERROR_TYPE_DEBUG, " get - hello expired on %s\n",
+         GNUNET_STRINGS_absolute_time_to_string (expiration));
+    GNUNET_free (peer->hello);
+    peer->hello = NULL;
+  }
+  return peer->hello;
+}
+
+
+/**
+ * Try to connect to a peer on TRANSPORT level.
+ *
+ * @param peer Peer to whom to connect.
+ */
+void
+GMP_try_connect (struct MeshPeer *peer)
+{
+  struct GNUNET_HELLO_Message *hello;
+  struct GNUNET_MessageHeader *mh;
+
+  if (NULL == transport_handle)
+    return;
+
+  hello = GMP_get_hello (peer);
+  if (NULL == hello)
+    return;
+
+  mh = GNUNET_HELLO_get_header (hello);
+  GNUNET_TRANSPORT_offer_hello (transport_handle, mh, try_connect, peer);
+}
+
+
+/**
+ * Notify a peer that a link between two other peers is broken. If any path
+ * used that link, eliminate it.
+ *
+ * @param peer Peer affected by the change.
+ * @param peer1 Peer whose link is broken.
+ * @param peer2 Peer whose link is broken.
+ */
+void
+GMP_notify_broken_link (struct MeshPeer *peer,
+                        struct GNUNET_PeerIdentity *peer1,
+                        struct GNUNET_PeerIdentity *peer2)
+{
+  struct MeshPeerPath *iter;
+  struct MeshPeerPath *next;
+  unsigned int i;
+  GNUNET_PEER_Id p1;
+  GNUNET_PEER_Id p2;
+
+  p1 = GNUNET_PEER_search (peer1);
+  p2 = GNUNET_PEER_search (peer2);
+
+  LOG (GNUNET_ERROR_TYPE_DEBUG, "Link %u-%u broken\n", p1, p2);
+  if (0 == p1 || 0 == p2)
+  {
+    /* We don't even know them */
+    return;
+  }
+
+  for (iter = peer->path_head; NULL != iter; iter = next)
+  {
+    next = iter->next;
+    for (i = 0; i < iter->length - 1; i++)
+    {
+      if ((iter->peers[i] == p1 && iter->peers[i + 1] == p2)
+          || (iter->peers[i] == p2 && iter->peers[i + 1] == p1))
+      {
+        char *s;
+
+        s = path_2s (iter);
+        LOG (GNUNET_ERROR_TYPE_DEBUG, " - invalidating %s\n", s);
+        GNUNET_free (s);
+
+        path_invalidate (iter);
+      }
+    }
+  }
+}
+
+
+/**
+ * Count the number of known paths toward the peer.
+ *
+ * @param peer Peer to get path info.
+ *
+ * @return Number of known paths.
+ */
+unsigned int
+GMP_count_paths (const struct MeshPeer *peer)
+{
+  struct MeshPeerPath *iter;
+  unsigned int i;
+
+  for (iter = peer->path_head, i = 0; NULL != iter; iter = iter->next)
+    i++;
+
+  return i;
+}
+
+
+/**
+ * Iterate all known peers.
+ *
+ * @param iter Iterator.
+ * @param cls Closure for @c iter.
+ */
+void
+GMP_iterate_all (GNUNET_CONTAINER_PeerMapIterator iter, void *cls)
+{
+  GNUNET_CONTAINER_multipeermap_iterate (peers, iter, cls);
+}
+
+
 /**
  * Get the static string for a peer ID.
  *
