@@ -26,6 +26,7 @@
 #include "platform.h"
 #include "mesh_test_lib.h"
 #include "gnunet_mesh_service.h"
+#include "gnunet_statistics_service.h"
 #include <gauger.h>
 
 
@@ -49,6 +50,7 @@
  */
 #define SETUP 0
 #define FORWARD 1
+#define KEEPALIVE 2
 #define SPEED 3
 #define SPEED_ACK 4
 #define SPEED_REL 8
@@ -74,19 +76,11 @@ static int test_backwards = GNUNET_NO;
  */
 static int ok;
 
- /**
-  * Each peer is supposed to generate the following callbacks:
-  * 1 incoming channel (@dest)
-  * 1 connected peer (@orig)
-  * 1 received data packet (@dest)
-  * 1 received data packet (@orig)
-  * 1 received channel destroy (@dest)
-  * _________________________________
-  * 5 x ok expected per peer
-  */
+/**
+ * Number of events expected to conclude the test successfully.
+ */
 int ok_goal;
 
-
 /**
  * Size of each test packet
  */
@@ -178,6 +172,11 @@ static struct GNUNET_MESH_Channel *incoming_ch;
  */
 static struct GNUNET_TIME_Absolute start_time;
 
+static struct GNUNET_TESTBED_Peer **testbed_peers;
+static struct GNUNET_TESTBED_Operation *stats_op;
+static unsigned int ka_sent;
+static unsigned int ka_received;
+
 
 /**
  * Show the results of the test (banwidth acheived) and log them to GAUGER
@@ -463,8 +462,7 @@ data_callback (void *cls, struct GNUNET_MESH_Channel *channel,
   if (client == expected_target_client) // Normally 4
   {
     data_received++;
-    GNUNET_log (GNUNET_ERROR_TYPE_INFO,
-                " received data %u\n", data_received);
+    GNUNET_log (GNUNET_ERROR_TYPE_INFO, " received data %u\n", data_received);
     if (SPEED != test || (ok_goal - 2) == ok)
     {
       GNUNET_MESH_notify_transmit_ready (channel, GNUNET_NO,
@@ -483,8 +481,7 @@ data_callback (void *cls, struct GNUNET_MESH_Channel *channel,
     if (test == SPEED_ACK || test == SPEED)
     {
       data_ack++;
-      GNUNET_log (GNUNET_ERROR_TYPE_INFO,
-              " received ack %u\n", data_ack);
+      GNUNET_log (GNUNET_ERROR_TYPE_INFO, " received ack %u\n", data_ack);
       GNUNET_MESH_notify_transmit_ready (channel, GNUNET_NO,
                                          GNUNET_TIME_UNIT_FOREVER_REL,
                                          size_payload, &tmt_rdy, (void *) 1L);
@@ -518,6 +515,87 @@ data_callback (void *cls, struct GNUNET_MESH_Channel *channel,
 }
 
 
+/**
+ * Stats callback. Finish the stats testbed operation and when all stats have
+ * been iterated, shutdown the test.
+ *
+ * @param cls closure
+ * @param op the operation that has been finished
+ * @param emsg error message in case the operation has failed; will be NULL if
+ *          operation has executed successfully.
+ */
+static void
+stats_cont (void *cls, struct GNUNET_TESTBED_Operation *op, const char *emsg)
+{
+  GNUNET_log (GNUNET_ERROR_TYPE_DEBUG, "stats_cont for peer %u\n", cls);
+  GNUNET_log (GNUNET_ERROR_TYPE_INFO, " sent: %u, received: %u\n",
+              ka_sent, ka_received);
+  if (ka_sent < 2 || ka_sent > ka_received + 1)
+    ok--;
+  GNUNET_TESTBED_operation_done (stats_op);
+
+  if (GNUNET_SCHEDULER_NO_TASK != disconnect_task)
+    GNUNET_SCHEDULER_cancel (disconnect_task);
+  disconnect_task = GNUNET_SCHEDULER_add_now (&disconnect_mesh_peers,
+                                              (void *) __LINE__);
+
+}
+
+
+/**
+ * Process statistic values.
+ *
+ * @param cls closure
+ * @param peer the peer the statistic belong to
+ * @param subsystem name of subsystem that created the statistic
+ * @param name the name of the datum
+ * @param value the current value
+ * @param is_persistent GNUNET_YES if the value is persistent, GNUNET_NO if not
+ * @return GNUNET_OK to continue, GNUNET_SYSERR to abort iteration
+ */
+static int
+stats_iterator (void *cls, const struct GNUNET_TESTBED_Peer *peer,
+                const char *subsystem, const char *name,
+                uint64_t value, int is_persistent)
+{
+  static const char *s_sent = "# keepalives sent";
+  static const char *s_recv = "# keepalives received";
+  uint32_t i;
+
+  i = GNUNET_TESTBED_get_index (peer);
+  GNUNET_log (GNUNET_ERROR_TYPE_DEBUG, "  %u - %s [%s]: %llu\n",
+              i, subsystem, name, value);
+  if (0 == strncmp (s_sent, name, strlen (s_sent)) && 0 == i)
+    ka_sent = value;
+
+  if (0 == strncmp(s_recv, name, strlen (s_recv)) && 4 == i)
+    ka_received = value;
+
+  return GNUNET_OK;
+}
+
+
+/**
+ * Task check that keepalives were sent and received.
+ *
+ * @param cls Closure (NULL).
+ * @param tc Task Context.
+ */
+static void
+check_keepalives (void *cls, const struct GNUNET_SCHEDULER_TaskContext *tc)
+{
+  if ((GNUNET_SCHEDULER_REASON_SHUTDOWN & tc->reason) != 0)
+    return;
+
+  disconnect_task = GNUNET_SCHEDULER_NO_TASK;
+  GNUNET_log (GNUNET_ERROR_TYPE_INFO, "check keepalives\n");
+  GNUNET_MESH_channel_destroy (ch);
+  stats_op = GNUNET_TESTBED_get_statistics (5, testbed_peers,
+                                            "mesh", NULL,
+                                            stats_iterator, stats_cont, NULL);
+}
+
+
 /**
  * Handlers, for diverse services
  */
@@ -560,9 +638,17 @@ incoming_channel (void *cls, struct GNUNET_MESH_Channel *channel,
   if (GNUNET_SCHEDULER_NO_TASK != disconnect_task)
   {
     GNUNET_SCHEDULER_cancel (disconnect_task);
-    disconnect_task = GNUNET_SCHEDULER_add_delayed (SHORT_TIME,
-                                                    &disconnect_mesh_peers,
-                                                    (void *) __LINE__);
+    if (KEEPALIVE == test)
+    {
+      struct GNUNET_TIME_Relative delay;
+      delay = GNUNET_TIME_relative_multiply (GNUNET_TIME_UNIT_SECONDS , 5);
+      disconnect_task =
+        GNUNET_SCHEDULER_add_delayed (delay, &check_keepalives, NULL);
+    }
+    else
+      disconnect_task = GNUNET_SCHEDULER_add_delayed (SHORT_TIME,
+                                                      &disconnect_mesh_peers,
+                                                      (void *) __LINE__);
   }
 
   return NULL;
@@ -584,8 +670,7 @@ channel_cleaner (void *cls, const struct GNUNET_MESH_Channel *channel,
   long i = (long) cls;
 
   GNUNET_log (GNUNET_ERROR_TYPE_INFO,
-              "Incoming channel disconnected at peer %d\n",
-              i);
+              "Incoming channel disconnected at peer %ld\n", i);
   if (4L == i)
   {
     ok++;
@@ -631,11 +716,11 @@ do_test (void *cls, const struct GNUNET_SCHEDULER_TaskContext *tc)
 {
   enum GNUNET_MESH_ChannelOption flags;
 
+  if ((GNUNET_SCHEDULER_REASON_SHUTDOWN & tc->reason) != 0)
+    return;
+
   GNUNET_log (GNUNET_ERROR_TYPE_DEBUG, "test_task\n");
-  GNUNET_log (GNUNET_ERROR_TYPE_DEBUG, "add peer 2\n");
 
-  GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
-              "schedule timeout in TIMEOUT\n");
   if (GNUNET_SCHEDULER_NO_TASK != disconnect_task)
   {
     GNUNET_SCHEDULER_cancel (disconnect_task);
@@ -652,6 +737,9 @@ do_test (void *cls, const struct GNUNET_SCHEDULER_TaskContext *tc)
   disconnect_task = GNUNET_SCHEDULER_add_delayed (SHORT_TIME,
                                                   &disconnect_mesh_peers,
                                                   (void *) __LINE__);
+  if (KEEPALIVE == test)
+    return; /* Don't send any data. */
+
   GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
               "Sending data initializer...\n");
   data_ack = 0;
@@ -717,6 +805,7 @@ tmain (void *cls,
   ok = 0;
   test_ctx = ctx;
   peers_running = num_peers;
+  testbed_peers = peers;
   h1 = meshes[0];
   h2 = meshes[num_peers - 1];
   disconnect_task = GNUNET_SCHEDULER_add_delayed (SHORT_TIME,
@@ -741,7 +830,7 @@ int
 main (int argc, char *argv[])
 {
   initialized = GNUNET_NO;
-  uint32_t ports[2];
+  static uint32_t ports[2];
   const char *config_file;
 
   GNUNET_log_setup ("test", "DEBUG", NULL);
@@ -764,31 +853,28 @@ main (int argc, char *argv[])
   }
   else if (strstr (argv[0], "_small_speed_ack") != NULL)
   {
-   /* Each peer is supposed to generate the following callbacks:
-    * 1 incoming channel (@dest)
-    * TOTAL_PACKETS received data packet (@dest)
-    * TOTAL_PACKETS received data packet (@orig)
-    * 1 received channel destroy (@dest)
-    * _________________________________
-    * 5 x ok expected per peer
-    */
+    /* Test is supposed to generate the following callbacks:
+     * 1 incoming channel (@dest)
+     * TOTAL_PACKETS received data packet (@dest)
+     * TOTAL_PACKETS received data packet (@orig)
+     * 1 received channel destroy (@dest)
+     */
+    ok_goal = TOTAL_PACKETS * 2 + 2;
     GNUNET_log (GNUNET_ERROR_TYPE_DEBUG, "SPEED_ACK\n");
     test = SPEED_ACK;
     test_name = "speed ack";
-    ok_goal = TOTAL_PACKETS * 2 + 2;
   }
   else if (strstr (argv[0], "_small_speed") != NULL)
   {
-   /* Each peer is supposed to generate the following callbacks:
-    * 1 incoming channel (@dest)
-    * 1 initial packet (@dest)
-    * TOTAL_PACKETS received data packet (@dest)
-    * 1 received data packet (@orig)
-    * 1 received channel destroy (@dest)
-    * _________________________________
-    */
-    GNUNET_log (GNUNET_ERROR_TYPE_DEBUG, "SPEED\n");
+    /* Test is supposed to generate the following callbacks:
+     * 1 incoming channel (@dest)
+     * 1 initial packet (@dest)
+     * TOTAL_PACKETS received data packet (@dest)
+     * 1 received data packet (@orig)
+     * 1 received channel destroy (@dest)
+     */
     ok_goal = TOTAL_PACKETS + 4;
+    GNUNET_log (GNUNET_ERROR_TYPE_DEBUG, "SPEED\n");
     if (strstr (argv[0], "_reliable") != NULL)
     {
       test = SPEED_REL;
@@ -801,6 +887,16 @@ main (int argc, char *argv[])
       test_name = "speed";
     }
   }
+  else if (strstr (argv[0], "_keepalive") != NULL)
+  {
+    test = KEEPALIVE;
+    /* Test is supposed to generate the following callbacks:
+     * 1 incoming channel (@dest)
+     * [wait]
+     * 1 received channel destroy (@dest)
+     */
+    ok_goal = 2;
+  }
   else
   {
     GNUNET_log (GNUNET_ERROR_TYPE_DEBUG, "UNKNOWN\n");
@@ -810,13 +906,9 @@ main (int argc, char *argv[])
 
   if (strstr (argv[0], "backwards") != NULL)
   {
-    char *aux;
-
     GNUNET_log (GNUNET_ERROR_TYPE_DEBUG, "BACKWARDS (LEAF TO ROOT)\n");
     test_backwards = GNUNET_YES;
-    aux = GNUNET_malloc (32);
-    sprintf (aux, "backwards %s", test_name);
-    test_name = aux;
+    GNUNET_asprintf (&test_name, "backwards %s", test_name);
   }
 
   p_ids = 0;
