@@ -1,6 +1,6 @@
 /*
      This file is part of GNUnet.
-     (C) 2009 Christian Grothoff (and other contributing authors)
+     (C) 2013 Christian Grothoff (and other contributing authors)
 
      GNUnet is free software; you can redistribute it and/or modify
      it under the terms of the GNU General Public License as published
@@ -19,10 +19,9 @@
 */
 /**
  * @file namestore/test_namestore_api_zone_iteration.c
- * @brief testcase for zone iteration functionality: iterate of a specific zone
+ * @brief testcase for zone iteration functionality: iterate all zones
  */
 #include "platform.h"
-#include "gnunet_common.h"
 #include "gnunet_namestore_service.h"
 #include "gnunet_testing_lib.h"
 #include "namestore.h"
@@ -35,19 +34,9 @@ static struct GNUNET_NAMESTORE_Handle * nsh;
 
 static GNUNET_SCHEDULER_TaskIdentifier endbadly_task;
 
-static GNUNET_SCHEDULER_TaskIdentifier stopiteration_task;
+static struct GNUNET_CRYPTO_EcdsaPrivateKey * privkey;
 
-static struct GNUNET_CRYPTO_RsaPrivateKey * privkey;
-
-static struct GNUNET_CRYPTO_RsaPublicKeyBinaryEncoded pubkey;
-
-static struct GNUNET_HashCode zone;
-
-static struct GNUNET_CRYPTO_RsaPrivateKey * privkey2;
-
-static struct GNUNET_CRYPTO_RsaPublicKeyBinaryEncoded pubkey2;
-
-static struct GNUNET_HashCode zone2;
+static struct GNUNET_CRYPTO_EcdsaPrivateKey * privkey2;
 
 static struct GNUNET_NAMESTORE_ZoneIterator *zi;
 
@@ -55,23 +44,19 @@ static int res;
 
 static int returned_records;
 
-static struct GNUNET_CRYPTO_RsaSignature *sig_1;
-
 static char * s_name_1;
 
-static struct GNUNET_NAMESTORE_RecordData *s_rd_1;
-
-static struct GNUNET_CRYPTO_RsaSignature *sig_2;
+static struct GNUNET_GNSRECORD_Data *s_rd_1;
 
 static char * s_name_2;
 
-static struct GNUNET_NAMESTORE_RecordData *s_rd_2;
-
-static struct GNUNET_CRYPTO_RsaSignature *sig_3;
+static struct GNUNET_GNSRECORD_Data *s_rd_2;
 
 static char * s_name_3;
 
-static struct GNUNET_NAMESTORE_RecordData *s_rd_3;
+static struct GNUNET_GNSRECORD_Data *s_rd_3;
+
+static char *directory;
 
 
 /**
@@ -83,19 +68,16 @@ static struct GNUNET_NAMESTORE_RecordData *s_rd_3;
 static void
 endbadly (void *cls, const struct GNUNET_SCHEDULER_TaskContext *tc)
 {
-  if (stopiteration_task != GNUNET_SCHEDULER_NO_TASK)
+  if (NULL != zi)
   {
-    GNUNET_SCHEDULER_cancel (stopiteration_task);
-    stopiteration_task = GNUNET_SCHEDULER_NO_TASK;
+    GNUNET_NAMESTORE_zone_iteration_stop (zi);
+    zi = NULL;
   }
-
   if (nsh != NULL)
+  {
     GNUNET_NAMESTORE_disconnect (nsh);
-  nsh = NULL;
-
-  GNUNET_free_non_null(sig_1);
-  GNUNET_free_non_null(sig_2);
-  GNUNET_free_non_null(sig_3);
+    nsh = NULL;
+  }
   GNUNET_free_non_null(s_name_1);
   GNUNET_free_non_null(s_name_2);
   GNUNET_free_non_null(s_name_3);
@@ -117,11 +99,11 @@ endbadly (void *cls, const struct GNUNET_SCHEDULER_TaskContext *tc)
   }
 
   if (privkey != NULL)
-    GNUNET_CRYPTO_rsa_key_free (privkey);
+    GNUNET_free (privkey);
   privkey = NULL;
 
   if (privkey2 != NULL)
-    GNUNET_CRYPTO_rsa_key_free (privkey2);
+    GNUNET_free (privkey2);
   privkey2 = NULL;
   res = 1;
 }
@@ -130,12 +112,11 @@ endbadly (void *cls, const struct GNUNET_SCHEDULER_TaskContext *tc)
 static void
 end (void *cls, const struct GNUNET_SCHEDULER_TaskContext *tc)
 {
-  if (stopiteration_task != GNUNET_SCHEDULER_NO_TASK)
+  if (NULL != zi)
   {
-    GNUNET_SCHEDULER_cancel (stopiteration_task);
-    stopiteration_task = GNUNET_SCHEDULER_NO_TASK;
+    GNUNET_NAMESTORE_zone_iteration_stop (zi);
+    zi = NULL;
   }
-
   if (endbadly_task != GNUNET_SCHEDULER_NO_TASK)
   {
     GNUNET_SCHEDULER_cancel (endbadly_task);
@@ -143,16 +124,13 @@ end (void *cls, const struct GNUNET_SCHEDULER_TaskContext *tc)
   }
 
   if (privkey != NULL)
-    GNUNET_CRYPTO_rsa_key_free (privkey);
+    GNUNET_free (privkey);
   privkey = NULL;
 
   if (privkey2 != NULL)
-    GNUNET_CRYPTO_rsa_key_free (privkey2);
+    GNUNET_free (privkey2);
   privkey2 = NULL;
 
-  GNUNET_free (sig_1);
-  GNUNET_free (sig_2);
-  GNUNET_free (sig_3);
   GNUNET_free (s_name_1);
   GNUNET_free (s_name_2);
   GNUNET_free (s_name_3);
@@ -179,44 +157,81 @@ end (void *cls, const struct GNUNET_SCHEDULER_TaskContext *tc)
 
 static void
 zone_proc (void *cls,
-	   const struct GNUNET_CRYPTO_RsaPublicKeyBinaryEncoded *zone_key,
-	   struct GNUNET_TIME_Absolute expire,
-	   const char *name,
-	   unsigned int rd_count,
-	   const struct GNUNET_NAMESTORE_RecordData *rd,
-	   const struct GNUNET_CRYPTO_RsaSignature *signature)
+           const struct GNUNET_CRYPTO_EcdsaPrivateKey *zone,
+           const char *label,
+           unsigned int rd_count,
+           const struct GNUNET_GNSRECORD_Data *rd)
 {
   int failed = GNUNET_NO;
-  if ((zone_key == NULL) && (name == NULL))
+
+  if ((zone == NULL) && (label == NULL))
   {
     GNUNET_break (3 == returned_records);
     if (3 == returned_records)
-      res = 0;
+    {
+      res = 0; /* Last iteraterator callback, we are done */
+      zi = NULL;
+    }
     else
       res = 1;
 
-    GNUNET_log (GNUNET_ERROR_TYPE_DEBUG, "Received last result, iteration done after receing %u results\n",returned_records );
+    GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
+    		"Received last result, iteration done after receing %u results\n",
+    		returned_records);
     GNUNET_SCHEDULER_add_now (&end, NULL);
+    return;
   }
-  else
+  if (0 == memcmp (zone, privkey, sizeof (struct GNUNET_CRYPTO_EcdsaPrivateKey)))
   {
-    /* verify signature returned from name store */
-    if (GNUNET_OK != GNUNET_NAMESTORE_verify_signature (zone_key, expire, name, rd_count, rd, signature))
+    if (0 == strcmp (label, s_name_1))
     {
-      struct GNUNET_HashCode zone_key_hash;
-      GNUNET_CRYPTO_hash (zone_key, sizeof (struct GNUNET_CRYPTO_RsaPublicKeyBinaryEncoded), &zone_key_hash);
-      GNUNET_log (GNUNET_ERROR_TYPE_DEBUG, "Verifying signature for `%s' in zone `%s' with %u records  and expiration %llu failed\n", name, GNUNET_h2s(&zone_key_hash), rd_count, expire.abs_value);
-
+      if (rd_count == 1)
+      {
+        if (GNUNET_YES != GNUNET_GNSRECORD_records_cmp(rd, s_rd_1))
+        {
+          failed = GNUNET_YES;
+          GNUNET_break (0);
+        }
+      }
+      else
+      {
+        failed = GNUNET_YES;
+        GNUNET_break (0);
+      }
+    }
+    else if (0 == strcmp (label, s_name_2))
+    {
+      if (rd_count == 1)
+      {
+        if (GNUNET_YES != GNUNET_GNSRECORD_records_cmp(rd, s_rd_2))
+        {
+          failed = GNUNET_YES;
+          GNUNET_break (0);
+        }
+      }
+      else
+      {
+        GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+                    "Received invalid record count\n");
+        failed = GNUNET_YES;
+        GNUNET_break (0);
+      }
+    }
+    else
+    {
+      GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
+                  "Comparing result failed: got name `%s' for first zone\n", label);
       failed = GNUNET_YES;
       GNUNET_break (0);
     }
-
-    GNUNET_log (GNUNET_ERROR_TYPE_DEBUG, "Comparing results name %s \n", name);
-    if (0 == strcmp (name, s_name_1))
+  }
+  else if (0 == memcmp (zone, privkey2, sizeof (struct GNUNET_CRYPTO_EcdsaPrivateKey)))
+  {
+    if (0 == strcmp (label, s_name_3))
     {
       if (rd_count == 1)
       {
-        if (GNUNET_YES != GNUNET_NAMESTORE_records_cmp(rd, s_rd_1))
+        if (GNUNET_YES != GNUNET_GNSRECORD_records_cmp(rd, s_rd_3))
         {
           failed = GNUNET_YES;
           GNUNET_break (0);
@@ -224,82 +239,39 @@ zone_proc (void *cls,
       }
       else
       {
-        failed = GNUNET_YES;
-        GNUNET_break (0);
-      }
-      if (0 != memcmp (signature, sig_1, sizeof (struct GNUNET_CRYPTO_RsaSignature)))
-      {
-        failed = GNUNET_YES;
-        GNUNET_break (0);
-      }
-    }
-    else if (0 == strcmp (name, s_name_2))
-    {
-      if (rd_count == 1)
-      {
-        if (GNUNET_YES != GNUNET_NAMESTORE_records_cmp(rd, s_rd_2))
-        {
-          failed = GNUNET_YES;
-          GNUNET_break (0);
-        }
-      }
-      else
-      {
-        failed = GNUNET_YES;
-        GNUNET_break (0);
-      }
-      if (0 != memcmp (signature, sig_2, sizeof (struct GNUNET_CRYPTO_RsaSignature)))
-      {
-        failed = GNUNET_YES;
-        GNUNET_break (0);
-      }
-    }
-    else if (0 == strcmp (name, s_name_3))
-    {
-      if (rd_count == 1)
-      {
-        if (GNUNET_YES != GNUNET_NAMESTORE_records_cmp(rd, s_rd_3))
-        {
-          failed = GNUNET_YES;
-          GNUNET_break (0);
-        }
-      }
-      else
-      {
-        failed = GNUNET_YES;
-        GNUNET_break (0);
-      }
-      if (GNUNET_OK != GNUNET_NAMESTORE_verify_signature(zone_key, expire, name, rd_count, rd, signature))
-      {
-        failed = GNUNET_YES;
-        GNUNET_break (0);
-      }
-
-      if (0 != memcmp (signature, sig_3, sizeof (struct GNUNET_CRYPTO_RsaSignature)))
-      {
+        GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+                    "Received invalid record count\n");
         failed = GNUNET_YES;
         GNUNET_break (0);
       }
     }
     else
     {
-      GNUNET_log (GNUNET_ERROR_TYPE_DEBUG, "Comparing result failed: got name `%s'\n", name);
-      res = 1;
+      GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
+                  "Comparing result failed: got name `%s' for first zone\n", label);
+      failed = GNUNET_YES;
       GNUNET_break (0);
-      GNUNET_SCHEDULER_add_now (&end, NULL);
     }
+  }
+  else
+  {
+    GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+                "Received invalid zone\n");
+    failed = GNUNET_YES;
+    GNUNET_break (0);
+  }
 
-    if (failed == GNUNET_NO)
-    {
-      returned_records ++;
-      GNUNET_log (GNUNET_ERROR_TYPE_DEBUG, "Telling namestore to send the next result\n");
-      GNUNET_NAMESTORE_zone_iterator_next (zi);
-    }
-    else
-    {
-      GNUNET_break (0);
-      GNUNET_SCHEDULER_add_now (&end, NULL);
-    }
+  if (failed == GNUNET_NO)
+  {
+    returned_records ++;
+    GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
+    		"Telling namestore to send the next result\n");
+    GNUNET_NAMESTORE_zone_iterator_next (zi);
+  }
+  else
+  {
+    GNUNET_break (0);
+    GNUNET_SCHEDULER_add_now (&end, NULL);
   }
 }
 
@@ -316,10 +288,13 @@ put_cont (void *cls, int32_t success, const char *emsg)
   }
   else
   {
-    GNUNET_log (GNUNET_ERROR_TYPE_ERROR, "Failed to created records\n");
+    GNUNET_log (GNUNET_ERROR_TYPE_ERROR, "Failed to created records: `%s'\n",
+    		emsg);
     GNUNET_break (0);
-    GNUNET_SCHEDULER_cancel (endbadly_task);
+    if (GNUNET_SCHEDULER_NO_TASK != endbadly_task)
+    	GNUNET_SCHEDULER_cancel (endbadly_task);
     endbadly_task = GNUNET_SCHEDULER_add_now (&endbadly, NULL);
+    return;
   }
 
   if (c == 3)
@@ -327,36 +302,37 @@ put_cont (void *cls, int32_t success, const char *emsg)
     res = 1;
     returned_records = 0;
     GNUNET_log (GNUNET_ERROR_TYPE_DEBUG, "All records created, starting iteration over all zones \n");
-    zi = GNUNET_NAMESTORE_zone_iteration_start(nsh,
-					       NULL,
-					       GNUNET_NAMESTORE_RF_NONE,
-					       GNUNET_NAMESTORE_RF_NONE,
-					       &zone_proc,
-					       &zone);
+    zi = GNUNET_NAMESTORE_zone_iteration_start (nsh,
+                                                NULL,
+                                                &zone_proc,
+                                                NULL);
     if (zi == NULL)
     {
       GNUNET_log (GNUNET_ERROR_TYPE_ERROR, "Failed to create zone iterator\n");
       GNUNET_break (0);
-      GNUNET_SCHEDULER_cancel (endbadly_task);
+      if (GNUNET_SCHEDULER_NO_TASK != endbadly_task)
+      	GNUNET_SCHEDULER_cancel (endbadly_task);
       endbadly_task = GNUNET_SCHEDULER_add_now (&endbadly, NULL);
+      return;
     }
   }
 }
 
 
-static struct GNUNET_NAMESTORE_RecordData *
+static struct GNUNET_GNSRECORD_Data *
 create_record (unsigned int count)
 {
   unsigned int c;
-  struct GNUNET_NAMESTORE_RecordData * rd;
+  struct GNUNET_GNSRECORD_Data * rd;
 
-  rd = GNUNET_malloc (count * sizeof (struct GNUNET_NAMESTORE_RecordData));
+  rd = GNUNET_malloc (count * sizeof (struct GNUNET_GNSRECORD_Data));
   for (c = 0; c < count; c++)
   {
-    rd[c].expiration_time = GNUNET_TIME_relative_to_absolute (GNUNET_TIME_UNIT_HOURS).abs_value;
+    rd[c].expiration_time = GNUNET_TIME_relative_to_absolute (GNUNET_TIME_UNIT_HOURS).abs_value_us;
     rd[c].record_type = 1111;
     rd[c].data_size = 50;
     rd[c].data = GNUNET_malloc(50);
+    rd[c].flags = 0;
     memset ((char *) rd[c].data, 'a', 50);
   }
   return rd;
@@ -370,82 +346,93 @@ create_record (unsigned int count)
  */
 static void
 empty_zone_proc (void *cls,
-		 const struct GNUNET_CRYPTO_RsaPublicKeyBinaryEncoded *zone_key,
-		 struct GNUNET_TIME_Absolute expire,
-		 const char *name,
+		 const struct GNUNET_CRYPTO_EcdsaPrivateKey *zone,
+		 const char *label,
 		 unsigned int rd_count,
-		 const struct GNUNET_NAMESTORE_RecordData *rd,
-		 const struct GNUNET_CRYPTO_RsaSignature *signature)
+		 const struct GNUNET_GNSRECORD_Data *rd)
 {
   char *hostkey_file;
-  struct GNUNET_TIME_Absolute et;
 
   GNUNET_assert (nsh == cls);
-  GNUNET_assert (NULL == name);
-  GNUNET_assert (NULL == zone_key);
+  if (NULL != zone)
+  {
+    GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+		_("Expected empty zone but received zone private key\n"));
+    GNUNET_break (0);
+    if (endbadly_task != GNUNET_SCHEDULER_NO_TASK)
+      GNUNET_SCHEDULER_cancel (endbadly_task);
+    endbadly_task = GNUNET_SCHEDULER_add_now (&endbadly, NULL);
+    return;
+  }
+  if ((NULL != label) || (NULL != rd) || (0 != rd_count))
+  {
+    GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+		_("Expected no zone content but received data\n"));
+    GNUNET_break (0);
+    if (endbadly_task != GNUNET_SCHEDULER_NO_TASK)
+      GNUNET_SCHEDULER_cancel (endbadly_task);
+    endbadly_task = GNUNET_SCHEDULER_add_now (&endbadly, NULL);
+    return;
+  }
+
+
   zi = NULL;
   GNUNET_asprintf(&hostkey_file,"zonefiles%s%s",DIR_SEPARATOR_STR,
       "N0UJMP015AFUNR2BTNM3FKPBLG38913BL8IDMCO2H0A1LIB81960.zkey");
   GNUNET_log (GNUNET_ERROR_TYPE_DEBUG, "Using zonekey file `%s' \n", hostkey_file);
-  privkey = GNUNET_CRYPTO_rsa_key_create_from_file(hostkey_file);
+  privkey = GNUNET_CRYPTO_ecdsa_key_create_from_file(hostkey_file);
   GNUNET_free (hostkey_file);
   GNUNET_assert (privkey != NULL);
-  GNUNET_CRYPTO_rsa_key_get_public(privkey, &pubkey);
-  GNUNET_CRYPTO_hash(&pubkey, sizeof (pubkey), &zone);
-
 
   GNUNET_asprintf(&hostkey_file,"zonefiles%s%s",DIR_SEPARATOR_STR,
       "HGU0A0VCU334DN7F2I9UIUMVQMM7JMSD142LIMNUGTTV9R0CF4EG.zkey");
   GNUNET_log (GNUNET_ERROR_TYPE_DEBUG, "Using zonekey file `%s' \n", hostkey_file);
-  privkey2 = GNUNET_CRYPTO_rsa_key_create_from_file(hostkey_file);
+  privkey2 = GNUNET_CRYPTO_ecdsa_key_create_from_file(hostkey_file);
   GNUNET_free (hostkey_file);
-
   GNUNET_assert (privkey2 != NULL);
-  GNUNET_CRYPTO_rsa_key_get_public(privkey2, &pubkey2);
-  GNUNET_CRYPTO_hash(&pubkey2, sizeof (pubkey), &zone2);
 
   GNUNET_log (GNUNET_ERROR_TYPE_DEBUG, "Created record 1\n");
 
   GNUNET_asprintf(&s_name_1, "dummy1");
   s_rd_1 = create_record(1);
-  et.abs_value = s_rd_1->expiration_time;
-  sig_1 = GNUNET_NAMESTORE_create_signature(privkey, et, s_name_1, s_rd_1, 1);
-  GNUNET_NAMESTORE_record_create(nsh, privkey, s_name_1, s_rd_1, &put_cont, NULL);
+  GNUNET_NAMESTORE_records_store (nsh, privkey, s_name_1,
+                                  1, s_rd_1,
+                                  &put_cont, NULL);
 
-
-  GNUNET_log (GNUNET_ERROR_TYPE_DEBUG, "Created record 2 \n");
+  GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
+              "Created record 2 \n");
   GNUNET_asprintf(&s_name_2, "dummy2");
   s_rd_2 = create_record(1);
+  GNUNET_NAMESTORE_records_store (nsh, privkey, s_name_2,
+                                  1, s_rd_2, &put_cont, NULL);
 
-  et.abs_value = s_rd_2->expiration_time;
-  sig_2 = GNUNET_NAMESTORE_create_signature(privkey, et, s_name_2, s_rd_2, 1);
-  GNUNET_NAMESTORE_record_create(nsh, privkey, s_name_2, s_rd_2, &put_cont, NULL);
+  GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
+              "Created record 3\n");
 
-  GNUNET_log (GNUNET_ERROR_TYPE_DEBUG, "Created record 3\n");
   /* name in different zone */
   GNUNET_asprintf(&s_name_3, "dummy3");
   s_rd_3 = create_record(1);
-  et.abs_value = s_rd_3->expiration_time;
-  sig_3 = GNUNET_NAMESTORE_create_signature(privkey2, et, s_name_3, s_rd_3, 1);
-  GNUNET_NAMESTORE_record_put (nsh, &pubkey2, s_name_3, GNUNET_TIME_UNIT_FOREVER_ABS, 1, s_rd_3, sig_3, &put_cont, NULL);
+  GNUNET_NAMESTORE_records_store (nsh, privkey2, s_name_3,
+                                  1, s_rd_3,
+                                  &put_cont, NULL);
 }
 
 
 static void
-run (void *cls, 
+run (void *cls,
      const struct GNUNET_CONFIGURATION_Handle *cfg,
      struct GNUNET_TESTING_Peer *peer)
 {
+  directory = NULL;
+  GNUNET_CONFIGURATION_get_value_string(cfg, "PATHS", "GNUNET_TEST_HOME", &directory);
+  GNUNET_DISK_directory_remove (directory);
+
   endbadly_task = GNUNET_SCHEDULER_add_delayed(TIMEOUT, &endbadly, NULL);
   nsh = GNUNET_NAMESTORE_connect (cfg);
   GNUNET_break (NULL != nsh);
   /* first, iterate over empty namestore */
   zi = GNUNET_NAMESTORE_zone_iteration_start(nsh,
-					     NULL,
-					     GNUNET_NAMESTORE_RF_NONE,
-					     GNUNET_NAMESTORE_RF_NONE,
-					     &empty_zone_proc,
-					     nsh);
+					     NULL, &empty_zone_proc, nsh);
   if (NULL == zi)
   {
     GNUNET_log (GNUNET_ERROR_TYPE_ERROR, "Failed to create zone iterator\n");
@@ -460,13 +447,19 @@ int
 main (int argc, char *argv[])
 {
   res = 1;
-  if (0 != 
-      GNUNET_TESTING_service_run ("test-namestore-api-zone-iteration",
-				  "namestore",
-				  "test_namestore_api.conf",
-				  &run,
-				  NULL))
-    return 1;
+  if (0 !=
+      GNUNET_TESTING_peer_run ("test-namestore-api-zone-iteration",
+                               "test_namestore_api.conf",
+                               &run,
+                               NULL))
+  {
+    res = 1;
+  }
+  if (NULL != directory)
+  {
+      GNUNET_DISK_directory_remove (directory);
+      GNUNET_free (directory);
+  }
   return res;
 }
 
