@@ -907,7 +907,7 @@ start_gnunet_nat_server (struct GNUNET_NAT_Handle *h)
     /* Start the server process */
     binary = GNUNET_OS_get_libexec_binary_path ("gnunet-helper-nat-server");
     h->server_proc =
-        GNUNET_OS_start_process (GNUNET_NO, 0, NULL, h->server_stdout,
+        GNUNET_OS_start_process (GNUNET_NO, 0, NULL, h->server_stdout, NULL,
                                  binary,
                                  "gnunet-helper-nat-server",
                                  h->internal_address, NULL);
@@ -1011,12 +1011,14 @@ resolve_dns (void *cls,
  *     the previous (now invalid) one
  * @param addr either the previous or the new public IP address
  * @param addrlen actual lenght of @a addr
+ * @param emsg NULL on success, otherwise an error message
  */
 static void
 upnp_add (void *cls,
           int add_remove,
           const struct sockaddr *addr,
-          socklen_t addrlen)
+          socklen_t addrlen,
+          const char *emsg)
 {
   struct GNUNET_NAT_Handle *h = cls;
   struct LocalAddressList *pos;
@@ -1443,7 +1445,7 @@ GNUNET_NAT_run_client (struct GNUNET_NAT_Handle *h,
        (unsigned int) h->adv_port);
   binary = GNUNET_OS_get_libexec_binary_path ("gnunet-helper-nat-client");
   proc =
-      GNUNET_OS_start_process (GNUNET_NO, 0, NULL, NULL,
+      GNUNET_OS_start_process (GNUNET_NO, 0, NULL, NULL, NULL,
                                binary,
                                "gnunet-helper-nat-client",
                                h->internal_address,
