@@ -615,6 +615,58 @@ shutdown_task (void *cls,
   }
 }
 
+/**
+ * Function called with the result of the check if the 'peerinfo'
+ * service is running.
+ *
+ * @param cls closure with our configuration
+ * @param result #GNUNET_YES if transport is running
+ */
+static void
+testservice_task (void *cls, int result)
+{
+  struct GNUNET_CRYPTO_EddsaPrivateKey *priv;
+  char *fn;
+
+  if (GNUNET_YES != result)
+  {
+    FPRINTF (stderr, _("Service `%s' is not running, please start GNUnet\n"), "peerinfo");
+    return;
+  }
+
+  if (NULL == (peerinfo = GNUNET_PEERINFO_connect (cfg)))
+  {
+    FPRINTF (stderr, "%s",  _("Could not access PEERINFO service.  Exiting.\n"));
+    return;
+  }
+  if ( (GNUNET_YES == get_self) || (GNUNET_YES == get_uri) || (NULL != dump_hello) )
+  {
+    /* load private key */
+    if (GNUNET_OK != GNUNET_CONFIGURATION_get_value_filename (cfg, "PEER",
+                                                              "PRIVATE_KEY",
+                                                               &fn))
+    {
+      GNUNET_log_config_missing (GNUNET_ERROR_TYPE_ERROR, "PEER", "PRIVATE_KEY");
+      return;
+    }
+    if (NULL == (priv = GNUNET_CRYPTO_eddsa_key_create_from_file (fn)))
+    {
+      FPRINTF (stderr, _("Loading hostkey from `%s' failed.\n"), fn);
+      GNUNET_free (fn);
+      return;
+    }
+    GNUNET_free (fn);
+    GNUNET_CRYPTO_eddsa_key_get_public (priv,
+                                        &my_peer_identity.public_key);
+    GNUNET_free (priv);
+  }
+
+  tt = GNUNET_SCHEDULER_add_now (&state_machine, NULL);
+  GNUNET_SCHEDULER_add_delayed (GNUNET_TIME_UNIT_FOREVER_REL,
+                                &shutdown_task,
+                                NULL);
+}
+
 
 /**
  * Main function that will be run by the scheduler.
@@ -628,9 +680,6 @@ static void
 run (void *cls, char *const *args, const char *cfgfile,
      const struct GNUNET_CONFIGURATION_Handle *c)
 {
-  struct GNUNET_CRYPTO_EddsaPrivateKey *priv;
-  char *fn;
-
   cfg = c;
   if ( (NULL != args[0]) &&
        (NULL == put_uri) &&
@@ -646,38 +695,9 @@ run (void *cls, char *const *args, const char *cfgfile,
 	     args[0]);
     return;
   }
-  if (NULL == (peerinfo = GNUNET_PEERINFO_connect (cfg)))
-  {
-    FPRINTF (stderr, "%s",  _("Could not access PEERINFO service.  Exiting.\n"));
-    return;
-  }
-  if ( (GNUNET_YES == get_self) || (GNUNET_YES == get_uri) || (NULL != dump_hello) )
-  {
-    /* load private key */
-    if (GNUNET_OK !=
-	GNUNET_CONFIGURATION_get_value_filename (cfg, "PEER", "PRIVATE_KEY",
-						 &fn))
-    {
-      FPRINTF (stderr, _("Could not find option `%s:%s' in configuration.\n"),
-	       "GNUNETD", "HOSTKEYFILE");
-      return;
-    }
-    if (NULL == (priv = GNUNET_CRYPTO_eddsa_key_create_from_file (fn)))
-    {
-      FPRINTF (stderr, _("Loading hostkey from `%s' failed.\n"), fn);
-      GNUNET_free (fn);
-      return;
-    }
-    GNUNET_free (fn);
-    GNUNET_CRYPTO_eddsa_key_get_public (priv,
-						    &my_peer_identity.public_key);
-    GNUNET_free (priv);
-  }
 
-  tt = GNUNET_SCHEDULER_add_now (&state_machine, NULL);
-  GNUNET_SCHEDULER_add_delayed (GNUNET_TIME_UNIT_FOREVER_REL,
-				&shutdown_task,
-				NULL);
+  GNUNET_CLIENT_service_test ("peerinfo", cfg, GNUNET_TIME_UNIT_SECONDS,
+      &testservice_task, (void *) cfg);
 }
 
 
@@ -780,9 +800,9 @@ main (int argc, char *const *argv)
     {'i', "info", NULL,
      gettext_noop ("list all known peers"),
      0, &GNUNET_GETOPT_set_one, &get_info},
-	  {'d', "dump-hello", NULL,
-		 gettext_noop ("dump hello to file"),
-		 1, &GNUNET_GETOPT_set_string, &dump_hello},
+    {'d', "dump-hello", NULL,
+     gettext_noop ("dump hello to file"),
+     1, &GNUNET_GETOPT_set_string, &dump_hello},
     {'g', "get-hello", NULL,
      gettext_noop ("also output HELLO uri(s)"),
      0, &GNUNET_GETOPT_set_one, &get_uri},
