@@ -224,6 +224,7 @@ listener_destroy (struct Listener *listener)
   {
     struct GNUNET_SERVER_Client *client = listener->client;
     listener->client = NULL;
+    GNUNET_log (GNUNET_ERROR_TYPE_DEBUG, "disconnecting listener client\n");
     GNUNET_SERVER_client_disconnect (client);
     return;
   }
@@ -452,6 +453,12 @@ incoming_destroy (struct Operation *incoming)
     GNUNET_SCHEDULER_cancel (incoming->state->timeout_task);
     incoming->state->timeout_task = GNUNET_SCHEDULER_NO_TASK;
   }
+  GNUNET_assert (NULL != incoming->state);
+  GNUNET_free (incoming->state);
+  // make sure that the tunnel end handler will not
+  // destroy us again
+  incoming->vt = NULL;
+  incoming->state = NULL;
   if (NULL != incoming->mq)
   {
     GNUNET_MQ_destroy (incoming->mq);
@@ -462,12 +469,6 @@ incoming_destroy (struct Operation *incoming)
     GNUNET_MESH_channel_destroy (incoming->channel);
     incoming->channel = NULL;
   }
-  GNUNET_assert (NULL != incoming->state);
-  GNUNET_free (incoming->state);
-  // make sure that the tunnel end handler will not
-  // destroy us again
-  incoming->vt = NULL;
-  incoming->state = NULL;
 }
 
 
@@ -728,8 +729,7 @@ handle_client_create_set (void *cls,
   switch (ntohs (msg->operation))
   {
   case GNUNET_SET_OPERATION_INTERSECTION:
-    // FIXME: implement intersection vt
-    // set->vt = _GSS_intersection_vt ();
+    set->vt = _GSS_intersection_vt ();
     break;
   case GNUNET_SET_OPERATION_UNION:
     set->vt = _GSS_union_vt ();
@@ -1054,24 +1054,30 @@ handle_client_cancel (void *cls,
     GNUNET_SERVER_client_disconnect (client);
     return;
   }
+
+  GNUNET_log (GNUNET_ERROR_TYPE_DEBUG, "client requested cancel for op %u\n", ntohl (msg->request_id));
+
   found = GNUNET_NO;
   for (op = set->ops_head; NULL != op; op = op->next)
   {
-    if (op->spec->client_request_id == msg->request_id)
+    if (op->spec->client_request_id == ntohl (msg->request_id))
     {
       found = GNUNET_YES;
       break;
     }
   }
 
-  if (GNUNET_NO == found)
-  {
-    GNUNET_break (0);
-    GNUNET_SERVER_client_disconnect (client);
-    return;
-  }
+  /* It may happen that the operation was destroyed due to
+   * the other peer disconnecting.  The client may not know about this
+   * yet and try to cancel the (non non-existent) operation.
+   */
+  if (GNUNET_NO != found)
+    _GSS_operation_destroy (op);
+  else
+    GNUNET_log (GNUNET_ERROR_TYPE_DEBUG, "client canceled non-existent op\n");
 
-  _GSS_operation_destroy (op);
+
+  GNUNET_SERVER_receive_done (client, GNUNET_OK);
 }
 
 
@@ -1094,21 +1100,6 @@ handle_client_accept (void *cls,
   struct Operation *op;
 
   msg = (const struct GNUNET_SET_AcceptRejectMessage *) mh;
-  op = get_incoming (ntohl (msg->accept_reject_id));
-
-  // incoming operation does not exist
-  if (NULL == op)
-  {
-    GNUNET_break (0);
-    GNUNET_SERVER_client_disconnect (client);
-    return;
-  }
-
-  GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
-              "client accepting %u\n",
-              ntohl (msg->accept_reject_id));
-
-  GNUNET_assert (GNUNET_YES == op->is_incoming);
 
   // client without a set requested an operation
   set = set_get (client);
@@ -1120,6 +1111,30 @@ handle_client_accept (void *cls,
     return;
   }
 
+  op = get_incoming (ntohl (msg->accept_reject_id));
+
+  /* it is not an error if the set op does not exist -- it may
+   * have been destroyed when the partner peer disconnected. */
+  if (NULL == op)
+  {
+    struct GNUNET_SET_ResultMessage *result_message;
+    struct GNUNET_MQ_Envelope *ev;
+    ev = GNUNET_MQ_msg (result_message, GNUNET_MESSAGE_TYPE_SET_RESULT);
+    result_message->request_id = msg->request_id;
+    result_message->element_type = 0;
+    result_message->result_status = htons (GNUNET_SET_STATUS_FAILURE);
+    GNUNET_MQ_send (set->client_mq, ev);
+    GNUNET_SERVER_receive_done (client, GNUNET_OK);
+    return;
+  }
+
+  GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
+              "client accepting %u\n",
+              ntohl (msg->accept_reject_id));
+
+  GNUNET_assert (GNUNET_YES == op->is_incoming);
+
+
   op->spec->set = set;
 
   incoming_retire (op);
