@@ -217,6 +217,11 @@ struct ValidationEntry
    */
   struct GNUNET_TIME_Absolute send_time;
 
+  /**
+   * At what time do we send the next validation request (PING)?
+   */
+  struct GNUNET_TIME_Absolute next_validation;
+
   /**
    * Until when is this address valid?
    * ZERO if it is not currently considered valid.
@@ -243,6 +248,10 @@ struct ValidationEntry
    */
   struct GNUNET_TIME_Relative latency;
 
+  /**
+   * Current state of this validation entry
+   */
+  enum GNUNET_TRANSPORT_ValidationState state;
   /**
    * Challenge number we used.
    */
@@ -264,10 +273,6 @@ struct ValidationEntry
    */
   int expecting_pong;
 
-  /* FIXME: DEBUGGING */
-  int last_line_set_to_no;
-  int last_line_set_to_yes;
-
   enum GNUNET_ATS_Network_Type network;
 };
 
@@ -340,6 +345,9 @@ static unsigned int validations_fast_start_threshold;
  */
 static struct GNUNET_TIME_Absolute validation_next;
 
+static GST_ValidationChangedCallback validation_entry_changed_cb;
+static void *validation_entry_changed_cb_cls;
+
 /**
  * Context for the validation entry match function.
  */
@@ -381,6 +389,24 @@ validation_entry_match (void *cls, const struct GNUNET_PeerIdentity * key, void
   return GNUNET_YES;
 }
 
+static void
+validation_entry_changed (struct ValidationEntry *ve, enum GNUNET_TRANSPORT_ValidationState state)
+{
+  char *t_sent = GNUNET_strdup(GNUNET_STRINGS_absolute_time_to_string(ve->send_time));
+  char *t_valid = GNUNET_strdup(GNUNET_STRINGS_absolute_time_to_string(ve->valid_until));
+  char *t_next = GNUNET_strdup(GNUNET_STRINGS_absolute_time_to_string(ve->next_validation));
+  GNUNET_log (GNUNET_ERROR_TYPE_DEBUG, "Validation entry changed for peer `%s' address `%s':\n\tSent:  %s\n\tValid: %s\n\tNext:  %s\n",
+      GNUNET_i2s(&ve->pid), GST_plugins_a2s(ve->address),
+      t_sent, t_valid, t_next);
+  ve->state = state;
+
+  GNUNET_free (t_sent);
+  GNUNET_free (t_valid);
+  GNUNET_free (t_next);
+  validation_entry_changed_cb (validation_entry_changed_cb_cls, &ve->pid,
+      ve->address, ve->send_time, ve->valid_until, ve->next_validation, state);
+}
+
 
 /**
  * Iterate over validation entries and free them.
@@ -395,6 +421,12 @@ cleanup_validation_entry (void *cls, const struct GNUNET_PeerIdentity * key, voi
 {
   struct ValidationEntry *ve = value;
 
+  ve->next_validation = GNUNET_TIME_absolute_get_zero_();
+  ve->valid_until = GNUNET_TIME_UNIT_ZERO_ABS;
+
+  /* Notify about deleted entry */
+  validation_entry_changed (ve, GNUNET_TRANSPORT_VS_REMOVE);
+
   if (NULL != ve->bc)
   {
     GST_blacklist_test_cancel (ve->bc);
@@ -487,7 +519,7 @@ transmit_ping_if_allowed (void *cls,
   if (GNUNET_NO == result)
   {
     GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
-                "Blacklist denies to send PING to `%s' %s %s\n",
+                "Blacklist denies to send PING to `%s' `%s' `%s'\n",
                 GNUNET_i2s (pid),
                 GST_plugins_a2s (ve->address),
                 ve->address->transport_name);
@@ -495,7 +527,7 @@ transmit_ping_if_allowed (void *cls,
   }
 
   GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
-              "Transmitting plain PING to `%s' %s %s\n",
+              "Transmitting plain PING to `%s' `%s' `%s'\n",
               GNUNET_i2s (pid),
               GST_plugins_a2s (ve->address),
               ve->address->transport_name);
@@ -557,11 +589,11 @@ transmit_ping_if_allowed (void *cls,
                           message_buf, tsize,
                           PING_PRIORITY, ACCEPTABLE_PING_DELAY,
                           NULL, NULL);
-        network = papi->get_network (ve->address, session);
+        network = papi->get_network (papi->cls, session);
         if (GNUNET_ATS_NET_UNSPECIFIED == network)
         {
           GNUNET_log(GNUNET_ERROR_TYPE_ERROR,
-              "Could not obtain a valid network for `%s' %s\n",
+              "Could not obtain a valid network for `%s' `%s'\n",
               GNUNET_i2s (pid), GST_plugins_a2s (ve->address));
           GNUNET_break(0);
         }
@@ -570,7 +602,7 @@ transmit_ping_if_allowed (void *cls,
       else
       {
         /* Could not get a valid session */
-        GNUNET_log (GNUNET_ERROR_TYPE_DEBUG, "Could not get a valid session for `%s' %s\n",
+        GNUNET_log (GNUNET_ERROR_TYPE_DEBUG, "Could not get a valid session for `%s' `%s'\n",
                     GNUNET_i2s (pid), GST_plugins_a2s (ve->address));
         ret = -1;
       }
@@ -590,6 +622,8 @@ transmit_ping_if_allowed (void *cls,
 	  GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
 	              "Validation started, %u validation processes running\n",
 	              validations_running);
+    /*  Notify about PING sent */
+    validation_entry_changed (ve, GNUNET_TRANSPORT_VS_UPDATE);
   }
 }
 
@@ -629,12 +663,13 @@ revalidate_address (void *cls,
   {
     /* should wait a bit longer */
     GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
-                "Waiting for %s longer before validating address %s\n",
+                "Waiting for %s longer before validating address `%s'\n",
                 GNUNET_STRINGS_relative_time_to_string (delay,
                                                         GNUNET_YES),
                 GST_plugins_a2s (ve->address));
     ve->revalidation_task =
         GNUNET_SCHEDULER_add_delayed (delay, &revalidate_address, ve);
+    ve->next_validation =  GNUNET_TIME_absolute_add(GNUNET_TIME_absolute_get(), delay);
     return;
   }
   blocked_for = GNUNET_TIME_absolute_get_remaining(validation_next);
@@ -643,12 +678,13 @@ revalidate_address (void *cls,
   {
     /* Validations are blocked, have to wait for blocked_for time */
     GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
-                "Validations blocked for another %s, delaying validating address %s\n",
+                "Validations blocked for another %s, delaying validating address `%s'\n",
                 GNUNET_STRINGS_relative_time_to_string (blocked_for,
                                                         GNUNET_YES),
                 GST_plugins_a2s (ve->address));
     ve->revalidation_task =
       GNUNET_SCHEDULER_add_delayed (blocked_for, &revalidate_address, ve);
+    ve->next_validation =  GNUNET_TIME_absolute_add(GNUNET_TIME_absolute_get(), blocked_for);
     return;
   }
   ve->revalidation_block = GNUNET_TIME_relative_to_absolute (canonical_delay);
@@ -673,12 +709,13 @@ revalidate_address (void *cls,
   }
   /* End debug code for mantis 0002726*/
   GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
-              "Validating now, next scheduled for %s, now validating address %s\n",
+              "Validating now, next scheduled for %s, now validating address `%s'\n",
               GNUNET_STRINGS_relative_time_to_string (blocked_for,
                                                       GNUNET_YES),
               GST_plugins_a2s (ve->address));
   ve->revalidation_task =
       GNUNET_SCHEDULER_add_delayed (delay, &revalidate_address, ve);
+  ve->next_validation =  GNUNET_TIME_absolute_add(GNUNET_TIME_absolute_get(), delay);
 
   /* start PINGing by checking blacklist */
   GNUNET_STATISTICS_update (GST_stats,
@@ -720,8 +757,6 @@ find_validation_entry (const struct GNUNET_CRYPTO_EddsaPublicKey *public_key,
     return NULL;
   ve = GNUNET_new (struct ValidationEntry);
   ve->in_use = GNUNET_SYSERR; /* not defined */
-  ve->last_line_set_to_no  = 0;
-  ve->last_line_set_to_yes  = 0;
   ve->address = GNUNET_HELLO_address_copy (address);
   ve->public_key = *public_key;
   ve->pid = address->peer;
@@ -736,6 +771,7 @@ find_validation_entry (const struct GNUNET_CRYPTO_EddsaPublicKey *public_key,
   GNUNET_CONTAINER_multipeermap_put (validation_map, &address->peer,
                                      ve,
                                      GNUNET_CONTAINER_MULTIHASHMAPOPTION_MULTIPLE);
+  validation_entry_changed (ve, GNUNET_TRANSPORT_VS_NEW);
   ve->expecting_pong = GNUNET_NO;
   return ve;
 }
@@ -783,10 +819,12 @@ add_valid_address (void *cls,
   if (GNUNET_SCHEDULER_NO_TASK == ve->revalidation_task)
   {
     GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
-                "Starting revalidations for valid address %s\n",
+                "Starting revalidations for valid address `%s'\n",
               GST_plugins_a2s (ve->address));
+    ve->next_validation = GNUNET_TIME_absolute_get();
     ve->revalidation_task = GNUNET_SCHEDULER_add_now (&revalidate_address, ve);
   }
+  validation_entry_changed (ve, GNUNET_TRANSPORT_VS_UPDATE);
 
   ats.type = htonl (GNUNET_ATS_NETWORK_TYPE);
   ats.value = htonl (ve->network);
@@ -813,7 +851,7 @@ process_peerinfo_hello (void *cls, const struct GNUNET_PeerIdentity *peer,
   if (NULL == hello)
     return;
   GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
-              "Handling HELLO for peer %s\n",
+              "Handling HELLO for peer `%s'\n",
               GNUNET_i2s (peer));
   GNUNET_assert (NULL ==
                  GNUNET_HELLO_iterate_addresses (hello, GNUNET_NO,
@@ -825,10 +863,12 @@ process_peerinfo_hello (void *cls, const struct GNUNET_PeerIdentity *peer,
 /**
  * Start the validation subsystem.
  *
+ * @param cb callback to call with changes to valdidation entries
+ * @param cb_cls cls for the callback
  * @param max_fds maximum number of fds to use
  */
 void
-GST_validation_start (unsigned int max_fds)
+GST_validation_start (GST_ValidationChangedCallback cb, void *cb_cls, unsigned int max_fds)
 {
   /**
    * Initialization for validation throttling
@@ -846,6 +886,8 @@ GST_validation_start (unsigned int max_fds)
   validation_delay.rel_value_us = (GNUNET_CONSTANTS_IDLE_CONNECTION_TIMEOUT.rel_value_us) / (max_fds / 2);
   validations_fast_start_threshold = (max_fds / 2);
   validations_running = 0;
+  validation_entry_changed_cb = cb;
+  validation_entry_changed_cb_cls = cb_cls;
   GNUNET_log (GNUNET_ERROR_TYPE_DEBUG, "Validation uses a fast start threshold of %u connections and a delay between of %s\n ",
               validations_fast_start_threshold,
               GNUNET_STRINGS_relative_time_to_string (validation_delay,
@@ -992,6 +1034,7 @@ GST_validation_handle_ping (const struct GNUNET_PeerIdentity *sender,
     addrend++;
     slen = strlen (addr) + 1;
     alen -= slen;
+    address.local_info = GNUNET_HELLO_ADDRESS_INFO_NONE;
     address.address = addrend;
     address.address_length = alen;
     address.transport_name = addr;
@@ -1222,8 +1265,8 @@ validate_address_iterator (void *cls,
   ve = find_validation_entry (&vac->public_key, address);
   if (GNUNET_SCHEDULER_NO_TASK == ve->revalidation_task)
   {
-    GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
-                "Starting validation for fresh address %s\n",
+    GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+                "Validation process started for fresh address `%s'\n",
                 GST_plugins_a2s (ve->address));
     ve->revalidation_task = GNUNET_SCHEDULER_add_now (&revalidate_address, ve);
   }
@@ -1304,6 +1347,7 @@ GST_validation_handle_pong (const struct GNUNET_PeerIdentity *sender,
   address.address = addr;
   address.address_length = addrlen;
   address.transport_name = tname;
+  address.local_info = GNUNET_HELLO_ADDRESS_INFO_NONE;
   ve = find_validation_entry (NULL, &address);
   if ((NULL == ve) || (GNUNET_NO == ve->expecting_pong))
   {
@@ -1358,7 +1402,7 @@ GST_validation_handle_pong (const struct GNUNET_PeerIdentity *sender,
     {
       GNUNET_break_op (0);
       GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
-                  "Failed to verify: invalid signature on address %s:%s from peer `%s'\n",
+                  "Failed to verify: invalid signature on address `%s':%s from peer `%s'\n",
                   tname,
                   GST_plugins_a2s (ve->address),
                   GNUNET_i2s (sender));
@@ -1370,8 +1414,8 @@ GST_validation_handle_pong (const struct GNUNET_PeerIdentity *sender,
     return GNUNET_SYSERR;
   }
 
-  GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
-              "Address validated for peer `%s' with plugin `%s': `%s'\n",
+  GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+              "Validation process successful for peer `%s' with plugin `%s' address `%s'\n",
               GNUNET_i2s (sender), tname, GST_plugins_a2s (ve->address));
   /* validity achieved, remember it! */
   ve->expecting_pong = GNUNET_NO;
@@ -1400,6 +1444,9 @@ GST_validation_handle_pong (const struct GNUNET_PeerIdentity *sender,
     GNUNET_break (0);
   }
 
+  /* Notify about new validity */
+  validation_entry_changed (ve, GNUNET_TRANSPORT_VS_UPDATE);
+
   /* build HELLO to store in PEERINFO */
   ve->copied = GNUNET_NO;
   hello = GNUNET_HELLO_create (&ve->public_key,
@@ -1442,6 +1489,11 @@ GST_validation_handle_hello (const struct GNUNET_MessageHeader *hello)
     return GNUNET_OK;
   /* Add peer identity without addresses to peerinfo service */
   h = GNUNET_HELLO_create (&vac.public_key, NULL, NULL, friend);
+  GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+              _("Validation received new %s message for peer `%s' with size %u\n"),
+              "HELLO",
+              GNUNET_i2s (&vac.pid),
+              ntohs (hello->size));
   GNUNET_PEERINFO_add_peer (GST_peerinfo, h, NULL, NULL);
 
   GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
@@ -1526,13 +1578,11 @@ GST_validation_get_addresses (const struct GNUNET_PeerIdentity *target,
  * @param session the session
  * @param in_use #GNUNET_YES if we are now using the address for a connection,
  *               #GNUNET_NO if we are no longer using the address for a connection
- * @param line line of caller just for DEBUGGING!
  */
 void
 GST_validation_set_address_use (const struct GNUNET_HELLO_Address *address,
                                 struct Session *session,
-                                int in_use,
-                                int line)
+                                int in_use)
 {
   struct ValidationEntry *ve;
 
@@ -1551,28 +1601,17 @@ GST_validation_set_address_use (const struct GNUNET_HELLO_Address *address,
     if (GNUNET_YES == in_use)
     {
       GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
-                  "Error setting address in use for peer `%s' `%s' to USED: set last time by %i, called now by %i\n",
-                  GNUNET_i2s (&address->peer), GST_plugins_a2s (address),
-                  ve->last_line_set_to_yes, line);
+                  "Error setting address in use for peer `%s' `%s' to USED\n",
+                  GNUNET_i2s (&address->peer), GST_plugins_a2s (address));
     }
     if (GNUNET_NO == in_use)
     {
       GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
-                  "Error setting address in use for peer `%s' `%s' to NOT_USED: set last time by %i, called now by %i\n",
-                  GNUNET_i2s (&address->peer), GST_plugins_a2s (address),
-                  ve->last_line_set_to_no, line);
+                  "Error setting address in use for peer `%s' `%s' to NOT_USED\n",
+                  GNUNET_i2s (&address->peer), GST_plugins_a2s (address));
     }
   }
 
-  if (GNUNET_YES == in_use)
-  {
-    ve->last_line_set_to_yes = line;
-  }
-  if (GNUNET_NO == in_use)
-  {
-    ve->last_line_set_to_no = line;
-  }
-
   GNUNET_break (ve->in_use != in_use);  /* should be different... */
   ve->in_use = in_use;
   if (in_use == GNUNET_YES)
@@ -1612,5 +1651,54 @@ GST_validation_get_address_latency (const struct GNUNET_PeerIdentity *sender,
   return ve->latency;
 }
 
+/**
+ * Closure for the validation_entries_iterate function.
+ */
+struct ValidationIteratorContext
+{
+  /**
+   * Function to call on each validation entry
+   */
+  GST_ValidationChangedCallback cb;
+
+  /**
+   * Closure for 'cb'.
+   */
+  void *cb_cls;
+};
+
+static int
+validation_entries_iterate (void *cls,
+                           const struct GNUNET_PeerIdentity *key,
+                           void *value)
+{
+  struct ValidationIteratorContext *ic = cls;
+  struct ValidationEntry *ve = value;
+
+  GNUNET_log (GNUNET_ERROR_TYPE_DEBUG, "Notifying about validation entry for peer `%s' address `%s' \n",
+      GNUNET_i2s (&ve->pid), GST_plugins_a2s (ve->address));
+  ic->cb (ic->cb_cls, &ve->pid, ve->address, ve->send_time,
+      ve->valid_until, ve->next_validation, ve->state);
+
+  return GNUNET_OK;
+}
+
+/**
+ * Iterate over all iteration entries
+ *
+ * @param cb function to call
+ * @param cb_cls closure for cb
+ */
+void
+GST_validation_iterate (GST_ValidationChangedCallback cb, void *cb_cls)
+{
+  struct ValidationIteratorContext ic;
+
+  if (NULL == validation_map)
+    return; /* can happen during shutdown */
+  ic.cb = cb;
+  ic.cb_cls = cb_cls;
+  GNUNET_CONTAINER_multipeermap_iterate (validation_map, &validation_entries_iterate, &ic);
+}
 
 /* end of file gnunet-service-transport_validation.c */
