@@ -198,6 +198,11 @@ struct Session
    */
   struct MacEndpoint *mac;
 
+  /**
+   * The address for this session
+   */
+  struct GNUNET_HELLO_Address *address;
+
   /**
    * Head of messages currently pending for transmission to this peer.
    */
@@ -606,7 +611,7 @@ send_ack (void *cls, uint32_t msg_id,
   get_wlan_header (endpoint->plugin,
 		   &radio_header->frame,
 		   &endpoint->addr.mac,
-		   size);
+		   sizeof (endpoint->addr.mac));
   memcpy (&radio_header[1], hdr, msize);
   if (NULL !=
       GNUNET_HELPER_send (endpoint->plugin->suid_helper,
@@ -656,7 +661,7 @@ free_session (struct Session *session)
   struct PendingMessage *pm;
 
   endpoint->plugin->env->session_end (endpoint->plugin->env->cls,
-				      &session->target,
+				      session->address,
 				      session);
   while (NULL != (pm = session->pending_message_head))
   {
@@ -680,6 +685,7 @@ free_session (struct Session *session)
   }
   GNUNET_STATISTICS_update (endpoint->plugin->env->stats, _("# Bluetooth sessions allocated"), -1,
                             GNUNET_NO);
+  GNUNET_HELLO_address_free (session->address);
   GNUNET_free (session);
 }
 
@@ -734,6 +740,8 @@ create_session (struct MacEndpoint *endpoint,
   GNUNET_CONTAINER_DLL_insert_tail (endpoint->sessions_head,
                                     endpoint->sessions_tail,
 				    session);
+  session->address = GNUNET_HELLO_address_allocate (peer, PLUGIN_NAME,
+     &endpoint->addr, sizeof (endpoint->addr), GNUNET_HELLO_ADDRESS_INFO_NONE);
   session->mac = endpoint;
   session->target = *peer;
   session->timeout = GNUNET_TIME_relative_to_absolute (GNUNET_CONSTANTS_IDLE_CONNECTION_TIMEOUT);
@@ -747,6 +755,29 @@ create_session (struct MacEndpoint *endpoint,
 }
 
 
+/**
+ * Lookup a new session
+ *
+ * @param endpoint pointer to the mac endpoint of the peer
+ * @param peer peer identity to use for this session
+ * @return returns the session or NULL
+ */
+static struct Session *
+lookup_session (struct MacEndpoint *endpoint,
+                const struct GNUNET_PeerIdentity *peer)
+{
+  struct Session *session;
+
+  for (session = endpoint->sessions_head; NULL != session; session = session->next)
+    if (0 == memcmp (peer, &session->target, sizeof (struct GNUNET_PeerIdentity)))
+    {
+      session->timeout = GNUNET_TIME_relative_to_absolute (GNUNET_CONSTANTS_IDLE_CONNECTION_TIMEOUT);
+      return session;
+    }
+  return NULL;
+}
+
+
 /**
  * Function called once we have successfully given the fragment
  * message to the SUID helper process and we are thus ready for
@@ -794,7 +825,7 @@ transmit_fragment (void *cls,
     get_wlan_header (endpoint->plugin,
 		     &radio_header->frame,
 		     &endpoint->addr.mac,
-		     size);
+		     sizeof (endpoint->addr.mac));
     memcpy (&radio_header[1], hdr, msize);
     GNUNET_assert (NULL == fm->sh);
     fm->sh = GNUNET_HELPER_send (endpoint->plugin->suid_helper,
@@ -994,7 +1025,7 @@ macendpoint_timeout (void *cls, const struct GNUNET_SCHEDULER_TaskContext *tc)
  */
 static struct MacEndpoint *
 create_macendpoint (struct Plugin *plugin,
-		    const struct WlanAddress *addr)
+		    struct WlanAddress *addr)
 {
   struct MacEndpoint *pos;
 
@@ -1002,7 +1033,7 @@ create_macendpoint (struct Plugin *plugin,
     if (0 == memcmp (addr, &pos->addr, sizeof (struct WlanAddress)))
       return pos;
   pos = GNUNET_new (struct MacEndpoint);
-  pos->addr = *addr;
+  pos->addr = (*addr);
   pos->plugin = plugin;
   pos->defrag =
     GNUNET_DEFRAGMENT_context_create (plugin->env->stats, WLAN_MTU,
@@ -1045,6 +1076,24 @@ bluetooth_get_network (void *cls,
 }
 
 
+/**
+ * Look up a session for a peer and create a new session if none is found
+ *
+ * @param endpoint pointer to the mac endpoint of the peer
+ * @param peer peer identity to use for this session
+ * @return returns the session
+ */
+static struct Session *
+get_session (struct MacEndpoint *endpoint,
+                const struct GNUNET_PeerIdentity *peer)
+{
+  struct Session *session;
+  if (NULL != (session = lookup_session (endpoint, peer)))
+        return session;
+  return create_session (endpoint, peer);
+}
+
+
 /**
  * Creates a new outbound session the transport service will use to send data to the
  * peer
@@ -1071,8 +1120,8 @@ bluetooth_plugin_get_session (void *cls,
        "Service asked to create session for peer `%s' with MAC `%s'\n",
        GNUNET_i2s (&address->peer),
        bluetooth_plugin_address_to_string(NULL, address->address, address->address_length));
-  endpoint = create_macendpoint (plugin, address->address);
-  return create_session (endpoint, &address->peer);
+  endpoint = create_macendpoint (plugin, (struct WlanAddress *) address->address);
+  return get_session (endpoint, &address->peer);
 }
 
 
@@ -1214,6 +1263,7 @@ static int
 process_data (void *cls, void *client, const struct GNUNET_MessageHeader *hdr)
 {
   struct Plugin *plugin = cls;
+  struct GNUNET_HELLO_Address *address;
   struct MacAndSession *mas = client;
   struct MacAndSession xmas;
   struct GNUNET_ATS_Information ats;
@@ -1240,27 +1290,31 @@ process_data (void *cls, void *client, const struct GNUNET_MessageHeader *hdr)
       GNUNET_break_op (0);
       break;
     }
+    if (NULL == mas->endpoint)
+    {
+      GNUNET_break (0);
+      break;
+    }
+
     LOG (GNUNET_ERROR_TYPE_DEBUG,
 	 "Processing %u bytes of HELLO from peer `%s' at MAC %s\n",
 	 (unsigned int) msize,
 	 GNUNET_i2s (&tmpsource),
-	 bluetooth_plugin_address_to_string (NULL, &mas->endpoint->addr, sizeof (struct WlanAddress)));
+	 bluetooth_plugin_address_to_string (NULL, &mas->endpoint->addr,
+	     sizeof (mas->endpoint->addr)));
 
     GNUNET_STATISTICS_update (plugin->env->stats,
 			      _("# HELLO messages received via Bluetooth"), 1,
 			      GNUNET_NO);
+
+    address = GNUNET_HELLO_address_allocate (&tmpsource, PLUGIN_NAME,
+        &mas->endpoint->addr, sizeof (mas->endpoint->addr),
+        GNUNET_HELLO_ADDRESS_INFO_INBOUND);
     plugin->env->receive (plugin->env->cls,
-			  &tmpsource,
-			  hdr,
-			  mas->session,
-			  (mas->endpoint == NULL) ? NULL : (const char *) &mas->endpoint->addr,
-			  (mas->endpoint == NULL) ? 0 : sizeof (struct GNUNET_TRANSPORT_WLAN_MacAddress));
+        address, mas->session, hdr);
     plugin->env->update_address_metrics (plugin->env->cls,
-					 &tmpsource,
-					 (mas->endpoint == NULL) ? NULL : (const char *) &mas->endpoint->addr,
-					 (mas->endpoint == NULL) ? 0 : sizeof (struct GNUNET_TRANSPORT_WLAN_MacAddress),
-					 mas->session,
-					 &ats, 1);
+        address, mas->session, &ats, 1);
+    GNUNET_HELLO_address_free (address);
     break;
   case GNUNET_MESSAGE_TYPE_FRAGMENT:
     if (NULL == mas->endpoint)
@@ -1271,7 +1325,8 @@ process_data (void *cls, void *client, const struct GNUNET_MessageHeader *hdr)
     LOG (GNUNET_ERROR_TYPE_DEBUG,
 	 "Processing %u bytes of FRAGMENT from MAC %s\n",
 	 (unsigned int) msize,
-	 bluetooth_plugin_address_to_string (NULL, &mas->endpoint->addr, sizeof (struct WlanAddress)));
+	 bluetooth_plugin_address_to_string (NULL, &mas->endpoint->addr,
+             sizeof (mas->endpoint->addr)));
     GNUNET_STATISTICS_update (plugin->env->stats,
                               _("# fragments received via Bluetooth"), 1, GNUNET_NO);
     (void) GNUNET_DEFRAGMENT_process_fragment (mas->endpoint->defrag,
@@ -1292,7 +1347,8 @@ process_data (void *cls, void *client, const struct GNUNET_MessageHeader *hdr)
       {
         LOG (GNUNET_ERROR_TYPE_DEBUG,
 	     "Got last ACK, finished message transmission to `%s' (%p)\n",
-	  	 bluetooth_plugin_address_to_string (NULL, &mas->endpoint->addr, sizeof (struct WlanAddress)),
+	     bluetooth_plugin_address_to_string (NULL, &mas->endpoint->addr,
+	                  sizeof (mas->endpoint->addr)),
 	     fm);
 	mas->endpoint->timeout = GNUNET_TIME_relative_to_absolute (MACENDPOINT_TIMEOUT);
 	if (NULL != fm->cont)
@@ -1307,13 +1363,15 @@ process_data (void *cls, void *client, const struct GNUNET_MessageHeader *hdr)
       {
         LOG (GNUNET_ERROR_TYPE_DEBUG,
 	     "Got an ACK, message transmission to `%s' not yet finished\n",
-	  	  bluetooth_plugin_address_to_string (NULL, &mas->endpoint->addr, sizeof (struct WlanAddress)));
+	     bluetooth_plugin_address_to_string (NULL, &mas->endpoint->addr,
+	                  sizeof (mas->endpoint->addr)));
         break;
       }
     }
     LOG (GNUNET_ERROR_TYPE_DEBUG,
 	 "ACK not matched against any active fragmentation with MAC `%s'\n",
-	 bluetooth_plugin_address_to_string (NULL, &mas->endpoint->addr, sizeof (struct WlanAddress)));
+	 bluetooth_plugin_address_to_string (NULL, &mas->endpoint->addr,
+	              sizeof (mas->endpoint->addr)));
     break;
   case GNUNET_MESSAGE_TYPE_WLAN_DATA:
     if (NULL == mas->endpoint)
@@ -1373,17 +1431,13 @@ process_data (void *cls, void *client, const struct GNUNET_MessageHeader *hdr)
 	 (unsigned int) ntohs (hdr->type),
 	 GNUNET_i2s (&mas->session->target));
     plugin->env->receive (plugin->env->cls,
-			  &mas->session->target,
-			  hdr,
-			  mas->session,
-			  (mas->endpoint == NULL) ? NULL : (const char *) &mas->endpoint->addr,
-			  (mas->endpoint == NULL) ? 0 : sizeof (struct WlanAddress));
+        mas->session->address,
+        mas->session,
+        hdr);
     plugin->env->update_address_metrics (plugin->env->cls,
-					 &mas->session->target,
-					 (mas->endpoint == NULL) ? NULL : (const char *) &mas->endpoint->addr,
-					 (mas->endpoint == NULL) ? 0 : sizeof (struct WlanAddress),
-					 mas->session,
-					 &ats, 1);
+        mas->session->address,
+        mas->session,
+        &ats, 1);
     break;
   }
   return GNUNET_OK;
@@ -1402,6 +1456,7 @@ handle_helper_message (void *cls, void *client,
 		       const struct GNUNET_MessageHeader *hdr)
 {
   struct Plugin *plugin = cls;
+  struct GNUNET_HELLO_Address *address;
   const struct GNUNET_TRANSPORT_WLAN_RadiotapReceiveMessage *rxinfo;
   const struct GNUNET_TRANSPORT_WLAN_HelperControlMessage *cm;
   struct WlanAddress wa;
@@ -1428,24 +1483,26 @@ handle_helper_message (void *cls, void *client,
       memset (&wa, 0, sizeof (struct WlanAddress));
       wa.mac = plugin->mac_address;
       wa.options = htonl(plugin->options);
-      plugin->env->notify_address (plugin->env->cls, GNUNET_NO,
-				   &wa,
-				   sizeof (wa),
-				   "bluetooth");
+      address = GNUNET_HELLO_address_allocate (plugin->env->my_identity,
+          PLUGIN_NAME, &wa, sizeof (wa), GNUNET_HELLO_ADDRESS_INFO_NONE);
+      plugin->env->notify_address (plugin->env->cls, GNUNET_NO, address);
+      GNUNET_HELLO_address_free (address);
     }
     plugin->mac_address = cm->mac;
     plugin->have_mac = GNUNET_YES;
     memset (&wa, 0, sizeof (struct WlanAddress));
     wa.mac = plugin->mac_address;
     wa.options = htonl(plugin->options);
+    address = GNUNET_HELLO_address_allocate (plugin->env->my_identity,
+        PLUGIN_NAME, &wa, sizeof (wa), GNUNET_HELLO_ADDRESS_INFO_NONE);
+
     LOG (GNUNET_ERROR_TYPE_DEBUG,
 	 "Received BT_HELPER_CONTROL message with MAC address `%s' for peer `%s'\n",
 	 mac_to_string (&cm->mac),
 	 GNUNET_i2s (plugin->env->my_identity));
-    plugin->env->notify_address (plugin->env->cls, GNUNET_YES,
-                                 &wa,
-                                 sizeof (struct WlanAddress),
-                                 "bluetooth");
+    plugin->env->notify_address (plugin->env->cls, GNUNET_YES, address);
+    GNUNET_HELLO_address_free (address);
+
     break;
   case GNUNET_MESSAGE_TYPE_WLAN_DATA_FROM_HELPER:
     LOG (GNUNET_ERROR_TYPE_DEBUG,
@@ -1682,7 +1739,8 @@ bluetooth_plugin_address_pretty_printer (void *cls, const char *type,
 void *
 libgnunet_plugin_transport_bluetooth_done (void *cls)
 {
-	struct WlanAddress wa;
+  struct WlanAddress wa;
+  struct GNUNET_HELLO_Address *address;
   struct GNUNET_TRANSPORT_PluginFunctions *api = cls;
   struct Plugin *plugin = api->cls;
   struct MacEndpoint *endpoint;
@@ -1696,14 +1754,17 @@ libgnunet_plugin_transport_bluetooth_done (void *cls)
 
   if (GNUNET_YES == plugin->have_mac)
   {
-  		memset (&wa, 0, sizeof (wa));
-  		wa.options = htonl (plugin->options);
-  		wa.mac = plugin->mac_address;
-      plugin->env->notify_address (plugin->env->cls, GNUNET_NO,
-                               &wa,
-                               sizeof (struct WlanAddress),
-                               "bluetooth");
-      plugin->have_mac = GNUNET_NO;
+    memset (&wa, 0, sizeof(wa));
+    wa.options = htonl (plugin->options);
+    wa.mac = plugin->mac_address;
+    address = GNUNET_HELLO_address_allocate (plugin->env->my_identity,
+        PLUGIN_NAME, &wa, sizeof (struct WlanAddress),
+        GNUNET_HELLO_ADDRESS_INFO_NONE);
+
+    plugin->env->notify_address (plugin->env->cls, GNUNET_NO, address);
+    plugin->have_mac = GNUNET_NO;
+
+    GNUNET_HELLO_address_free (address);
   }
 
   if (GNUNET_SCHEDULER_NO_TASK != plugin->beacon_task)
@@ -1882,7 +1943,7 @@ libgnunet_plugin_transport_bluetooth_init (void *cls)
                          0, GNUNET_NO);
   GNUNET_STATISTICS_set (plugin->env->stats, _("# Bluetooth MAC endpoints allocated"),
                          0, 0);
-  GNUNET_BANDWIDTH_tracker_init (&plugin->tracker,
+  GNUNET_BANDWIDTH_tracker_init (&plugin->tracker, NULL, NULL,
                                  GNUNET_BANDWIDTH_value_init (100 * 1024 *
                                                               1024 / 8), 100);
   plugin->fragment_data_tokenizer = GNUNET_SERVER_mst_create (&process_data, plugin);
