@@ -59,7 +59,7 @@
 #define HTTP_DEFAULT_PORT 80
 #define HTTPS_DEFAULT_PORT 443
 
-enum HTTP_OPTIONS
+enum HTTP_ADDRESS_OPTIONS
 {
   HTTP_OPTIONS_NONE = 0,
   HTTP_OPTIONS_VERIFY_CERTIFICATE = 1
